@@ -15,6 +15,7 @@ class CReport;
 class CommandInterface;
 
 #include "ItemData.h"
+#include "PWSfile.h"
 #include "StringX.h"
 #include "os/UUID.h"
 
@@ -36,31 +37,29 @@ class CommandInterface;
 class Command
 {
 public:
+  enum CommandDBChange { NONE = -1, MULTICOMMAND, DB, DBPREFS, DBHEADER, DBEMPTYGROUP, DBPOLICYNAMES,
+    DBFILTERS };
+
   virtual ~Command();
   virtual int Execute() = 0;
   virtual int Redo() {return Execute();} // common case
   virtual void Undo() = 0;
 
   void SetNoGUINotify() {m_bNotifyGUI = false;}
-  virtual void ResetSavedState(bool bNewDBState) // overrode in MultiCommands
-  {m_bSaveDBChanged = bNewDBState;}
   bool GetGUINotify() const {return m_bNotifyGUI;}
 
+  virtual bool WasDBChanged() const;
+  
 protected:
   Command(CommandInterface *pcomInt); // protected constructor!
-  void SaveState();
-  void RestoreState();
 
-protected:
   CommandInterface *m_pcomInt;
-  bool m_bSaveDBChanged;
-  bool m_bUniqueGTUValidated;
   bool m_bNotifyGUI;
-  int m_RC;
-  bool m_bState;
 
-  // Changed groups
-  std::vector<StringX> m_saved_vnodes_modified;
+  int m_RC;
+
+  // The command change value is ONLY set during Execute
+  CommandDBChange m_CommandDBChange;
 };
 
 // GUI related commands
@@ -94,22 +93,26 @@ public:
     GUI_UNDO_MERGESYNC,
     GUI_REFRESH_TREE,
     GUI_REFRESH_ENTRY,
+    GUI_REFRESH_BOTHVIEWS,
     GUI_DB_PREFERENCES_CHANGED,
     GUI_PWH_CHANGED_IN_DB
   };
 
+  // Note: entryUUID only used in GUI_REFRESH_ENTRY
   static UpdateGUICommand *Create(CommandInterface *pcomInt,
-                                  ExecuteFn When, GUI_Action ga)
-  { return new UpdateGUICommand(pcomInt, When, ga); }
+                                  ExecuteFn When, GUI_Action ga,
+                                  const pws_os::CUUID &entryUUID = pws_os::CUUID::NullUUID())
+  { return new UpdateGUICommand(pcomInt, When, ga, entryUUID); }
   int Execute();
   void Undo();
 
 private:
   UpdateGUICommand& operator=(const UpdateGUICommand&); // Do not implement
   UpdateGUICommand(CommandInterface *pcomInt, ExecuteFn When,
-                   GUI_Action ga);
+                   GUI_Action ga, const pws_os::CUUID &entryUUID);
   const ExecuteFn m_When;
   const GUI_Action m_ga;
+  pws_os::CUUID m_entryUUID;
 };
 
 // PWS related commands
@@ -127,7 +130,6 @@ private:
   DBPrefsCommand(CommandInterface *pcomInt, StringX &sxNewDBPrefs);
   StringX m_sxOldDBPrefs;
   StringX m_sxNewDBPrefs;
-  bool m_bOldState;
 };
 
 class DBPolicyNamesCommand : public Command
@@ -156,7 +158,7 @@ private:
   StringX m_sxPolicyName;
   PWPolicy m_st_ppp;
   Function m_function;
-  bool m_bOldState, m_bSingleAdd;
+  bool m_bSingleAdd;
 };
 
 class DBEmptyGroupsCommand : public Command
@@ -189,7 +191,7 @@ private:
   std::vector<StringX> m_vNewEmptyGroups;
   StringX m_sxEmptyGroup, m_sxOldGroup, m_sxNewGroup;
   Function m_function;
-  bool m_bOldState, m_bSingleGroup;
+  bool m_bSingleGroup;
 };
 
 class DeleteEntryCommand;
@@ -204,6 +206,7 @@ public:
   ~AddEntryCommand();
   int Execute();
   void Undo();
+
   friend class DeleteEntryCommand; // allow access to c'tor
 
 private:
@@ -226,6 +229,7 @@ public:
   ~DeleteEntryCommand();
   int Execute();
   void Undo();
+
   friend class AddEntryCommand; // allow access to c'tor
 
 private:
@@ -348,6 +352,7 @@ private:
   CReport *m_pRpt;
   CItemData::EntryType m_type;
   int m_iVia;
+
   // Alias/Shortcut structures
   // Permanent Multimap: since potentially more than one alias/shortcut per base
   //  Key = base uuid; Value = multiple alias/shortcut uuids
@@ -397,6 +402,9 @@ private:
   pws_os::CUUID m_from_baseuuid;
   pws_os::CUUID m_to_baseuuid;
   CItemData::EntryType m_type;
+
+  ItemMMap m_saved_base2aliases_mmap;
+  ItemMMap m_saved_base2shortcuts_mmap;
 };
 
 class UpdatePasswordHistoryCommand : public Command
@@ -434,6 +442,38 @@ private:
    StringX m_sxOldPath, m_sxNewPath;
 };
 
+class ChangeDBHeaderCommand : public Command {
+public:
+  static ChangeDBHeaderCommand *Create(CommandInterface *pcomInt,
+    const StringX sxNewValue, const PWSfile::HeaderType ht)
+  { return new ChangeDBHeaderCommand(pcomInt, sxNewValue, ht); }
+  int Execute();
+  void Undo();
+
+private:
+  ChangeDBHeaderCommand(CommandInterface *pcomInt,
+    StringX sxOldValue, const PWSfile::HeaderType ht);
+
+  StringX m_sxOldValue, m_sxNewValue;
+  PWSfile::HeaderType m_ht;
+};
+
+class DBFiltersCommand : public Command {
+public:
+  static DBFiltersCommand *Create(CommandInterface *pcomInt,
+    PWSFilters &MapFilters)
+  { return new DBFiltersCommand(pcomInt, MapFilters); }
+
+  int Execute();
+  void Undo();
+
+private:
+  DBFiltersCommand(CommandInterface *pcomInt, PWSFilters &MapFilters);
+
+  PWSFilters m_NewMapFilters;
+  PWSFilters m_OldMapFilters;
+};
+
 // Derived MultiCommands class
 class MultiCommands : public Command
 {
@@ -445,17 +485,17 @@ public:
   void Undo();
 
   void Add(Command *pcmd);
-  void Insert(Command *pcmd); // VERY INEFFICIENT - use sparingly
+  void Insert(Command *pcmd, size_t ioffset = 0); // VERY INEFFICIENT - use sparingly
   bool Remove(Command *pcmd);
   bool Remove();
   bool GetRC(Command *pcmd, int &rc);
   bool GetRC(const size_t ncmd, int &rc);
   std::size_t GetSize() const {return m_vpcmds.size();}
-  void ResetSavedState(bool bNewDBState);
+
+  std::vector<Command *> m_vpcmds;
 
  private:
   MultiCommands(CommandInterface *pcomInt);
-  std::vector<Command *> m_vpcmds;
   std::vector<int> m_vRCs;
 };
 
