@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2015 Rony Shapiro <ronys@users.sourceforge.net>.
+* Copyright (c) 2003-2016 Rony Shapiro <ronys@pwsafe.org>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -43,6 +43,7 @@
 #include <vector>
 #include <algorithm>
 #include <set>
+#include <type_traits> // for static_assert
 
 // These column names must match the field names defined in core_st.cpp
 const TCHAR *EXPORTHEADER  = _T("Group/Title\tUsername\tPassword\tURL\tAutoType\tCreated Time\tPassword Modified Time\tLast Access Time\tPassword Expiry Date\tPassword Expiry Interval\tRecord Modified Time\tPassword Policy\tPassword Policy Name\tHistory\tRun Command\tDCA\tShift+DCA\te-mail\tProtected\tSymbols\tKeyboard Shortcut\tNotes");
@@ -85,16 +86,16 @@ int PWScore::TestSelection(const bool bAdvanced,
                            const stringT &subgroup_name,
                            const int &subgroup_object,
                            const int &subgroup_function,
-                           const OrderedItemList *il)
+                           const OrderedItemList *pOIL)
 {
   // Check if any pass restricting criteria
   if (bAdvanced) {
     bool bAnyMatch(false);
-    if (il != NULL) {
-      if (find_if(il->begin(), il->end(),
+    if (pOIL != NULL) {
+      if (find_if(pOIL->begin(), pOIL->end(),
                   ExportTester(subgroup_name,
                                subgroup_object,
-                               subgroup_function)) != il->end())
+                               subgroup_function)) != pOIL->end())
         bAnyMatch = true;
     } else {
       if (find_if(m_pwlist.begin(), m_pwlist.end(),
@@ -107,8 +108,8 @@ int PWScore::TestSelection(const bool bAdvanced,
     if (!bAnyMatch)
       return FAILURE;
   } else {
-    if (il != NULL)
-      return il->empty() ? NO_ENTRIES_EXPORTED : SUCCESS;
+    if (pOIL != NULL)
+      return pOIL->empty() ? NO_ENTRIES_EXPORTED : SUCCESS;
     else
       return m_pwlist.empty() ? NO_ENTRIES_EXPORTED : SUCCESS;
   }
@@ -286,7 +287,7 @@ int PWScore::WritePlaintextFile(const StringX &filename,
                                 const int &subgroup_object,
                                 const int &subgroup_function,
                                 const TCHAR &delimiter, int &numExported,
-                                const OrderedItemList *il, CReport *pRpt)
+                                const OrderedItemList *pOIL, CReport *pRpt)
 {
   numExported = 0;
 
@@ -296,8 +297,8 @@ int PWScore::WritePlaintextFile(const StringX &filename,
 
   // Although the MFC UI prevents the user selecting export of an
   // empty database, other UIs might not, so:
-  if ((il != NULL && il->empty()) ||
-      (il == NULL && m_pwlist.empty()))
+  if ((pOIL != NULL && pOIL->empty()) ||
+      (pOIL == NULL && m_pwlist.empty()))
     return NO_ENTRIES_EXPORTED;
 
   FILE *txtfile = pws_os::FOpen(filename.c_str(), _T("wt"));
@@ -331,8 +332,8 @@ int PWScore::WritePlaintextFile(const StringX &filename,
   TextRecordWriter put_text(subgroup_name, subgroup_object, subgroup_function,
                    bsFields, delimiter, ofs, txtfile, numExported, pRpt, this);
 
-  if (il != NULL) {
-    for_each(il->begin(), il->end(), put_text);
+  if (pOIL != NULL) {
+    for_each(pOIL->begin(), pOIL->end(), put_text);
   } else {
     for_each(m_pwlist.begin(), m_pwlist.end(), put_text);
   }
@@ -444,9 +445,10 @@ int PWScore::WriteXMLFile(const StringX &filename,
                           const CItemData::FieldBits &bsFields,
                           const stringT &subgroup_name,
                           const int &subgroup_object, const int &subgroup_function,
-                          const TCHAR &delimiter, int &numExported,
+                          const TCHAR &delimiter, const stringT &exportgroup, int &numExported,
                           const OrderedItemList *il,
-                          const bool &bFilterActive, CReport *pRpt)
+                          const bool &bFilterActive,
+                          CReport *pRpt)
 {
   numExported = 0;
 
@@ -525,7 +527,7 @@ int PWScore::WriteXMLFile(const StringX &filename,
     ofs << "\"" << endl;
   }
 
-  CUUID huuid(*m_hdr.m_file_uuid.GetARep(), true); // true to print canoncally
+  CUUID huuid(*m_hdr.m_file_uuid.GetARep(), true); // true to print canonically
 
   ofs << "Database_uuid=\"" << huuid << "\"" << endl;
   ofs << "xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"" << endl;
@@ -548,7 +550,7 @@ int PWScore::WriteXMLFile(const StringX &filename,
   conv.ToUTF8(prefs.c_str(), utf8, utf8Len);
   ofs.write(reinterpret_cast<const char *>(utf8), utf8Len);
 
-  stringT pwpolicies = GetXMLPWPolicies();
+  stringT pwpolicies = GetXMLPWPolicies(il);
   if (!pwpolicies.empty()) {
     // Write out password policies stored in database
     LoadAString(cs_temp, IDSC_XMLEXP_POLICIES);
@@ -560,14 +562,27 @@ int PWScore::WriteXMLFile(const StringX &filename,
 
     conv.ToUTF8(pwpolicies.c_str(), utf8, utf8Len);
     ofs.write(reinterpret_cast<const char *>(utf8), utf8Len);
+    ofs << endl;
   }
 
   if (!m_vEmptyGroups.empty()) {
     // Write out empty groups stored in database
+    // However, if exporting a group, only export empty groups
+    // contained within it
+    std::vector<StringX> vsubemptygroups;
+    if (!exportgroup.empty()) {
+      for (size_t n = 0; n < m_vEmptyGroups.size(); n++) {
+        if (m_vEmptyGroups[n].substr(0, exportgroup.length()) == StringX(exportgroup.c_str()))
+          vsubemptygroups.push_back(m_vEmptyGroups[n]);
+      }
+    } else {
+      vsubemptygroups = m_vEmptyGroups;
+    }
+
     ostringstreamT os;
     os << "\t<EmptyGroups>" << endl;
-    for (size_t n = 0; n < m_vEmptyGroups.size(); n++) {
-      stringT sTemp = PWSUtil::GetSafeXMLString(m_vEmptyGroups[n]);
+    for (size_t n = 0; n < vsubemptygroups.size(); n++) {
+      stringT sTemp = PWSUtil::GetSafeXMLString(vsubemptygroups[n]);
       os << "\t\t<EGName>" << sTemp << "</EGName>" << endl;
     }
     os << "\t</EmptyGroups>" << endl << endl;
@@ -666,8 +681,8 @@ int PWScore::WriteXMLFile(const StringX &filename,
   if (bStartComment) {
     bStartComment = false;
     ofs << " --> " << endl;
+    ofs << endl;
   }
-  ofs << endl;
 
   // Write what we have and reset the buffer
   fwrite(ofs.str().c_str(), 1, ofs.str().length(), xmlfile);
@@ -705,7 +720,7 @@ int PWScore::ImportXMLFile(const stringT & /*ImportedPrefix*/,
                            int & /*numValidated*/, int & /*numImported*/, int & /*numSkipped*/,
                            int & /*numPWHErrors*/, int & /*numRenamed*/,
                            int & /*numNoPolicy*/,  int & /*numRenamedPolicies*/,
-                           int & /*numShortcutsRemoved*/,
+                           int & /*numShortcutsRemoved*/, int & /* numEmptyGroupsImported */,
                            CReport & /*rpt*/, Command *& /*pcommand*/)
 {
   return UNIMPLEMENTED;
@@ -718,7 +733,7 @@ int PWScore::ImportXMLFile(const stringT &ImportedPrefix, const stringT &strXMLF
                            int &numValidated, int &numImported, int &numSkipped,
                            int &numPWHErrors, int &numRenamed,
                            int &numNoPolicy, int &numRenamedPolicies,
-                           int &numShortcutsRemoved,
+                           int &numShortcutsRemoved, int &numEmptyGroupsImported,
                            CReport &rpt, Command *&pcommand)
 {
   UUIDVector Possible_Aliases, Possible_Shortcuts;
@@ -742,6 +757,7 @@ int PWScore::ImportXMLFile(const stringT &ImportedPrefix, const stringT &strXMLF
   if (!status) {
     return XML_FAILED_VALIDATION;
   }
+
   numValidated = iXML.getNumEntriesValidated();
 
   validation = false;
@@ -755,6 +771,7 @@ int PWScore::ImportXMLFile(const stringT &ImportedPrefix, const stringT &strXMLF
   numNoPolicy = iXML.getNumNoPolicies();
   numRenamedPolicies = iXML.getNumRenamedPolicies();
   numShortcutsRemoved = iXML.getNumShortcutsRemoved();
+  numEmptyGroupsImported = iXML.getNumEmptyGroupsImported();
 
   strXMLErrors = iXML.getXMLErrors();
   strSkippedList = iXML.getSkippedList();
@@ -767,9 +784,6 @@ int PWScore::ImportXMLFile(const stringT &ImportedPrefix, const stringT &strXMLF
     return XML_FAILED_IMPORT;
   }
 
-  if (numImported > 0)
-    SetDBChanged(true);
-
   return ((numRenamed + numPWHErrors) == 0) ? SUCCESS : OK_WITH_ERRORS;
 }
 #endif
@@ -901,7 +915,7 @@ int PWScore::ImportPlaintextFile(const StringX &ImportedPrefix,
 #define HDR_MAP_ENTRY(e) {e, CItemData::e},
 #define HDR_MAP_ENTRY2(e, ie) {e, CItemData::ie},
     
-        // These must be defined in the same order as Fields enum above, or it will assesrt below
+        // These must be defined in the same order as Fields enum above, or it will assert below
       
         HDR_MAP_ENTRY(GROUPTITLE) HDR_MAP_ENTRY(USER)               HDR_MAP_ENTRY(PASSWORD)  HDR_MAP_ENTRY(URL)
         HDR_MAP_ENTRY(AUTOTYPE)   HDR_MAP_ENTRY(CTIME)              HDR_MAP_ENTRY(PMTIME)    HDR_MAP_ENTRY(ATIME)
@@ -915,7 +929,7 @@ int PWScore::ImportPlaintextFile(const StringX &ImportedPrefix,
     };
     
     // make sure all elements are there
-    ASSERT( NumberOf(fieldMap) == NUMFIELDS );
+  static_assert((NumberOf(fieldMap) == NUMFIELDS), "Mismatch between fieldMap size and NUMFIELDS");
 
   to = 0;
   do {
@@ -1436,9 +1450,6 @@ int PWScore::ImportPlaintextFile(const StringX &ImportedPrefix,
                                             UpdateGUICommand::GUI_REDO_IMPORT);
   pmulticmds->Add(pcmd2);
 
-  if (numImported > 0)
-    SetDBChanged(true);
-
   return ((numSkipped + numRenamed + numPWHErrors + numshortcutsremoved)) == 0 ?
              SUCCESS : OK_WITH_ERRORS;
 }
@@ -1454,7 +1465,7 @@ int PWScore::ImportKeePassV1TXTFile(const StringX &filename,
   The checkbox "Encode/replace newline characters by '\n'" MUST be selected bu the
   user during the export or this import will fail and may give unexpected results.
 
-  The line that starts with '[' and ends with ']' is equivalant to the Title field.
+  The line that starts with '[' and ends with ']' is equivalent to the Title field.
 
   The following entries are supported:
      "Group: ", "Group Tree: ", "User Name: ", "URL: ", "Password: ",
@@ -1809,7 +1820,6 @@ int PWScore::ImportKeePassV1TXTFile(const StringX &filename,
     rpt.WriteLine(sx_imported.c_str());
   }
 
-  SetDBChanged(true);
   return SUCCESS;
 }
 
@@ -2295,13 +2305,10 @@ int PWScore::ImportKeePassV1CSVFile(const StringX &filename,
     rpt.WriteLine(sx_imported.c_str());
   } // file processing for (;;) loop
 
-  if (numImported > 0)
-    SetDBChanged(true);
-
   return ((numSkipped + numRenamed)) == 0 ? SUCCESS : OK_WITH_ERRORS;
 }
 
-stringT PWScore::GetXMLPWPolicies()
+stringT PWScore::GetXMLPWPolicies(const OrderedItemList *pOIL)
 {
   stringT retval(_T(""));
   ostringstreamT os;
@@ -2309,46 +2316,72 @@ stringT PWScore::GetXMLPWPolicies()
   if (m_MapPSWDPLC.empty())
     return retval;
 
-  os << _T("\t<NamedPasswordPolicies>") << endl;
-
+  std::vector<StringX> vPWPolicies;
+  const bool bSubset = pOIL->size() != GetNumEntries();
+  bool bNamedPasswordPolicies = !bSubset;
   PSWDPolicyMapCIter iter;
-  for (iter = m_MapPSWDPLC.begin(); iter != m_MapPSWDPLC.end(); iter++) {
-    os << "\t\t<Policy>" << endl;
-    stringT sTemp = PWSUtil::GetSafeXMLString(iter->first);
-    os << "\t\t\t<PWName>" << sTemp << "</PWName>" << endl;
 
-    os << "\t\t\t<PWDefaultLength>" << iter->second.length <<
-          "</PWDefaultLength>" << endl;
+  if (bSubset) {
+    // If not exporting the whole database, only get referenced Password Policies
+    PopulatePWPVector pwpv(&vPWPolicies);
+    for_each(pOIL->begin(), pOIL->end(), pwpv);
 
-    if (iter->second.flags & PWPolicy::UseLowercase)
-      os << "\t\t\t<PWUseLowercase>1</PWUseLowercase>" << endl;
-
-    if (iter->second.flags & PWPolicy::UseUppercase)
-      os << "\t\t\t<PWUseUppercase>1</PWUseUppercase>" << endl;
-
-    if (iter->second.flags & PWPolicy::UseDigits)
-      os << "\t\t\t<PWUseDigits>1</PWUseDigits>" << endl;
-
-    if (iter->second.flags & PWPolicy::UseSymbols)
-      os << "\t\t\t<PWUseSymbols>1</PWUseSymbols>" << endl;
-
-    if (iter->second.flags & PWPolicy::UseHexDigits)
-      os << "\t\t\t<PWUseHexDigits>1</PWUseHexDigits>" << endl;
-
-    if (iter->second.flags & PWPolicy::UseEasyVision)
-      os << "\t\t\t<PWUseEasyVision>1</PWUseEasyVision>" << endl;
-
-    if (iter->second.flags & PWPolicy::MakePronounceable)
-      os << "\t\t\t<PWMakePronounceable>1</PWMakePronounceable>" << endl;
-
-    if (!iter->second.symbols.empty()) {
-      sTemp = PWSUtil::GetSafeXMLString(iter->second.symbols);
-      os << "\t\t\t<symbols>" << sTemp << "</symbols>" << endl;
+    for (iter = m_MapPSWDPLC.begin(); iter != m_MapPSWDPLC.end(); iter++) {
+      // If OrderedList (i.e. not the whole database), only export a Password Policy
+      // if it is referenced by one of the entries being exported
+      if (std::find(vPWPolicies.begin(), vPWPolicies.end(), iter->first) != vPWPolicies.end()) {
+        bNamedPasswordPolicies = true;
+        break;
+      }
     }
-    os << "\t\t</Policy>" << endl;
   }
 
-  os << "\t</NamedPasswordPolicies>" << endl << endl;
-  retval = os.str().c_str();
+  if (bNamedPasswordPolicies) {
+    os << _T("\t<NamedPasswordPolicies>") << endl;
+
+    for (iter = m_MapPSWDPLC.begin(); iter != m_MapPSWDPLC.end(); iter++) {
+      // If OrderedList (i.e. not the whole database), only export a Password Policy
+      // if it is referenced by one of the entries being exported
+      if (bSubset && std::find(vPWPolicies.begin(), vPWPolicies.end(), iter->first) == vPWPolicies.end())
+        continue;
+
+      os << "\t\t<Policy>" << endl;
+      stringT sTemp = PWSUtil::GetSafeXMLString(iter->first);
+      os << "\t\t\t<PWName>" << sTemp << "</PWName>" << endl;
+
+      os << "\t\t\t<PWDefaultLength>" << iter->second.length <<
+        "</PWDefaultLength>" << endl;
+
+      if (iter->second.flags & PWPolicy::UseLowercase)
+        os << "\t\t\t<PWUseLowercase>1</PWUseLowercase>" << endl;
+
+      if (iter->second.flags & PWPolicy::UseUppercase)
+        os << "\t\t\t<PWUseUppercase>1</PWUseUppercase>" << endl;
+
+      if (iter->second.flags & PWPolicy::UseDigits)
+        os << "\t\t\t<PWUseDigits>1</PWUseDigits>" << endl;
+
+      if (iter->second.flags & PWPolicy::UseSymbols)
+        os << "\t\t\t<PWUseSymbols>1</PWUseSymbols>" << endl;
+
+      if (iter->second.flags & PWPolicy::UseHexDigits)
+        os << "\t\t\t<PWUseHexDigits>1</PWUseHexDigits>" << endl;
+
+      if (iter->second.flags & PWPolicy::UseEasyVision)
+        os << "\t\t\t<PWUseEasyVision>1</PWUseEasyVision>" << endl;
+
+      if (iter->second.flags & PWPolicy::MakePronounceable)
+        os << "\t\t\t<PWMakePronounceable>1</PWMakePronounceable>" << endl;
+
+      if (!iter->second.symbols.empty()) {
+        sTemp = PWSUtil::GetSafeXMLString(iter->second.symbols);
+        os << "\t\t\t<symbols>" << sTemp << "</symbols>" << endl;
+      }
+      os << "\t\t</Policy>" << endl;
+    }
+
+    os << "\t</NamedPasswordPolicies>" << endl;
+    retval = os.str().c_str();
+  }
   return retval;
 }
