@@ -84,7 +84,6 @@ PWSfileV4::CKeyBlocks PWSfileV4::CKeyBlocks::operator=(const PWSfileV4::CKeyBloc
   return *this;
 }
 
-
 PWSfileV4::PWSfileV4(const StringX &filename, RWmode mode, VERSION version)
   : PWSfile(filename, mode, version),
     m_effectiveFileLength(0), m_nHashIters(MIN_HASH_ITERATIONS)
@@ -252,7 +251,7 @@ size_t PWSfileV4::WriteCBC(unsigned char type, const unsigned char *data,
 
   m_hmac.Update(&type, 1);
   m_hmac.Update(buf, sizeof(buf));
-  m_hmac.Update(data, len32);
+  m_hmac.Update(data, length);
 
   return PWSfile::WriteCBC(type, data, length);
 }
@@ -274,7 +273,7 @@ int PWSfileV4::WriteRecord(const CItemAtt &att)
   // Following writes AttIV, AttEK, AttAK, AttContent
   // and AttContentHMAC per format spec.
   // All except the content are generated internally.
-int PWSfileV4::WriteContentFields(unsigned char *content, size_t len)
+size_t PWSfileV4::WriteContentFields(unsigned char *content, size_t len)
 {
   if (len == 0)
     return SUCCESS;
@@ -345,7 +344,7 @@ size_t PWSfileV4::ReadCBC(unsigned char &type, unsigned char* &data,
 
     m_hmac.Update(&type, 1);
     m_hmac.Update(buf, sizeof(buf));
-    m_hmac.Update(data, len32);
+    m_hmac.Update(data, length);
   }
 
   return numRead;
@@ -380,7 +379,7 @@ int PWSfileV4::ReadRecord(CItemData &item)
       RestoreState();
       status = WRONG_RECORD;
     }
-  } else if (unsigned(fpos) == m_effectiveFileLength)
+  } else if (fpos == m_effectiveFileLength)
     status = END_OF_FILE;
   else // fpos >= effectiveFileLength !?
     status = READ_FAIL;
@@ -493,7 +492,6 @@ void PWSfileV4::ComputeEndKB(const unsigned char hnonce[SHA256::HASHLEN],
   m_hmac.Final(digest);
 }
 
-
 #define SAFE_FWRITE(p, sz, cnt, stream) \
   { \
     size_t _ret = fwrite(p, sz, cnt, stream); \
@@ -643,17 +641,17 @@ int PWSfileV4::WriteHeader()
                         m_hdr.m_whatlastsaved);
   if (numWritten <= 0) { status = FAILURE; goto end; }
 
-  if (!m_hdr.m_dbname.empty()) {
-    numWritten = WriteCBC(HDR_DBNAME, m_hdr.m_dbname);
+  if (!m_hdr.m_DB_Name.empty()) {
+    numWritten = WriteCBC(HDR_DBNAME, m_hdr.m_DB_Name);
     if (numWritten <= 0) { status = FAILURE; goto end; }
   }
-  if (!m_hdr.m_dbdesc.empty()) {
-    numWritten = WriteCBC(HDR_DBDESC, m_hdr.m_dbdesc);
+  if (!m_hdr.m_DB_Description.empty()) {
+    numWritten = WriteCBC(HDR_DBDESC, m_hdr.m_DB_Description);
     if (numWritten <= 0) { status = FAILURE; goto end; }
   }
-  if (!m_MapFilters.empty()) {
+  if (!m_MapDBFilters.empty()) {
     coStringXStream oss;  // XML is always char not wchar_t
-    m_MapFilters.WriteFilterXMLFile(oss, m_hdr, _T(""));
+    m_MapDBFilters.WriteFilterXMLFile(oss, m_hdr, _T(""));
     numWritten = WriteCBC(HDR_FILTERS,
                           reinterpret_cast<const unsigned char *>(oss.str().c_str()),
                           oss.str().length());
@@ -665,7 +663,7 @@ int PWSfileV4::WriteHeader()
     if (num > 255)
       num = 255;  // Only save up to max as defined by FormatV3.
 
-    int buflen = (num * sizeof(uuid_array_t)) + 1;
+    size_t buflen = (num * sizeof(uuid_array_t)) + 1;
     unsigned char *buf = new unsigned char[buflen];
     buf[0] = (unsigned char)num;
     unsigned char *buf_ptr = buf + 1;
@@ -962,7 +960,7 @@ bool PWSfileV4::CKeyBlocks::RemoveKeyBlock(const StringX &passkey)
     return false;
 
   KeyBlockFinder find_kb(passkey);
-  const unsigned old_size = m_kbs.size();
+  const unsigned long old_size = m_kbs.size();
   m_kbs.erase(remove_if(m_kbs.begin(), m_kbs.end(), find_kb),
                m_kbs.end());
 
@@ -1086,13 +1084,13 @@ int PWSfileV4::ReadHeader()
     case HDR_DBNAME:
       if (utf8 != NULL) utf8[utf8Len] = '\0';
       utf8status = m_utf8conv.FromUTF8(utf8, utf8Len, text);
-      m_hdr.m_dbname = text;
+      m_hdr.m_DB_Name = text;
       break;
 
     case HDR_DBDESC:
       if (utf8 != NULL) utf8[utf8Len] = '\0';
       utf8status = m_utf8conv.FromUTF8(utf8, utf8Len, text);
-      m_hdr.m_dbdesc = text;
+      m_hdr.m_DB_Description = text;
       break;
 
 #if !defined(USE_XML_LIBRARY) || (!defined(_WIN32) && USE_XML_LIBRARY == MSXML)
@@ -1123,7 +1121,7 @@ int PWSfileV4::ReadHeader()
           m_UHFL.push_back(unkhfe);
           break;
         }
-        int rc = m_MapFilters.ImportFilterXMLFile(FPOOL_DATABASE, text.c_str(), _T(""),
+        int rc = m_MapDBFilters.ImportFilterXMLFile(FPOOL_DATABASE, text.c_str(), _T(""),
                                                   XSDFilename.c_str(),
                                                   strErrors, m_pAsker);
         if (rc != PWScore::SUCCESS) {
@@ -1259,6 +1257,9 @@ int PWSfileV4::ReadHeader()
     delete[] utf8; utf8 = NULL; utf8Len = 0;
   } while (fieldType != HDR_END);
 
+  // Now sort it for when we compare.
+  std::sort(m_vEmptyGroups.begin(), m_vEmptyGroups.end());
+
   return SUCCESS;
 }
 
