@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2015 Rony Shapiro <ronys@users.sourceforge.net>.
+* Copyright (c) 2003-2016 Rony Shapiro <ronys@pwsafe.org>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -71,7 +71,7 @@ const PWSprefs::boolPref PWSprefs::m_bool_prefs[NumBoolPrefs] = {
   {_T("AlwaysOnTop"), false, ptApplication},                // application
   {_T("ShowPWDefault"), false, ptDatabase},                 // database
   {_T("ShowPasswordInTree"), false, ptDatabase},            // database
-  {_T("SortAscending"), true, ptDatabase},                  // database
+  {_T("SortAscending"), true, ptObsolete},                  // obsolete in 3.40 - replaced by new app pref
   {_T("UseDefaultUser"), false, ptDatabase},                // database
   {_T("SaveImmediately"), true, ptDatabase},                // database
   {_T("PWUseLowercase"), true, ptDatabase},                 // database
@@ -128,7 +128,9 @@ const PWSprefs::boolPref PWSprefs::m_bool_prefs[NumBoolPrefs] = {
   {_T("UsePrimarySelectionForClipboard"), false, ptApplication}, //application
   {_T("CopyPasswordWhenBrowseToURL"), false, ptDatabase},   // database
   {_T("UseAltAutoType"), false, ptApplication},             //application
-  {_T("IgnoreHelpLoadError"), false, ptApplication},         //application
+  {_T("IgnoreHelpLoadError"), false, ptApplication},        //application
+  {_T("VKPlaySound"), false, ptApplication},                //application
+  {_T("ListSortAscending"), true, ptApplication},           //application
 };
 
 // Default value = -1 means set at runtime
@@ -167,6 +169,9 @@ const PWSprefs::intPref PWSprefs::m_int_prefs[NumIntPrefs] = {
   {_T("DefaultAutotypeDelay"), 10, ptApplication,
                             1, 60000},                              // application
   {_T("DlgOrientation"), AUTO, ptApplication, AUTO, WIDE},         // application
+  {_T("TimedTaskChainDelay"), 100, ptApplication, -1, -1},         // application
+  {_T("AutotypeSelectAllKeyCode"), 0, ptApplication, 0, 255},         // application
+  {_T("AutotypeSelectAllModMask"), 0, ptApplication, 0, 255},         // application
 };
 
 const PWSprefs::stringPref PWSprefs::m_string_prefs[NumStringPrefs] = {
@@ -192,6 +197,11 @@ const PWSprefs::stringPref PWSprefs::m_string_prefs[NumStringPrefs] = {
   {_T("AltNotesEditor"), _T(""), ptApplication},                    // application
   {_T("LanguageFile"), _T(""), ptApplication},                      // application
   {_T("DefaultSymbols"), _T(""), ptDatabase},                       // database
+  {_T("NotesFont"), _T(""), ptApplication},                         // application
+  {_T("NotesSampleText"), _T("AaBbYyZz 0O1IlL"), ptApplication},    // application
+  {_T("AutotypeTaskDelays"), _T("100,100,100"), ptApplication},     // application
+  {_T("AddEditFont"), _T(""), ptApplication },                      // application
+  {_T("AddEditSampleText"), _T("AaBbYyZz 0O1IlL"), ptApplication},  // application
 };
 
 PWSprefs *PWSprefs::GetInstance()
@@ -391,7 +401,7 @@ int PWSprefs::GetMRUList(stringT *MRUFiles) const
 
 int PWSprefs::SetMRUList(const stringT *MRUFiles, int n, int max_MRU)
 {
-  ASSERT(MRUFiles != NULL);
+  ASSERT(n == 0 || MRUFiles != NULL); // if n is zero, wx passes NULL
 
   if (m_ConfigOption == CF_NONE || m_ConfigOption == CF_REGISTRY ||
       m_ConfigOption == CF_FILE_RO)
@@ -1119,7 +1129,6 @@ void PWSprefs::InitializePreferences()
             m_configfilename.c_str(),
             isRO ? L"R/O" : L"R/W");
 
-
   // Does the registry entry exist for this user?
   m_bRegistryKeyExists = CheckRegistryExists();
 
@@ -1866,3 +1875,25 @@ void PWSprefs::ClearUnknownPrefs()
   m_vUnknownIPrefs.clear();
   m_vUnknownSPrefs.clear();
 }
+
+stringT PWSprefs::GetDCAdescription(int dca)
+{
+  const int ids[maxDCA+1] = {
+    IDSC_STATCOPYPASSWORD,    // DoubleClickCopyPassword = 0
+    IDSC_STATVIEWEDIT,        // DoubleClickViewEdit = 1
+    IDSC_STATAUTOTYPE,        // DoubleClickAutoType = 2
+    IDSC_STATBROWSE,          // DoubleClickBrowse = 3
+    IDSC_STATCOPYNOTES,       // DoubleClickCopyNotes = 4
+    IDSC_STATCOPYUSERNAME,    // DoubleClickCopyUsername = 5
+    IDSC_STATCOPYPASSWORDMIN, // DoubleClickCopyPasswordMinimize = 6
+    IDSC_STATBROWSEPLUS,      // DoubleClickBrowsePlus = 7
+    IDSC_STATRUN,             // DoubleClickRun = 8
+    IDSC_STATSENDEMAIL,       // DoubleClickSendEmail = 9
+  };
+
+  int id = (dca >= minDCA && dca <= maxDCA) ? ids[dca] : IDSC_STATCOMPANY;
+
+  stringT retval;
+  LoadAString(retval, id);
+  return retval;
+}
