@@ -66,108 +66,6 @@
 
 #endif  /* WINVER < 0x0501 */
 
-// Custom message event used for system tray handling
-#define PWS_MSG_ICON_NOTIFY             (WM_APP + 10)
-
-// To catch post Header drag
-#define PWS_MSG_HDR_DRAG_COMPLETE       (WM_APP + 20)
-#define PWS_MSG_CCTOHDR_DD_COMPLETE     (WM_APP + 21)
-#define PWS_MSG_HDRTOCC_DD_COMPLETE     (WM_APP + 22)
-
-// Process Compare Result Dialog click/menu functions
-#define PWS_MSG_COMPARE_RESULT_FUNCTION (WM_APP + 30)
-#define PWS_MSG_COMPARE_RESULT_ALLFNCTN (WM_APP + 31)
-
-// Equivalent one from Expired Password dialog
-#define PWS_MSG_EXPIRED_PASSWORD_EDIT   (WM_APP + 32)
-
-// Edit/Add extra context menu messages
-#define PWS_MSG_CALL_EXTERNAL_EDITOR    (WM_APP + 40)
-#define PWS_MSG_EXTERNAL_EDITOR_ENDED   (WM_APP + 41)
-#define PWS_MSG_EDIT_WORDWRAP           (WM_APP + 42)
-#define PWS_MSG_EDIT_SHOWNOTES          (WM_APP + 43)
-#define PWS_MSG_EDIT_APPLY              (WM_APP + 44)
-#define PWS_MSG_CALL_NOTESZOOMIN        (WM_APP + 45)
-#define PWS_MSG_CALL_NOTESZOOMOUT       (WM_APP + 46)
-
-// Simulate Ctrl+F from Find Toolbar "enter"
-#define PWS_MSG_TOOLBAR_FIND            (WM_APP + 50)
-
-// Perform Drag Autotype
-#define PWS_MSG_DRAGAUTOTYPE            (WM_APP + 55)
-
-// Update current filters whilst SetFilters dialog is open
-#define PWS_MSG_EXECUTE_FILTERS         (WM_APP + 60)
-
-// Notification from tree control that a file was dropped on it
-#define PWS_MSG_DROPPED_FILE            (WM_APP + 65)
-
-/* Message to get Virtual Keyboard buffer.  Here for doc. only. See VKeyBoardDlg.h
-#define PWS_MSG_INSERTBUFFER            (WM_APP + 70)
-#define PWS_MSG_RESETTIMER              (WM_APP + 71)
-*/
-
-
-/*
-  Timer related values (note - all documented her but some defined only where needed.
-*/
-
-/* Timer event number used to by PupText.  Here for doc. only
-#define TIMER_PUPTEXT             0x03 */
-// Timer event number used to check if the workstation is locked
-#define TIMER_LOCKONWTSLOCK       0x04
-// Timer event number used to support lock on user-defined idle timeout
-#define TIMER_LOCKDBONIDLETIMEOUT 0x05
-// Definition of a minute in milliseconds
-#define MINUTE 60000
-// How ofter should idle timeout timer check:
-#define IDLE_CHECK_RATE 2
-#define IDLE_CHECK_INTERVAL (MINUTE/IDLE_CHECK_RATE)
-// Timer event number used to support Find in PWListCtrl when icons visible
-#define TIMER_FIND                0x06
-// Timer event number used to support display of notes in List & Tree controls
-#define TIMER_ND_HOVER            0x07
-#define TIMER_ND_SHOWING          0x08
-// Timer event number used to support DragBar
-#define TIMER_DRAGBAR             0x09
-/* Timer event numbers used to by ControlExtns for ListBox tooltips.  Here for doc. only
-#define TIMER_LB_HOVER            0x0A
-#define TIMER_LB_SHOWING          0x0B 
-/* Timer event numbers used by StatusBar for tooltips.  Here for doc. only
-#define TIMER_SB_HOVER            0x0C
-#define TIMER_SB_SHOWING          0x0D */
-// Timer event for daily expired entries check
-#define TIMER_EXPENT              0x0E
-
-/*
-HOVER_TIME_ND       The length of time the pointer must remain stationary
-                    within a tool's bounding rectangle before the tool tip
-                    window appears.
-*/
-#define HOVER_TIME_ND      2000
-
-/*
-TIMEINT_ND_SHOWING The length of time the tool tip window remains visible
-                   if the pointer is stationary within a tool's bounding
-                   rectangle.
-*/
-#define TIMEINT_ND_SHOWING 5000
-
-// DragBar time interval 
-#define TIMER_DRAGBAR_TIME 100
-
-// Hotkey value ID to maximum value allowed by Windows for an app.
-#define PWS_HOTKEY_ID      0xBFFF
-
-// Arbitrary string to mean that the saved DB preferences are empty.
-#define EMPTYSAVEDDBPREFS L"#Empty#"
-
-// Maximum number of characters that can be added to a MFC CEdit control
-// by default (i.e. without calling SetLimitText). Note: 30000 not 32K!
-// Although this limit can be changed to up to 2GB of characters
-// (4GB memory if Unicode), it would make the database size absolutely enormous!
-#define MAXTEXTCHARS       30000
-
 // For ShutdownBlockReasonCreate & ShutdownBlockReasonDestroy
 typedef BOOL (WINAPI *PSBR_CREATE) (HWND, LPCWSTR);
 typedef BOOL (WINAPI *PSBR_DESTROY) (HWND);
@@ -212,7 +110,7 @@ public:
   DboxMain(CWnd* pParent = NULL);
   ~DboxMain();
 
-  enum SaveType {ST_INVALID = -1, ST_NORMALEXIT = 0, 
+  enum SaveType {ST_INVALID = -1, ST_NORMALEXIT = 0, ST_SAVEIMMEDIATELY,
                  ST_ENDSESSIONEXIT, ST_WTSLOGOFFEXIT, ST_FAILSAFESAVE};
 
   // Find entry by title and user name, exact match
@@ -259,9 +157,9 @@ public:
 
   // For InsertItemIntoGUITreeList and RefreshViews (mainly when refreshing views)
   // Note: iBothViews = iListOnly + iTreeOnly
-  enum {iListOnly = 1, iTreeOnly = 2, iBothViews = 3};
+  enum ViewType {LISTONLY = 1, TREEONLY = 2, BOTHVIEWS = 3};
 
-  void RefreshViews(const int iView = iBothViews);
+  void RefreshViews(const ViewType iView = BOTHVIEWS);
 
   // Set the section to the entry.  MakeVisible will scroll list, if needed.
   BOOL SelectEntry(const int i, BOOL MakeVisible = FALSE);
@@ -269,8 +167,17 @@ public:
   void SelectFirstEntry();
 
   int CheckPasskey(const StringX &filename, const StringX &passkey, PWScore *pcore = NULL);
-  enum ChangeType {Clear, Data, TimeStamp, DBPrefs, ClearDBPrefs};
-  void SetChanged(ChangeType changed);
+
+  // These specific changed states are only needed when no other change has been made
+  // AND the user has requested that:
+  // 1. Maintain timestamps or
+  // 2. Open database with the group display the same as when last saved
+  // NOTE: The DB core is not informed of these changes as they are only used by the UI
+  // to determine if the DB should be saved on close/exit if no other changes have been made
+  void SetEntryTimestampsChanged(const bool bEntryTimestampsChanged)
+  {m_bEntryTimestampsChanged = bEntryTimestampsChanged;}
+
+  // This updates the Save menu/toolbar button depending if there are unsaved changes
   void ChangeOkUpdate();
 
   // when Group, Title or User edited in tree
@@ -285,9 +192,11 @@ public:
   void UpdateListItemPassword(const int lindex, const StringX &sxnewPassword)
   {UpdateListItemField(lindex, CItemData::PASSWORD, sxnewPassword);}
   
-  void SetHeaderInfo();
-  CString GetHeaderText(int iType) const;
-  int GetHeaderWidth(int iType) const;
+  void SetHeaderInfo(const bool bSetWidths = true);
+  void RestoreColumnWidths();
+  void SaveColumnWidths();
+  void GetHeaderColumnProperties(const int &iType, CString &csText, int &iWidth,
+    int &iSortColumn);
   void CalcHeaderWidths();
   void UnFindItem();
   void SetLocalStrings();
@@ -299,7 +208,10 @@ public:
   void InvalidateSearch() {m_FindToolBar.InvalidateSearch();}
   void ResumeOnDBNotification() {m_core.ResumeOnDBNotification();}
   void SuspendOnDBNotification() {m_core.SuspendOnDBNotification();}
+  bool GetDBNotificationState() {return m_core.GetDBNotificationState();}
   bool IsDBReadOnly() const {return m_core.IsReadOnly();}
+  bool IsDBOpen() const { return m_bOpen; }
+  void SetDBprefsState(const bool bState) { m_bDBState = bState; }
   void SetStartSilent(bool state);
   void SetStartClosed(bool state) {m_IsStartClosed = state;}
   void SetDBInitiallyRO(bool state) {m_bDBInitiallyRO = state;}
@@ -311,7 +223,6 @@ public:
   void SetInitialDatabaseDisplay();
   void U3ExitNow(); // called when U3AppStop sends message to Pwsafe Listener
   bool ExitRequested() const {return m_inExit;}
-  void SetCapsLock(const bool bState);
   void AutoResizeColumns();
   void ResetIdleLockCounter(UINT event = WM_SIZE); // default arg always resets
   bool ClearClipboardData() {return m_clipboard.ClearData();}
@@ -323,17 +234,18 @@ public:
                          const StringX &user, const int IDS_MESSAGE) const
   {return m_core.GetUniqueTitle(group, title, user, IDS_MESSAGE);}
   void FixListIndexes();
-  void Delete(); // "Top level" delete, calls the following 2 and Execute()
+  void Delete(MultiCommands *pmcmd); // "Top level" delete, calls the following 2 and Execute()
   Command *Delete(const CItemData *pci); // create command for deleting a single item
   // For deleting a group:
   void Delete(HTREEITEM ti,
               std::vector<Command *> &vbases,
               std::vector<Command *> &vdeps,
-              std::vector<Command *> &vemptygrps); 
+              std::vector<Command *> &vemptygrps,
+              bool bExcludeTopGroup = false); 
 
-  void SaveGroupDisplayState(); // call when tree expansion state changes
+  void SaveGroupDisplayState(const bool bClear = false); // call when tree expansion state changes
   void RestoreGUIStatusEx();
-  void SaveGUIStatusEx(const int iView);
+  void SaveGUIStatusEx(const ViewType iView);
 
   const CItemData *GetBaseEntry(const CItemData *pAliasOrSC) const
   {return m_core.GetBaseEntry(pAliasOrSC);}
@@ -356,6 +268,7 @@ public:
   CItemData *GetLastSelected() const;
   StringX GetGroupName(const bool bFullPath = false) const;
   void UpdateGroupNamesInMap(const StringX sxOldPath, const StringX sxNewPath);
+  void UpdateNotesTooltipFont();
 
   void SetFilter(FilterPool selectedpool, CString selectedfiltername)
   {m_currentfilterpool = selectedpool; m_selectedfiltername = selectedfiltername;}
@@ -410,7 +323,7 @@ public:
   void SetUpdateWizardWindow(CWnd *pWnd)
   {m_pWZWnd = pWnd;}
 
-  stringT DoMerge(PWScore *pothercore,
+  std::wstring DoMerge(PWScore *pothercore,
                   const bool bAdvanced, CReport *prpt, bool *pbCancel);
   bool DoCompare(PWScore *pothercore,
                  const bool bAdvanced, CReport *prpt, bool *pbCancel);
@@ -424,10 +337,11 @@ public:
                   int &numExported, CReport *prpt);
 
   int DoExportDB(const StringX &sx_Filename, const UINT nID,
+                 const bool bExportDBFilters,
                  const StringX &sx_ExportKey, int &numExported, CReport *prpt);
 
   int TestSelection(const bool bAdvanced,
-                    const stringT &subgroup_name,
+                    const std::wstring &subgroup_name,
                     const int &subgroup_object,
                     const int &subgroup_function,
                     const OrderedItemList *pOIL) const
@@ -438,6 +352,12 @@ public:
   bool MakeMatchingGTUSet(GTUSet &setGTU, const StringX &sxPolicyName) const
   {return m_core.InitialiseGTU(setGTU, sxPolicyName);}
   CItemData *getSelectedItem();
+  void GetSelectedItems(pws_os::CUUID &entry_uuid,
+                        pws_os::CUUID &tree_find_entry_uuid, pws_os::CUUID &list_find_entry_uuid,
+                        StringX &sxGroupPath);
+  void ReSelectItems(pws_os::CUUID entry_uuid,
+                     pws_os::CUUID &tree_find_entry_uuid, pws_os::CUUID &list_find_entry_uuid,
+                     StringX sxGroupPath);
   void UpdateGUIDisplay();
   CString ShowCompareResults(const StringX sx_Filename1, const StringX sx_Filename2,
                              PWScore *pothercore, CReport *prpt);
@@ -471,7 +391,9 @@ public:
   PWPolicy m_pwp;
 
   // Mapping Group to Tree Item to save searching all the time!
+  // Be nice to have a bimap implementation
   std::map<StringX, HTREEITEM> m_mapGroupToTreeItem;
+  std::map<HTREEITEM, StringX> m_mapTreeItemToGroup;
   void GetAllGroups(std::vector<std::wstring> &vGroups) const;
 
   // Process Special Shortcuts for Tree & List controls
@@ -498,6 +420,7 @@ public:
   // If we have processed it returns 0 else 1
   BOOL ProcessEntryShortcut(WORD &wVirtualKeyCode, WORD &wModifiers);
   bool IsWorkstationLocked() const;
+  void BlockLogoffShutdown(const bool bChanged);
 
   std::set<StringX> GetAllMediaTypes() const
   {return m_core.GetAllMediaTypes();}
@@ -520,6 +443,8 @@ public:
   bool m_bIsRestoring;
   bool m_bOpen;
   bool m_bInRestoreWindowsData;
+  bool m_bUserDeclinedSave;
+  bool m_bRestoredDBUnsaved;
 
   bool m_bSetup;          // invoked with '--setup'?
   bool m_bNoValidation;   // invoked with '--novalidate'?
@@ -528,7 +453,7 @@ public:
   CString m_titlebar; // what's displayed in the title bar
 
   CPWFindToolBar m_FindToolBar;  // Find toolbar
-  CPWStatusBar m_statusBar;
+  CPWStatusBar m_StatusBar;
   BOOL m_toolbarsSetup;
   UINT m_toolbarMode;
   UINT statustext[CPWStatusBar::SB_TOTAL];
@@ -540,7 +465,9 @@ public:
   bool m_bSortAscending;
   int m_iTypeSortColumn;
 
-  bool m_bTSUpdated;
+  bool m_bDBState;
+  bool m_bEntryTimestampsChanged;
+  bool m_bGroupDisplayChanged;
   INT_PTR m_iSessionEndingStatus;
 
   // Used for Advanced functions
@@ -560,7 +487,7 @@ public:
   CMenuTipManager m_menuTipManager;
 
   int InsertItemIntoGUITreeList(CItemData &itemData, int iIndex = -1, 
-                 const bool bSort = true, const int iView = iBothViews);
+                 const bool bSort = true, const ViewType iView = BOTHVIEWS);
 
   BOOL SelItemOk();
   void setupBars();
@@ -603,7 +530,7 @@ public:
   BOOL PreTranslateMessage(MSG* pMsg);
 
   void UpdateAlwaysOnTop();
-  void ClearData(const bool clearMRE = true);
+  void ClearData(const bool bClearMRE = true);
   int NewFile(StringX &filename);
 
   void SetListView();
@@ -615,7 +542,7 @@ public:
 
   //Version of message functions with return values
   int Save(const SaveType savetype = DboxMain::ST_INVALID);
-  int SaveAs(void);
+  int SaveAs();
   int Open(const UINT uiTitle = IDS_CHOOSEDATABASE);
   int Open(const StringX &sx_Filename, const bool bReadOnly, const bool bHideReadOnly = false);
   int CheckEmergencyBackupFiles(StringX sx_Filename, StringX &passkey);
@@ -671,7 +598,6 @@ public:
   afx_msg void OnTraySelect(UINT nID);
   afx_msg void OnUpdateTraySelect(CCmdUI *pCmdUI);
 
-
   afx_msg LRESULT OnAreYouMe(WPARAM, LPARAM);
   afx_msg LRESULT OnWH_SHELL_CallBack(WPARAM wParam, LPARAM lParam);
   afx_msg LRESULT OnSessionChange(WPARAM wParam, LPARAM lParam);
@@ -750,6 +676,7 @@ public:
   afx_msg void OnChangePswdFont();
   afx_msg void OnChangeNotesFont();
   afx_msg void OnChangeVKFont();
+  afx_msg void OnChangeAddEditFont();
   afx_msg void OnViewReportsByID(UINT nID);  // From View->Reports menu
   afx_msg void OnViewReports();
   afx_msg void OnManageFilters(); // From Toolbar button
@@ -765,6 +692,7 @@ public:
   afx_msg void OnAutoType();
   afx_msg void OnGotoBaseEntry();
   afx_msg void OnEditBaseEntry();
+  afx_msg void OnViewAttachment();
   afx_msg void OnUndo();
   afx_msg void OnRedo();
   afx_msg void OnRunCommand();
@@ -820,7 +748,7 @@ private:
                          CItemData::FieldType ft, bool bUpdateGUI);
   virtual void GUISetupDisplayInfo(CItemData &ci);
   virtual void GUIRefreshEntry(const CItemData &ci);
-  virtual void UpdateWizard(const stringT &s);
+  virtual void UpdateWizard(const std::wstring &s);
 
   static int CALLBACK CompareFunc(LPARAM lParam1, LPARAM lParam2, LPARAM lParamSort);
 
@@ -848,18 +776,21 @@ private:
   int m_nColumnTypeByIndex[CItem::LAST_DATA];
   int m_nColumnWidthByIndex[CItem::LAST_DATA];
   int m_nColumnHeaderWidthByType[CItem::LAST_DATA];
+  int m_nSaveColumnHeaderWidthByType[CItem::LAST_DATA];
   int m_iheadermaxwidth;
 
   pws_os::CUUID m_LUUIDSelectedAtMinimize; // to restore List entry selection upon un-minimize
   pws_os::CUUID m_TUUIDSelectedAtMinimize; // to restore Tree entry selection upon un-minimize
-  StringX m_sxSelectedGroup;              // to restore Tree group selection upon un-minimize
+  StringX m_sxSelectedGroup;               // to restore Tree group selection upon un-minimize
   pws_os::CUUID m_LUUIDVisibleAtMinimize;  // to restore List entry position  upon un-minimize
   pws_os::CUUID m_TUUIDVisibleAtMinimize;  // to restore Tree entry position  upon un-minimize
-  StringX m_sxVisibleGroup;               // to restore Tree group position  upon un-minimize
+  StringX m_sxVisibleGroup;                // to restore Tree group position  upon un-minimize
+
+  StringX m_sxOriginalGroup;                 // Needed when doing recursive deletions of groups
 
   bool m_inExit; // help U3ExitNow
   std::vector<bool> m_vGroupDisplayState; // used to save/restore display state over minimize/restore
-  StringX m_savedDBprefs;  // used across minimize/restore events
+  StringX m_savedDBprefs;                 // used across minimize/restore events
 
   PWSclipboard m_clipboard;
 
@@ -875,6 +806,7 @@ private:
   void SetIdleLockCounter(UINT iMinutes); // set to timer counts
   bool DecrementAndTestIdleLockCounter();
   int SaveIfChanged();
+  int SaveImmediately();
   void CheckExpireList(const bool bAtOpen = false); // Upon open, timer + menu, check list, show exp.
   void TellUserAboutExpiredPasswords();
   bool RestoreWindowsData(bool bUpdateWindows, bool bShow = true);
@@ -882,7 +814,7 @@ private:
   void RestoreGroupDisplayState();
   std::vector<bool> GetGroupDisplayState(); // get current display state from window
   void SetGroupDisplayState(const std::vector<bool> &displaystatus); // changes display
-  void SetColumns();  // default order
+  void SetDefaultColumns();  // default order
   void SetColumns(const CString cs_ListColumns);
   void SetColumnWidths(const CString cs_ListColumnsWidths);
   void SetupColumnChooser(const bool bShowHide);
@@ -907,7 +839,7 @@ private:
   void AddToGUI(CItemData &ci);
   void RefreshEntryFieldInGUI(CItemData &ci, CItemData::FieldType ft);
   void RefreshEntryPasswordInGUI(CItemData &ci);
-  void RebuildGUI(const int iView = iBothViews);
+  void RebuildGUI(const ViewType iView = BOTHVIEWS);
   void UpdateEntryinGUI(CItemData &ci);
   StringX GetListViewItemText(CItemData &ci, const int &icolumn);
   
@@ -935,7 +867,7 @@ private:
 
   // Global Filters
   PWSFilterManager m_FilterManager;
-  PWSFilters m_MapFilters;
+  PWSFilters m_MapAllFilters;  // Includes DB and temporary (added, imported, autoloaded etc.)
   FilterPool m_currentfilterpool;
   CString m_selectedfiltername;
 
@@ -1000,6 +932,8 @@ private:
   bool m_bInRename;
   // When in AddGroup and where AddGroup initiated
   bool m_bInAddGroup, m_bWhitespaceRightClick;
+  // When Wizard dialog is active
+  bool m_bWizardActive;
 
   // Change languages on the fly
   void SetLanguage(LCID lcid);
