@@ -11,6 +11,7 @@
 //-----------------------------------------------------------------------------
 
 #include "PasswordSafe.h"
+#include "Windowsdefs.h"
 #include "ThisMfcApp.h"
 #include "GeneralMsgBox.h"
 #include "Shortcut.h"
@@ -88,7 +89,7 @@ int DboxMain::BackupSafe()
     dir = cdrive + cdir;
   }
 
-  //SaveAs-type dialog box
+  // SaveAs-type dialog box
   while (1) {
     CPWFileDialog fd(FALSE,
                      L"bak",
@@ -122,7 +123,7 @@ int DboxMain::BackupSafe()
   rc = m_core.WriteFile(tempname, m_core.GetReadFileVersion(), false);
   if (rc == PWScore::CANT_OPEN_FILE) {
     CGeneralMsgBox gmb;
-    cs_temp.Format(IDS_CANTOPENWRITING, tempname);
+    cs_temp.Format(IDS_CANTOPENWRITING, static_cast<LPCWSTR>(tempname.c_str()));
     cs_title.LoadString(IDS_FILEWRITEERROR);
     gmb.MessageBox(cs_temp, cs_title, MB_OK | MB_ICONWARNING);
     return PWScore::CANT_OPEN_FILE;
@@ -141,16 +142,19 @@ void DboxMain::OnRestoreSafe()
 int DboxMain::RestoreSafe()
 {
   int rc;
-  StringX backup, passkey, temp;
-  StringX currbackup =
+  StringX sx_backup, sx_passkey;
+  StringX sx_currbackup =
     PWSprefs::GetInstance()->GetPref(PWSprefs::CurrentBackup);
 
-  rc = SaveIfChanged();
-  if (rc != PWScore::SUCCESS && rc != PWScore::USER_DECLINED_SAVE)
-    return rc;
-   
-  // Reset changed flag to stop being asked again (only if rc == PWScore::USER_DECLINED_SAVE)
-  SetChanged(Clear);
+  if (m_bOpen) {
+    rc = SaveIfChanged();
+    if (rc != PWScore::SUCCESS && rc != PWScore::USER_DECLINED_SAVE)
+      return rc;
+
+    // Reset changed flag to stop being asked again (only if rc == PWScore::USER_DECLINED_SAVE)
+    if (rc == PWScore::USER_DECLINED_SAVE)
+      m_bUserDeclinedSave = true;
+  }
 
   CString cs_text, cs_temp, cs_title;
   cs_text.LoadString(IDS_PICKRESTORE);
@@ -168,7 +172,7 @@ int DboxMain::RestoreSafe()
   while (1) {
     CPWFileDialog fd(TRUE,
                      L"bak",
-                     currbackup.c_str(),
+                     sx_currbackup.c_str(),
                      OFN_FILEMUSTEXIST | OFN_HIDEREADONLY | OFN_LONGNAMES,
                      CString(MAKEINTRESOURCE(IDS_FDF_BUS)),
                      this);
@@ -188,20 +192,20 @@ int DboxMain::RestoreSafe()
       return PWScore::USER_CANCEL;
     }
     if (rc2 == IDOK) {
-      backup = fd.GetPathName();
+      sx_backup = fd.GetPathName();
       break;
     } else
       return PWScore::USER_CANCEL;
   }
 
-  rc = GetAndCheckPassword(backup, passkey, GCP_NORMAL);  // OK, CANCEL, HELP
+  rc = GetAndCheckPassword(sx_backup, sx_passkey, GCP_NORMAL);  // OK, CANCEL, HELP
 
   CGeneralMsgBox gmb;
   switch (rc) {
     case PWScore::SUCCESS:
       break; // Keep going...
     case PWScore::CANT_OPEN_FILE:
-      cs_temp.Format(IDS_CANTOPEN, backup);
+      cs_temp.Format(IDS_CANTOPEN, static_cast<LPCWSTR>(sx_backup.c_str()));
       cs_title.LoadString(IDS_FILEOPENERROR);
       gmb.MessageBox(cs_temp, cs_title, MB_OK | MB_ICONWARNING);
     case TAR_OPEN:
@@ -229,18 +233,22 @@ int DboxMain::RestoreSafe()
 
   // Validate it unless user says NO
   CReport Rpt;
-  rc = m_core.ReadFile(backup, passkey, !m_bNoValidation, MAXTEXTCHARS, &Rpt);
+  rc = m_core.ReadFile(sx_backup, sx_passkey, !m_bNoValidation, MAXTEXTCHARS, &Rpt);
   if (rc == PWScore::CANT_OPEN_FILE) {
-    cs_temp.Format(IDS_CANTOPENREADING, backup);
+    cs_temp.Format(IDS_CANTOPENREADING, static_cast<LPCWSTR>(sx_backup.c_str()));
     cs_title.LoadString(IDS_FILEREADERROR);
     gmb.MessageBox(cs_temp, cs_title, MB_OK | MB_ICONWARNING);
     return PWScore::CANT_OPEN_FILE;
   }
 
   m_core.SetCurFile(L"");    // Force a Save As...
-  m_core.SetDBChanged(true); // So that the restored file will be saved
+  // Rather than set the DB as having been changed to force it to
+  // be saved, use new variable
+  m_bRestoredDBUnsaved = true;
+
   m_titlebar.LoadString(IDS_UNTITLEDRESTORE);
   app.SetTooltipText(L"PasswordSafe");
+  
   ChangeOkUpdate();
   RefreshViews();
 
@@ -265,6 +273,9 @@ void DboxMain::OnOptions()
   else
     bAppHotKeyEnabled = prefs->GetPref(PWSprefs::HotKeyEnabled) ? TRUE : FALSE;
 
+  // Get current status of how the user wants to the initial display
+  bool bTreeOpenStatus = prefs->GetPref(PWSprefs::TreeDisplayStatusAtOpen) != PWSprefs::AsPerLastSave;
+
   // Disable Hotkey around this as the user may press the current key when 
   // selecting the new key!
   UnregisterHotKey(m_hWnd, PWS_HOTKEY_ID); // clear last - never hurts
@@ -325,11 +336,12 @@ void DboxMain::OnOptions()
         uiMessage = IDSC_STATCOMPANY;
     }
     statustext[CPWStatusBar::SB_DBLCLICK] = uiMessage;
-    m_statusBar.SetIndicators(statustext, CPWStatusBar::SB_TOTAL);
+    m_StatusBar.SetIndicators(statustext, CPWStatusBar::SB_TOTAL);
     UpdateStatusBar();
+
     // Make a sunken or recessed border around the first pane
-    m_statusBar.SetPaneInfo(CPWStatusBar::SB_DBLCLICK,
-                            m_statusBar.GetItemID(CPWStatusBar::SB_DBLCLICK),
+    m_StatusBar.SetPaneInfo(CPWStatusBar::SB_DBLCLICK,
+                            m_StatusBar.GetItemID(CPWStatusBar::SB_DBLCLICK),
                             SBPS_STRETCH, NULL);
 
     int iTrayColour = pOptionsPS->GetTrayIconColour();
@@ -384,15 +396,15 @@ void DboxMain::OnOptions()
       }
     }
 
+    m_ctlItemList.SetHighlightChanges(pOptionsPS->HighlightChanges() &&
+      !pOptionsPS->SaveImmediately());
+    m_ctlItemTree.SetHighlightChanges(pOptionsPS->HighlightChanges() &&
+      !pOptionsPS->SaveImmediately());
+
     if (pOptionsPS->RefreshViews()) {
-      m_ctlItemList.SetHighlightChanges(pOptionsPS->HighlightChanges());
-      m_ctlItemTree.SetHighlightChanges(pOptionsPS->HighlightChanges());
       RefreshViews();
     }
 
-    if (pOptionsPS->SaveGroupDisplayState())
-      SaveGroupDisplayState();
-
     if (pOptionsPS->UpdateShortcuts()) {
       // Create vector of shortcuts for user's config file
       std::vector<st_prefShortcut> vShortcuts;
@@ -494,6 +506,7 @@ void DboxMain::OnOptions()
         }
         pcmd = DBPrefsCommand::Create(&m_core, sxNewDBPrefsString);
         pmulticmds->Add(pcmd);
+
         if (bNeedGUITreeUpdate) {
           pcmd = UpdateGUICommand::Create(&m_core,
                                                   UpdateGUICommand::WN_EXECUTE_REDO,
@@ -501,12 +514,19 @@ void DboxMain::OnOptions()
           pmulticmds->Add(pcmd);
         }
       }
+
+      // Save group display if the user has switched to Explorer mode or
+      // has changed the TreeDisplayStatusAtOpen to AsPerLastSave
+      if (pOptionsPS->SaveGroupDisplayState() ||
+        (!bTreeOpenStatus &&
+          prefs->GetPref(PWSprefs::TreeDisplayStatusAtOpen, true) == PWSprefs::AsPerLastSave)) {
+        SaveGroupDisplayState();
+      }
     }
 
     const int iAction = pOptionsPS->GetPWHAction();
     const int new_default_max = pOptionsPS->GetPWHistoryMax();
     size_t ipwh_exec(0);
-    int num_altered(0);
 
     if (iAction != 0) {
       pcmd = UpdateGUICommand::Create(&m_core,
@@ -527,8 +547,11 @@ void DboxMain::OnOptions()
 
     // If DB preferences changed and/or password history options
     if (pmulticmds != NULL) {
+      int num_altered(0);
       if (pmulticmds->GetSize() > 0) {
+        // Do it
         Execute(pmulticmds);
+
         if (ipwh_exec > 0) {
           // We did do PWHistory update
           if (pmulticmds->GetRC(ipwh_exec, num_altered)) {
@@ -562,21 +585,16 @@ void DboxMain::OnOptions()
               gmb.AfxMessageBox(cs_Msg);
             }
           }
+
+          if (num_altered > 0) {
+            ChangeOkUpdate();
+          }
         } 
       } else {
         // Was created but no commands added in the end.
         delete pmulticmds;
       }
     }
-
-    if (m_core.HaveDBPrefsChanged() || num_altered > 0) {
-      if (m_core.HaveDBPrefsChanged())
-        SetChanged(DBPrefs);
-      if (num_altered > 0)
-        SetChanged(Data);
-      
-      ChangeOkUpdate();
-    }
   }
 
   // Restore hotkey as it was or as user changed it - if user pressed OK
@@ -707,6 +725,8 @@ void DboxMain::OnManagePasswordPolicies()
     Command *pcmd = DBPolicyNamesCommand::Create(&m_core, MapPSWDPLC,
                              DBPolicyNamesCommand::NP_REPLACEALL);
     pmulticmds->Add(pcmd);
+
+    // Do it
     Execute(pmulticmds);
 
     // Update Minidump user streams
