@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2015 Rony Shapiro <ronys@users.sourceforge.net>.
+* Copyright (c) 2003-2016 Rony Shapiro <ronys@pwsafe.org>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -17,6 +17,8 @@
 
 #include "core/PWSprefs.h"
 
+#include "os/file.h"
+
 #include "resource.h"
 #include "resource2.h"  // Menu, Toolbar & Accelerator resources
 #include "resource3.h"  // String resources
@@ -278,7 +280,7 @@ void DboxMain::SetUpInitialMenuStrings()
   // Do Manage Menu
   InsertShortcuts(pMainMenu, m_MapMenuShortcuts, ID_MANAGEMENU);
 
-  // No need to do Manage Menu Langauges submenu as rebuilt every time!
+  // No need to do Manage Menu Languages submenu as rebuilt every time!
 
   // Do Help Menu
   InsertShortcuts(pMainMenu, m_MapMenuShortcuts, ID_HELPMENU);
@@ -425,7 +427,7 @@ void DboxMain::UpdateAccelTable()
   // Add on space of 3 reserved shortcuts (Ctrl-Q, F4, F1)
   numscs = (int)std::count_if(m_MapMenuShortcuts.begin(), m_MapMenuShortcuts.end(),
                          cntscs) + 3;
-  // But take off 1 if there is a shprtcut for AutoType
+  // But take off 1 if there is a shortcut for AutoType
   if (m_wpAutotypeKey != 0)
     numscs--;
 
@@ -486,8 +488,8 @@ void DboxMain::SetUpMenuStrings(CMenu *pPopupMenu)
       if (iter != m_MapMenuShortcuts.end()) {
         CString str;
         if (iter->second.siVirtKey != 0) {
-          str.Format(L"%s\t%s", iter->second.name.c_str(), 
-                       CMenuShortcut::FormatShortcut(iter));
+          str.Format(L"%s\t%s", static_cast<LPCWSTR>(iter->second.name.c_str()),
+                     static_cast<LPCWSTR>(CMenuShortcut::FormatShortcut(iter)));
         } else {
           str = iter->second.name.c_str();
         }
@@ -511,7 +513,7 @@ void DboxMain::CustomiseMenu(CMenu *pPopupMenu, const UINT uiMenuID,
   const bool bTreeView = m_ctlItemTree.IsWindowVisible() == TRUE;
   const bool bItemSelected = (SelItemOk() == TRUE);
   const bool bReadOnly = m_core.IsReadOnly();
-  const CItemData *pci(NULL);
+  const CItemData *pci(NULL), *pbci(NULL);
   const wchar_t *tc_dummy = L" ";
 
   ASSERT_VALID(pPopupMenu);
@@ -523,9 +525,63 @@ void DboxMain::CustomiseMenu(CMenu *pPopupMenu, const UINT uiMenuID,
 
   // Change the 'Change Mode' text as appropriate
   if (uiMenuID == ID_FILEMENU) {
+    // Otherwise set the correct menu item text
     pPopupMenu->ModifyMenu(ID_MENUITEM_CHANGEMODE, MF_BYCOMMAND,
-                           ID_MENUITEM_CHANGEMODE,
-                           bReadOnly ? CS_READWRITE : CS_READONLY);
+      ID_MENUITEM_CHANGEMODE,
+      bReadOnly ? CS_READWRITE : CS_READONLY);
+
+    // Disable menu item if no DB or if DB is read-only on disk
+    bool bFileIsReadOnly;
+    if (!m_bOpen || (pws_os::FileExists(m_core.GetCurFile().c_str(), bFileIsReadOnly) && bFileIsReadOnly)) {
+      pPopupMenu->EnableMenuItem(ID_MENUITEM_CHANGEMODE, MF_BYCOMMAND | MF_GRAYED);
+    }
+
+    if (app.GetMRU()->GetNumUsed() == 0) {
+      pPopupMenu->EnableMenuItem(ID_MENUITEM_CLEAR_MRU, MF_BYCOMMAND | MF_GRAYED);
+      pPopupMenu->EnableMenuItem(ID_MENUITEM_MRUENTRY, MF_BYCOMMAND | MF_GRAYED);
+    }
+
+    // Remove the corresponding Export V3/V4
+    int isubmenu_pos;
+    CMenu *pSubMenu;
+    isubmenu_pos = app.FindMenuItem(pPopupMenu, ID_EXPORTMENU);
+    ASSERT(isubmenu_pos != -1);
+    pSubMenu = pPopupMenu->GetSubMenu(isubmenu_pos);
+    ASSERT_VALID(pSubMenu);
+
+    const PWSfile::VERSION current_version = m_core.GetReadFileVersion();
+
+    // Delete both menu items and then add the appropriate one
+    pSubMenu->DeleteMenu(ID_MENUITEM_EXPORT2V3FORMAT, MF_BYCOMMAND);
+    pSubMenu->DeleteMenu(ID_MENUITEM_EXPORT2V4FORMAT, MF_BYCOMMAND);
+
+    switch (current_version) {
+      case PWSfile::V30:
+      case PWSfile::V40:
+      {
+        CString tmpStr(MAKEINTRESOURCE((current_version == PWSfile::V40) ?
+                         IDS_MENUITEM_EXPORT2V3FORMAT : IDS_MENUITEM_EXPORT2V4FORMAT));
+        
+        MENUITEMINFO MII = {0};
+        MII.cbSize = sizeof(MII);
+        MII.fMask = MIIM_STRING | MIIM_ID;
+        MII.fType = MFT_STRING;
+        MII.fState = MFS_ENABLED;
+        MII.wID = (current_version == PWSfile::V40) ? ID_MENUITEM_EXPORT2V3FORMAT : ID_MENUITEM_EXPORT2V4FORMAT;
+        MII.hSubMenu = NULL;
+        MII.hbmpChecked = NULL;
+        MII.hbmpUnchecked = NULL;
+        MII.dwTypeData = (LPTSTR)&tmpStr;
+        MII.cch = tmpStr.GetLength();
+
+        // Add after export to V2
+        pSubMenu->InsertMenuItem(2, &MII, TRUE);
+        break;
+      }
+      default:
+        break;
+    }
+
     return;
   }
 
@@ -535,15 +591,6 @@ void DboxMain::CustomiseMenu(CMenu *pPopupMenu, const UINT uiMenuID,
 
   // If View menu selected (contains 'Flattened &List' menu item)
   if (uiMenuID == ID_VIEWMENU) {
-    // Delete Show Find Toolbar menu item - don't know if there or previously deleted
-    pPopupMenu->RemoveMenu(ID_MENUITEM_SHOWFINDTOOLBAR, MF_BYCOMMAND);
-    if (!m_FindToolBar.IsVisible()) {
-      // Put it back if not visible - before the separator
-      int pos = app.FindMenuItem(pPopupMenu, ID_MENUITEM_SHOWHIDE_DRAGBAR);
-      pPopupMenu->InsertMenu(pos + 1,
-                             MF_BYPOSITION  | MF_ENABLED | MF_STRING,
-                             ID_MENUITEM_SHOWFINDTOOLBAR, tc_dummy);
-    }
     pPopupMenu->CheckMenuRadioItem(ID_MENUITEM_LIST_VIEW, ID_MENUITEM_TREE_VIEW,
                                    bTreeView ? ID_MENUITEM_TREE_VIEW : ID_MENUITEM_LIST_VIEW,
                                    MF_BYCOMMAND);
@@ -585,15 +632,14 @@ void DboxMain::CustomiseMenu(CMenu *pPopupMenu, const UINT uiMenuID,
   if (bItemSelected) {
     pci = getSelectedItem();
     ASSERT(pci != NULL);
+    if (pci->IsDependent())
+      pbci = m_core.GetBaseEntry(pci);
   }
 
   // Save original entry type before possibly changing pci
   const CItemData::EntryType etype_original = 
           pci == NULL ? CItemData::ET_INVALID : pci->GetEntryType();
 
-  if (bItemSelected && pci->IsShortcut())
-    pci = m_core.GetBaseEntry(pci);
-
   if (bTreeView) {
     HTREEITEM hi = m_ctlItemTree.GetSelectedItem();
     bGroupSelected = (hi != NULL && !m_ctlItemTree.IsLeaf(hi));
@@ -665,20 +711,22 @@ void DboxMain::CustomiseMenu(CMenu *pPopupMenu, const UINT uiMenuID,
                                ID_MENUITEM_ADD, tc_dummy);
       }
 
-      // Only have Find Next/Previous if find still active and entries were found
+      // Add Find Next/Previous if find still active and entries were found
       if (m_FindToolBar.IsVisible() && m_FindToolBar.EntriesFound()) {
         pPopupMenu->AppendMenu(MF_ENABLED | MF_STRING,
                                ID_MENUITEM_FIND, tc_dummy);
         pPopupMenu->AppendMenu(MF_ENABLED | MF_STRING,
                                ID_MENUITEM_FINDUP, tc_dummy);
-      } else { // otherwise just add "Find..."
-        pPopupMenu->AppendMenu(MF_ENABLED | MF_STRING,
-                               ID_MENUITEM_FINDELLIPSIS, tc_dummy);
-      }
+      } 
+
+      // Always add "Find..."
+      pPopupMenu->AppendMenu(MF_ENABLED | MF_STRING,
+                              ID_MENUITEM_FINDELLIPSIS, tc_dummy);
 
       pPopupMenu->InsertMenu((UINT)-1, MF_SEPARATOR);
       pPopupMenu->AppendMenu(MF_ENABLED | MF_STRING,
                              ID_MENUITEM_GROUPENTER, tc_dummy);
+      
       if (!bReadOnly) {
         pPopupMenu->AppendMenu(MF_ENABLED | MF_STRING,
                                ID_MENUITEM_DELETEGROUP, tc_dummy);
@@ -700,21 +748,24 @@ void DboxMain::CustomiseMenu(CMenu *pPopupMenu, const UINT uiMenuID,
         }
       }
 
-      CMenu GGsubMenu;
-      CString GGstr;
-      GGsubMenu.CreatePopupMenu();
-      // Re-use entry menu texts
-      GGstr.LoadString(IDS_EXPORTENT2PLAINTEXT);
-      GGsubMenu.AppendMenu(MF_ENABLED | MF_STRING,
-                           ID_MENUITEM_EXPORTGRP2PLAINTEXT, GGstr);
-      GGstr.LoadString(IDS_EXPORTENT2XML);
-      GGsubMenu.AppendMenu(MF_ENABLED | MF_STRING,
-                           ID_MENUITEM_EXPORTGRP2XML, GGstr);
-      GGstr.LoadString(IDS_EXPORTENT2DB);
-      GGsubMenu.AppendMenu(MF_ENABLED | MF_STRING,
-                           ID_MENUITEM_EXPORTGRP2DB, GGstr);
-      GGstr.LoadString(IDS_EXPORTGRPMENU);
-      pPopupMenu->AppendMenu(MF_POPUP, (UINT)GGsubMenu.Detach(), GGstr);
+      // Only allow export of a group to anything if non-empty
+      if (m_ctlItemTree.CountLeafChildren(m_ctlItemTree.GetSelectedItem()) != 0) {
+        CMenu GGsubMenu;
+        CString GGstr;
+        GGsubMenu.CreatePopupMenu();
+        // Re-use entry menu texts
+        GGstr.LoadString(IDS_EXPORTENT2PLAINTEXT);
+        GGsubMenu.AppendMenu(MF_ENABLED | MF_STRING,
+          ID_MENUITEM_EXPORTGRP2PLAINTEXT, GGstr);
+        GGstr.LoadString(IDS_EXPORTENT2XML);
+        GGsubMenu.AppendMenu(MF_ENABLED | MF_STRING,
+          ID_MENUITEM_EXPORTGRP2XML, GGstr);
+        GGstr.LoadString(IDS_EXPORTENT2DB);
+        GGsubMenu.AppendMenu(MF_ENABLED | MF_STRING,
+          ID_MENUITEM_EXPORTGRP2DB, GGstr);
+        GGstr.LoadString(IDS_EXPORTGRPMENU);
+        pPopupMenu->AppendMenu(MF_POPUP, (UINT)GGsubMenu.Detach(), GGstr);
+      }
 
       pPopupMenu->InsertMenu((UINT)-1, MF_SEPARATOR);
       if (m_core.AnyToUndo() || m_core.AnyToRedo()) {
@@ -755,6 +806,7 @@ void DboxMain::CustomiseMenu(CMenu *pPopupMenu, const UINT uiMenuID,
         pPopupMenu->AppendMenu(MF_ENABLED | MF_STRING,
                                ID_MENUITEM_ADD, tc_dummy);
       }
+
       pPopupMenu->AppendMenu(MF_ENABLED | MF_STRING,
                              m_core.IsReadOnly() ? ID_MENUITEM_VIEW : ID_MENUITEM_EDIT,
                              tc_dummy);
@@ -767,16 +819,19 @@ void DboxMain::CustomiseMenu(CMenu *pPopupMenu, const UINT uiMenuID,
                                  ID_MENUITEM_RENAMEENTRY, tc_dummy);
         }
       }
+
       // Only have Find Next/Previous if find still active and entries were found
       if (m_FindToolBar.IsVisible() && m_FindToolBar.EntriesFound()) {
         pPopupMenu->AppendMenu(MF_ENABLED | MF_STRING,
                                ID_MENUITEM_FIND, tc_dummy);
         pPopupMenu->AppendMenu(MF_ENABLED | MF_STRING,
                                ID_MENUITEM_FINDUP, tc_dummy);
-      } else { // otherwise just add "Find..."
-        pPopupMenu->AppendMenu(MF_ENABLED | MF_STRING,
-                               ID_MENUITEM_FINDELLIPSIS, tc_dummy);
       }
+
+      // Always add "Find..."
+      pPopupMenu->AppendMenu(MF_ENABLED | MF_STRING,
+                             ID_MENUITEM_FINDELLIPSIS, tc_dummy);
+
       if (!bReadOnly) {
         pPopupMenu->AppendMenu(MF_ENABLED | MF_STRING,
                                ID_MENUITEM_DUPLICATEENTRY, tc_dummy);
@@ -786,7 +841,9 @@ void DboxMain::CustomiseMenu(CMenu *pPopupMenu, const UINT uiMenuID,
                                  ID_MENUITEM_ADDGROUP, tc_dummy);
         }
       }
+
       pPopupMenu->InsertMenu((UINT)-1, MF_SEPARATOR);
+      
       if (m_core.AnyToUndo() || m_core.AnyToRedo()) {
         pPopupMenu->AppendMenu(MF_ENABLED | MF_STRING,
                                ID_MENUITEM_UNDO, tc_dummy);
@@ -794,6 +851,7 @@ void DboxMain::CustomiseMenu(CMenu *pPopupMenu, const UINT uiMenuID,
                                ID_MENUITEM_REDO, tc_dummy);
         pPopupMenu->InsertMenu((UINT)-1, MF_SEPARATOR);
       }
+
       pPopupMenu->AppendMenu(MF_ENABLED | MF_STRING,
                              ID_MENUITEM_CLEARCLIPBOARD, tc_dummy);
       pPopupMenu->InsertMenu((UINT)-1, MF_SEPARATOR);
@@ -802,11 +860,11 @@ void DboxMain::CustomiseMenu(CMenu *pPopupMenu, const UINT uiMenuID,
       pPopupMenu->AppendMenu(MF_ENABLED | MF_STRING,
                              ID_MENUITEM_PASSWORDSUBSET, tc_dummy);
 
-      if (!pci->IsUserEmpty())
+      if (!pci->IsFieldValueEmpty(CItemData::USER, pbci))
         pPopupMenu->AppendMenu(MF_ENABLED | MF_STRING,
                                ID_MENUITEM_COPYUSERNAME, tc_dummy);
 
-      if (!pci->IsNotesEmpty())
+      if (!pci->IsFieldValueEmpty(CItemData::NOTES, pbci))
         pPopupMenu->AppendMenu(MF_ENABLED | MF_STRING,
                                ID_MENUITEM_COPYNOTESFLD, tc_dummy);
 
@@ -817,41 +875,45 @@ void DboxMain::CustomiseMenu(CMenu *pPopupMenu, const UINT uiMenuID,
       *    3. If URL is not empty and is an email address, add email menuitem
       *       (if not already added)
       */
-      bool bAddCopyEmail = !pci->IsEmailEmpty();
-      bool bAddSendEmail = bAddCopyEmail || (!pci->IsURLEmpty() && pci->IsURLEmail());
-      bool bAddURL = !pci->IsURLEmpty();
+      bool bAddCopyEmail = !pci->IsFieldValueEmpty(CItemData::EMAIL, pbci);
+      bool bAddSendEmail = bAddCopyEmail || 
+               (!pci->IsFieldValueEmpty(CItemData::URL, pbci) && pci->IsURLEmail(pbci));
+      bool bAddURL = !pci->IsFieldValueEmpty(CItemData::URL, pbci);
 
       // Add copies in order
       if (bAddURL) {
         pPopupMenu->AppendMenu(MF_ENABLED | MF_STRING,
                                ID_MENUITEM_COPYURL, tc_dummy);
       }
+
       if (bAddCopyEmail) {
         pPopupMenu->AppendMenu(MF_ENABLED | MF_STRING,
                                ID_MENUITEM_COPYEMAIL, tc_dummy);
       }
 
-      if (!pci->IsRunCommandEmpty())
+      if (!pci->IsFieldValueEmpty(CItemData::RUNCMD, pbci))
         pPopupMenu->AppendMenu(MF_ENABLED | MF_STRING,
                                ID_MENUITEM_COPYRUNCOMMAND, tc_dummy);
 
       pPopupMenu->InsertMenu((UINT)-1, MF_SEPARATOR);
 
       // Add actions in order
-      if (bAddURL && !pci->IsURLEmail()) {
+      if (bAddURL && !pci->IsURLEmail(pbci)) {
         pPopupMenu->AppendMenu(MF_ENABLED | MF_STRING,
                                ID_MENUITEM_BROWSEURL, tc_dummy);
         pPopupMenu->AppendMenu(MF_ENABLED | MF_STRING,
                                ID_MENUITEM_BROWSEURLPLUS, tc_dummy);
       }
+
       if (bAddSendEmail) {
         pPopupMenu->AppendMenu(MF_ENABLED | MF_STRING,
                                ID_MENUITEM_SENDEMAIL, tc_dummy);
       }
 
-      if (!pci->IsRunCommandEmpty())
+      if (!pci->IsFieldValueEmpty(CItemData::RUNCMD, pbci)) {
         pPopupMenu->AppendMenu(MF_ENABLED | MF_STRING,
                                ID_MENUITEM_RUNCOMMAND, tc_dummy);
+      }
 
       pPopupMenu->AppendMenu(MF_ENABLED | MF_STRING,
                              ID_MENUITEM_AUTOTYPE, tc_dummy);
@@ -880,6 +942,11 @@ void DboxMain::CustomiseMenu(CMenu *pPopupMenu, const UINT uiMenuID,
           ASSERT(0);
       }
 
+      if (pci->IsShortcut() ? pbci->HasAttRef() : pci->HasAttRef()) {
+        pPopupMenu->AppendMenu(MF_ENABLED | MF_STRING,
+                               ID_MENUITEM_VIEWATTACHMENT, tc_dummy);
+      }
+
       CMenu EEsubMenu;
       CString EEstr;
       EEsubMenu.CreatePopupMenu();
@@ -895,6 +962,11 @@ void DboxMain::CustomiseMenu(CMenu *pPopupMenu, const UINT uiMenuID,
       EEstr.LoadString(IDS_EXPORTENTMENU);
       pPopupMenu->AppendMenu(MF_POPUP, (UINT)EEsubMenu.Detach(), EEstr);
 
+      if (pci->IsShortcut() ? pbci->HasAttRef() : pci->HasAttRef()) {
+        pPopupMenu->AppendMenu(MF_ENABLED | MF_STRING,
+          ID_MENUITEM_EXPORT_ATTACHMENT, tc_dummy);
+      }
+
       if (!bReadOnly && etype_original != CItemData::ET_SHORTCUT)
         pPopupMenu->AppendMenu(MF_ENABLED | MF_STRING,
                                pci->IsProtected() ? ID_MENUITEM_UNPROTECT : ID_MENUITEM_PROTECT,
@@ -1102,7 +1174,7 @@ void DboxMain::OnContextMenu(CWnd * /* pWnd */, CPoint screen)
   BOOL brc;
   CPoint client;
   int item = -1;
-  const CItemData *pci = NULL;
+  const CItemData *pci(NULL), *pbci(NULL);
   CMenu menu;
 
   MENUINFO minfo ={0};
@@ -1110,10 +1182,10 @@ void DboxMain::OnContextMenu(CWnd * /* pWnd */, CPoint screen)
   minfo.fMask = MIM_MENUDATA;
 
   // Note if point = (-1, -1) then invoked via keyboard.
-  // Need coordinates of current selected itme instead on mouse position when message sent
+  // Need coordinates of current selected item instead of mouse position when message sent
   bool bKeyboard = (screen.x == -1 && screen.y == -1);
 
-  CPoint mp; // Screen co-ords (from "message point" or via Shift+F10 selected item
+  CPoint mp; // Screen coords (from "message point" or via Shift+F10 selected item
   CRect rect, appl_rect;
 
   // Get client window position
@@ -1138,7 +1210,7 @@ void DboxMain::OnContextMenu(CWnd * /* pWnd */, CPoint screen)
     }
     mp.x = (r.left + r.right) / 2;
     mp.y = (r.top + r.bottom) / 2;
-    screen = mp;  // In screen co-ords
+    screen = mp;  // In screen coords
   } else {
     mp = ::GetMessagePos();
   }
@@ -1154,7 +1226,7 @@ void DboxMain::OnContextMenu(CWnd * /* pWnd */, CPoint screen)
       brc = menu.SetMenuInfo(&minfo);
       ASSERT(brc != 0);
 
-      CMenu* pPopup = menu.GetSubMenu(0);
+      CMenu *pPopup = menu.GetSubMenu(0);
       ASSERT_VALID(pPopup);
 
       // Use this DboxMain for commands
@@ -1220,9 +1292,22 @@ void DboxMain::OnContextMenu(CWnd * /* pWnd */, CPoint screen)
           minfo.dwMenuData = IDR_POPEDITGROUP;
           menu.SetMenuInfo(&minfo);
 
-          CMenu* pPopup = menu.GetSubMenu(0);
+          CMenu *pPopup = menu.GetSubMenu(0);
           ASSERT_VALID(pPopup);
-          m_TreeViewGroup = m_ctlItemTree.GetGroup(ti);
+          m_TreeViewGroup = m_mapTreeItemToGroup[ti];
+
+          // Deal with empty groups before removing protect menu items
+          if (m_ctlItemTree.CountLeafChildren(ti) == 0) {
+            // This is an empty group or, if it has sub-groups, they are
+            // all empty too - so remove export to anything
+            CMenu *pExportPopup = pPopup->GetSubMenu(7);
+            pExportPopup->RemoveMenu(ID_MENUITEM_EXPORTGRP2PLAINTEXT, MF_BYCOMMAND);
+            pExportPopup->RemoveMenu(ID_MENUITEM_EXPORTGRP2XML, MF_BYCOMMAND);
+            pExportPopup->RemoveMenu(ID_MENUITEM_EXPORTGRP2DB, MF_BYCOMMAND);
+
+            // Since nothing left, remove this pop completely
+            pPopup->RemoveMenu(7, MF_BYPOSITION);
+          }
 
           int numProtected, numUnprotected;
           bool bProtect = GetSubtreeEntriesProtectedStatus(numProtected, numUnprotected);
@@ -1320,19 +1405,19 @@ void DboxMain::OnContextMenu(CWnd * /* pWnd */, CPoint screen)
       pPopup->RemoveMenu(pci->IsProtected() ? ID_MENUITEM_PROTECT : ID_MENUITEM_UNPROTECT, MF_BYCOMMAND);
     }
 
-    // NOTE: after here, if an entry is a Shortcut, pci points to its base entry!
-    if (pci->IsShortcut()) {
-      pci = m_core.GetBaseEntry(pci);
+    if (pci->IsDependent()) {
+      pbci = m_core.GetBaseEntry(pci);
     }
 
-    bool bCopyEmail = !pci->IsEmailEmpty();
-    bool bSendEmail = bCopyEmail || (!pci->IsURLEmpty() && pci->IsURLEmail());
-    bool bUseURL = !pci->IsURLEmpty() && !pci->IsURLEmail();
+    bool bCopyEmail = !pci->IsFieldValueEmpty(CItemData::EMAIL, pbci);
+    bool bSendEmail = bCopyEmail ||
+               (!pci->IsFieldValueEmpty(CItemData::URL, pbci) && pci->IsURLEmail(pbci));
+    bool bUseURL = !pci->IsFieldValueEmpty(CItemData::URL, pbci) && !pci->IsURLEmail(pbci);
 
-    if (pci->IsUserEmpty())
+    if (pci->IsFieldValueEmpty(CItemData::USER, pbci))
       pPopup->RemoveMenu(ID_MENUITEM_COPYUSERNAME, MF_BYCOMMAND);
 
-    if (pci->IsNotesEmpty())
+    if (pci->IsFieldValueEmpty(CItemData::NOTES, pbci))
       pPopup->RemoveMenu(ID_MENUITEM_COPYNOTESFLD, MF_BYCOMMAND);
 
     if (!bCopyEmail)
@@ -1341,7 +1426,7 @@ void DboxMain::OnContextMenu(CWnd * /* pWnd */, CPoint screen)
     if (!bSendEmail)
       pPopup->RemoveMenu(ID_MENUITEM_SENDEMAIL, MF_BYCOMMAND);
 
-    if (pci->IsURLEmpty())
+    if (pci->IsFieldValueEmpty(CItemData::URL, pbci))
       pPopup->RemoveMenu(ID_MENUITEM_COPYURL, MF_BYCOMMAND);
 
     if (!bUseURL) {
@@ -1349,11 +1434,16 @@ void DboxMain::OnContextMenu(CWnd * /* pWnd */, CPoint screen)
       pPopup->RemoveMenu(ID_MENUITEM_BROWSEURLPLUS, MF_BYCOMMAND);
     }
 
-    if (pci->IsRunCommandEmpty()) {
+    if (pci->IsFieldValueEmpty(CItemData::RUNCMD, pbci)) {
       pPopup->RemoveMenu(ID_MENUITEM_COPYRUNCOMMAND, MF_BYCOMMAND);
       pPopup->RemoveMenu(ID_MENUITEM_RUNCOMMAND, MF_BYCOMMAND);
     }
 
+    if (!pci->HasAttRef()) {
+      pPopup->RemoveMenu(ID_MENUITEM_EXPORT_ATTACHMENT, MF_BYCOMMAND);
+      pPopup->RemoveMenu(ID_MENUITEM_VIEWATTACHMENT, MF_BYCOMMAND);
+    }
+
     if (m_IsListView) {
       // Rename not valid in List View
       pPopup->RemoveMenu(ID_MENUITEM_RENAMEENTRY, MF_BYCOMMAND);
@@ -1499,7 +1589,7 @@ const unsigned int DboxMain::GetMenuShortcut(const unsigned short int &siVirtKey
   if (!sxMenuItemName.empty()) {
     // These may have the shortcut hardcoded after a tab character e.g. "\tF1" for Help
     // Remove the tab and shortcut
-    size_t found = sxMenuItemName.find_first_of(_T("\t"));
+    size_t found = sxMenuItemName.find_first_of(L"\t");
     if (found != StringX::npos)
       sxMenuItemName.erase(found);
   }
