@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2015 Rony Shapiro <ronys@users.sourceforge.net>.
+* Copyright (c) 2003-2016 Rony Shapiro <ronys@pwsafe.org>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -68,6 +68,8 @@ COptionsBackup::COptionsBackup(CWnd *pParent, st_Opt_master_data *pOPTMD)
   path = pws_os::makepath(drive, dir, L"", L"");
   m_currentFileDir = path.c_str();
   m_currentFileBasename = base.c_str();
+
+  m_bKillActiveInProgress = false;
 }
 
 void COptionsBackup::DoDataExchange(CDataExchange* pDX)
@@ -85,6 +87,11 @@ void COptionsBackup::DoDataExchange(CDataExchange* pDX)
   DDX_Text(pDX, IDC_BACKUPMAXINC, m_MaxNumIncBackups);
 
   DDX_Control(pDX, IDC_SAVEIMMEDIATELY, m_chkbox);
+
+  DDX_Control(pDX, IDC_BACKUPBEFORESAVEHELP, m_Help1);
+  DDX_Control(pDX, IDC_USERBACKUPOTHERLOCATIONHELP, m_Help2);
+  DDX_Control(pDX, IDC_USERBACKUPOTHERLOCATIONHELP2, m_Help3);
+  DDX_Control(pDX, IDC_SAVEIMMEDIATELYHELP, m_Help4);
   //}}AFX_DATA_MAP
 }
 
@@ -102,6 +109,7 @@ BEGIN_MESSAGE_MAP(COptionsBackup, COptions_PropertyPage)
   ON_STN_CLICKED(IDC_STATIC_PREFERENCES, OnPreferencesHelp)
   ON_CBN_SELCHANGE(IDC_BACKUPSUFFIX, OnComboChanged)
   ON_EN_KILLFOCUS(IDC_USERBACKUPPREFIXVALUE, OnUserPrefixKillfocus)
+  ON_EN_KILLFOCUS(IDC_USERBACKUPOTHRLOCATIONVALUE, OnUserBkpLocationKillfocus)
   ON_MESSAGE(PSM_QUERYSIBLINGS, OnQuerySiblings)
   //}}AFX_MSG_MAP
 END_MESSAGE_MAP()
@@ -113,8 +121,20 @@ BOOL COptionsBackup::OnInitDialog()
   m_chkbox.SetTextColour(CR_DATABASE_OPTIONS);
   m_chkbox.ResetBkgColour(); //Use current window's background
 
-  if (!GetMainDlg()->IsDBReadOnly())
+  if (GetMainDlg()->IsDBOpen() && !GetMainDlg()->IsDBReadOnly()) {
     GetDlgItem(IDC_STATIC_DB_PREFS_RO_WARNING)->ShowWindow(SW_HIDE);
+  }
+
+  // Database preferences - can't change in R/O mode of if no DB is open
+  if (!GetMainDlg()->IsDBOpen() || GetMainDlg()->IsDBReadOnly()) {
+    CString cs_Preference_Warning;
+    CString cs_temp(MAKEINTRESOURCE(GetMainDlg()->IsDBOpen() ? IDS_DB_READ_ONLY : IDS_NO_DB));
+
+    cs_Preference_Warning.Format(IDS_STATIC_DB_PREFS_RO_WARNING, static_cast<LPCWSTR>(cs_temp));
+    GetDlgItem(IDC_STATIC_DB_PREFS_RO_WARNING)->SetWindowText(cs_Preference_Warning);
+
+    GetDlgItem(IDC_SAVEIMMEDIATELY)->EnableWindow(FALSE);
+  }
 
   if (m_backupsuffix_cbox.GetCount() == 0) {
     // add the strings in alphabetical order
@@ -149,11 +169,32 @@ BOOL COptionsBackup::OnInitDialog()
   OnComboChanged();
   OnBackupBeforeSave();
 
-  InitToolTip(TTS_BALLOON | TTS_NOPREFIX, 4);
-  // Note naming convention: string IDS_xxx corresponds to control IDC_xxx
-  AddTool(IDC_BACKUPBEFORESAVE,        IDS_BACKUPBEFORESAVE);
-  AddTool(IDC_USERBACKUPOTHERLOCATION, IDS_USERBACKUPOTHERLOCATION);
-  ActivateToolTip();
+  if (m_BackupLocation == 1) {
+    ExpandBackupPath();
+  }
+
+  if (InitToolTip(TTS_BALLOON | TTS_NOPREFIX, 0)) {
+    m_Help1.Init(IDB_QUESTIONMARK);
+    m_Help2.Init(IDB_QUESTIONMARK);
+    m_Help3.Init(IDB_QUESTIONMARK);
+    m_Help4.Init(IDB_QUESTIONMARK);
+
+    // Note naming convention: string IDS_xxx corresponds to control IDC_xxx_HELP
+    AddTool(IDC_BACKUPBEFORESAVEHELP, IDS_BACKUPBEFORESAVE);
+    AddTool(IDC_USERBACKUPOTHERLOCATIONHELP, IDS_USERBACKUPOTHERLOCATION);
+    AddTool(IDC_USERBACKUPOTHERLOCATIONHELP2, IDS_USERBACKUPOTHERLOCATION2);
+    AddTool(IDC_SAVEIMMEDIATELYHELP, IDS_SAVEIMMEDIATELY);
+    ActivateToolTip();
+  } else {
+    m_Help1.EnableWindow(FALSE);
+    m_Help1.ShowWindow(SW_HIDE);
+    m_Help2.EnableWindow(FALSE);
+    m_Help2.ShowWindow(SW_HIDE);
+    m_Help3.EnableWindow(FALSE);
+    m_Help3.ShowWindow(SW_HIDE);
+    m_Help4.EnableWindow(FALSE);
+    m_Help4.ShowWindow(SW_HIDE);
+  }
 
   return TRUE;
 }
@@ -177,7 +218,7 @@ LRESULT COptionsBackup::OnQuerySiblings(WPARAM wParam, LPARAM )
       break;
     case PP_UPDATE_VARIABLES:
       // Since OnOK calls OnApply after we need to verify and/or
-      // copy data into the entry - we do it ourselfs here first
+      // copy data into the entry - we do it ourselves here first
       if (OnApply() == FALSE)
         return 1L;
   }
@@ -188,6 +229,9 @@ BOOL COptionsBackup::OnApply()
 {
   UpdateData(TRUE);
 
+  if (VerifyFields() == FALSE)
+    return FALSE;
+
   M_UserBackupPrefix() = m_UserBackupPrefix;
   M_BackupSuffix() = m_BackupSuffix;
   M_BackupLocation() = m_BackupLocation;
@@ -214,32 +258,68 @@ BOOL COptionsBackup::PreTranslateMessage(MSG* pMsg)
 
 BOOL COptionsBackup::OnKillActive()
 {
+  m_bKillActiveInProgress = true;
+
   COptions_PropertyPage::OnKillActive();
 
+  BOOL brc = VerifyFields();
+
+  m_bKillActiveInProgress = false;
+  return brc;
+}
+
+BOOL COptionsBackup::VerifyFields()
+{
   if (m_BackupBeforeSave != TRUE)
     return TRUE;
 
   CGeneralMsgBox gmb;
   // Check that correct fields are non-blank.
-  if (m_BackupPrefix == 1  && m_UserBackupPrefix.IsEmpty()) {
+  if (m_BackupPrefix == 1 && m_UserBackupPrefix.IsEmpty()) {
     gmb.AfxMessageBox(IDS_OPTBACKUPPREF);
     ((CEdit*)GetDlgItem(IDC_USERBACKUPPREFIXVALUE))->SetFocus();
     return FALSE;
   }
 
   if (m_BackupLocation == 1) {
+    ExpandBackupPath();
+
     if (m_UserBackupOtherLocation.IsEmpty()) {
       gmb.AfxMessageBox(IDS_OPTBACKUPLOCATION);
       ((CEdit*)GetDlgItem(IDC_USERBACKUPOTHRLOCATIONVALUE))->SetFocus();
       return FALSE;
     }
 
-    if (m_UserBackupOtherLocation.Right(1) != L"\\") {
-      m_UserBackupOtherLocation += L"\\";
-      UpdateData(FALSE);
+    if (m_csExpandedPath.GetLength() > 0) {
+      if (m_csExpandedPath.Right(1) != L"\\") {
+        m_csExpandedPath += L"\\";
+        m_UserBackupOtherLocation += L"\\";
+        UpdateData(FALSE);
+      }
+    } else {
+      if (m_UserBackupOtherLocation.Right(1) != L"\\") {
+        m_UserBackupOtherLocation += L"\\";
+        UpdateData(FALSE);
+      }
     }
 
-    if (PathIsDirectory(m_UserBackupOtherLocation) == FALSE) {
+    // PathIsDirectory will return OK even if no drive specified i.e.
+    // User specified %homepath% rather than, say, D:%homepath% or %homedrive%%homepath%
+    // This may work but we should enforce a proper expanded form.
+    CString csBackupPath = m_csExpandedPath.GetLength() > 0 ?
+      m_csExpandedPath : m_UserBackupOtherLocation;
+
+    std::wstring cdrive, cdir, dontCare;
+    pws_os::splitpath(std::wstring(csBackupPath),
+                        cdrive, cdir, dontCare, dontCare);
+
+    if (cdrive.length() == 0) {
+      gmb.AfxMessageBox(IDS_OPTBACKUPNODRIVE);
+      ((CEdit*)GetDlgItem(IDC_USERBACKUPOTHRLOCATIONVALUE))->SetFocus();
+      return FALSE;
+    }
+
+    if (PathIsDirectory(csBackupPath) == FALSE) {
       gmb.AfxMessageBox(IDS_OPTBACKUPNOLOC);
       ((CEdit*)GetDlgItem(IDC_USERBACKUPOTHRLOCATIONVALUE))->SetFocus();
       return FALSE;
@@ -247,14 +327,13 @@ BOOL COptionsBackup::OnKillActive()
   }
 
   if (m_BackupSuffix == PWSprefs::BKSFX_IncNumber &&
-    ((m_MaxNumIncBackups < 1) || (m_MaxNumIncBackups > 999))) {
-      gmb.AfxMessageBox(IDS_OPTBACKUPMAXNUM);
-      ((CEdit*)GetDlgItem(IDC_BACKUPMAXINC))->SetFocus();
-      return FALSE;
+      ((m_MaxNumIncBackups < 1) || (m_MaxNumIncBackups > 999))) {
+    gmb.AfxMessageBox(IDS_OPTBACKUPMAXNUM);
+    ((CEdit*)GetDlgItem(IDC_BACKUPMAXINC))->SetFocus();
+    return FALSE;
   }
 
   //End check
-
   return TRUE;
 }
 
@@ -313,16 +392,32 @@ void COptionsBackup::OnBackupDirectory()
     case 0:
       GetDlgItem(IDC_USERBACKUPOTHRLOCATIONVALUE)->EnableWindow(FALSE);
       GetDlgItem(IDC_BROWSEFORLOCATION)->EnableWindow(FALSE);
+
+      GetDlgItem(IDC_EXPANDEDUSERBACKUPOTHRLOC)->EnableWindow(FALSE);
+      GetDlgItem(IDC_EXPANDEDUSERBACKUPOTHRLOC)->ShowWindow(SW_HIDE);
+      GetDlgItem(IDC_EXPANDEDUSERBACKUPOTHRLOC)->SetWindowText(L"");
+
+      m_Help3.EnableWindow(FALSE);
+      m_Help3.ShowWindow(SW_HIDE);
+
       m_UserBackupOtherLocation = L"";
       break;
     case 1:
       GetDlgItem(IDC_USERBACKUPOTHRLOCATIONVALUE)->EnableWindow(TRUE);
       GetDlgItem(IDC_BROWSEFORLOCATION)->EnableWindow(TRUE);
+
+      m_Help3.EnableWindow(TRUE);
+      m_Help3.ShowWindow(SW_SHOW);
       break;
     default:
       ASSERT(0);
       break;
   }
+
+  if (m_BackupLocation == 1) {
+    ExpandBackupPath();
+  }
+
   UpdateData(FALSE);
 }
 
@@ -390,11 +485,75 @@ void COptionsBackup::SetExample()
   GetDlgItem(IDC_BACKUPEXAMPLE)->SetWindowText(cs_example);
 }
 
+void COptionsBackup::ExpandBackupPath()
+{
+  bool bSetRealPath(false);
+  if (m_BackupLocation == 1) {
+    wchar_t wsExpandedPath[MAX_PATH + 1];
+    DWORD dwResult = ExpandEnvironmentStrings(m_UserBackupOtherLocation,
+                                              wsExpandedPath, MAX_PATH + 1);
+    if (dwResult == 0 || dwResult > (MAX_PATH + 1)) {
+      CGeneralMsgBox gmb;
+      CString cs_msg, cs_title(MAKEINTRESOURCE(IDS_EXPANDPATH));
+      cs_msg.Format(IDS_CANT_EXPANDPATH, static_cast<LPCWSTR>(m_UserBackupOtherLocation));
+      gmb.MessageBox(cs_msg, cs_title, MB_OK | MB_ICONEXCLAMATION);
+    } else {
+      m_csExpandedPath = wsExpandedPath;
+      if (m_UserBackupOtherLocation != m_csExpandedPath) {
+        bSetRealPath = true;
+      }
+    }
+  }
+
+  if (!bSetRealPath)
+    m_csExpandedPath.Empty();
+
+  GetDlgItem(IDC_EXPANDEDUSERBACKUPOTHRLOC)->EnableWindow(TRUE);
+  GetDlgItem(IDC_EXPANDEDUSERBACKUPOTHRLOC)->ShowWindow(SW_SHOW);
+  GetDlgItem(IDC_EXPANDEDUSERBACKUPOTHRLOC)->SetWindowText(bSetRealPath ? m_csExpandedPath : L"");
+
+  m_Help3.EnableWindow(TRUE);
+  m_Help3.ShowWindow(SW_SHOW);
+}
+
 void COptionsBackup::OnUserPrefixKillfocus()
 {
   SetExample();
 }
 
+void COptionsBackup::OnUserBkpLocationKillfocus()
+{
+  // Windows getting the focus
+  CWnd *pWnd = GetFocus();
+
+  // Don't bother verifying data if user is clicking on the other option
+  // or browsing for a directory
+  if (pWnd == GetDlgItem(IDC_DFLTBACKUPLOCATION) ||
+      pWnd == GetDlgItem(IDC_BROWSEFORLOCATION)) {
+    return;
+  }
+
+  // Don't bother verifying data if user is cancelling the whole thing
+  // Rather complicated!!!!
+  if (pWnd == m_options_psh->GetDlgItem(IDCANCEL)) {
+    // Reset value to last good one
+    m_UserBackupOtherLocation = M_UserBackupOtherLocation();
+
+    // Now simulate the pressing of the cancel button
+    WPARAM WParam = MAKEWPARAM(IDCANCEL, BN_CLICKED);
+    m_options_psh->PostMessage(WM_COMMAND, WParam, NULL);
+    return;
+  }
+
+  UpdateData(TRUE);
+  ExpandBackupPath();
+
+  // If OnKillActive active - skip it again
+  if (!m_bKillActiveInProgress) {
+    VerifyFields();
+  }
+}
+
 void COptionsBackup::OnBrowseForLocation()
 {
   CString cs_initiallocation;
@@ -433,6 +592,8 @@ void COptionsBackup::OnBrowseForLocation()
     // free the item id list
     CoTaskMemFree(pIDL);
   }
+
+  ExpandBackupPath();
 }
 
 //  SetSelProc
