@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2014 Rony Shapiro <ronys@users.sourceforge.net>.
+* Copyright (c) 2003-2016 Rony Shapiro <ronys@pwsafe.org>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -14,7 +14,6 @@
 
 #include "resource.h"
 
-
 #include "core/pwsprefs.h"
 #include "core/core.h" // for IDSC_UNKNOWN_ERROR
 #include "core/Util.h" // for trashMemory()
@@ -44,14 +43,18 @@ CPKBaseDlg::~CPKBaseDlg()
 {
   delete m_pctlPasskey;
 
-  if (m_pVKeyBoardDlg != NULL && m_pVKeyBoardDlg->SaveKLID()) {
-    // Save Last Used Keyboard
-    UINT uiKLID = m_pVKeyBoardDlg->GetKLID();
-    std::wostringstream os;
-    os.fill(L'0');
-    os << std::nouppercase << std::hex << std::setw(8) << uiKLID;
-    StringX cs_KLID = os.str().c_str();
-    PWSprefs::GetInstance()->SetPref(PWSprefs::LastUsedKeyboard, cs_KLID);
+  if (m_pVKeyBoardDlg != NULL) {
+    if (m_pVKeyBoardDlg->SaveKLID()) {
+      // Save Last Used Keyboard
+      UINT uiKLID = m_pVKeyBoardDlg->GetKLID();
+      std::wostringstream os;
+      os.fill(L'0');
+      os << std::nouppercase << std::hex << std::setw(8) << uiKLID;
+      StringX cs_KLID = os.str().c_str();
+      PWSprefs::GetInstance()->SetPref(PWSprefs::LastUsedKeyboard, cs_KLID);
+    }
+
+    PWSprefs::GetInstance()->SetPref(PWSprefs::VKPlaySound, m_pVKeyBoardDlg->PlaySound());
 
     m_pVKeyBoardDlg->DestroyWindow();
     delete m_pVKeyBoardDlg;
@@ -76,8 +79,9 @@ void CPKBaseDlg::DoDataExchange(CDataExchange* pDX)
 }
 
 BEGIN_MESSAGE_MAP(CPKBaseDlg, CPWDialog)
-//{{AFX_MSG_MAP(CPKBaseDlg)
-//}}AFX_MSG_MAP
+  ON_WM_CTLCOLOR()
+  //{{AFX_MSG_MAP(CPKBaseDlg)
+  //}}AFX_MSG_MAP
 END_MESSAGE_MAP()
 
 BOOL CPKBaseDlg::OnInitDialog(void)
@@ -127,6 +131,13 @@ BOOL CPKBaseDlg::PreTranslateMessage(MSG* pMsg)
 {
   RelayToolTipEvent(pMsg);
 
+  // Show/hide caps lock indicator
+  CWnd *pCapsLock = GetDlgItem(IDC_CAPSLOCK);
+  if (pCapsLock != NULL) {
+    pCapsLock->ShowWindow(((GetKeyState(VK_CAPITAL) & 0x0001) == 0x0001) ?
+                          SW_SHOW : SW_HIDE);
+  }
+
   return CPWDialog::PreTranslateMessage(pMsg);
 }
 
@@ -152,7 +163,7 @@ void CPKBaseDlg::yubiShowChallengeSent()
 {
   // A request's in the air, setup GUI to wait for reply
   m_yubi_status.ShowWindow(SW_HIDE);
-  m_yubi_status.SetWindowText(_T(""));
+  m_yubi_status.SetWindowText(L"");
   m_yubi_timeout.ShowWindow(SW_SHOW);
   m_yubi_timeout.SetPos(15);
 }
@@ -164,8 +175,8 @@ void CPKBaseDlg::yubiProcessCompleted(YKLIB_RC yrc, unsigned short ts, const BYT
     m_yubi_status.ShowWindow(SW_SHOW);
     m_yubi_timeout.ShowWindow(SW_HIDE);
     m_yubi_timeout.SetPos(0);
-    m_yubi_status.SetWindowText(_T(""));
-    TRACE(_T("yubiCheckCompleted: YKLIB_OK"));
+    m_yubi_status.SetWindowText(L"");
+    pws_os::Trace(L"yubiCheckCompleted: YKLIB_OK");
     m_passkey = Bin2Hex(respBuf, SHA1_DIGEST_SIZE);
     // The returned hash is the passkey
     ProcessPhrase();
@@ -191,7 +202,7 @@ void CPKBaseDlg::yubiProcessCompleted(YKLIB_RC yrc, unsigned short ts, const BYT
     m_yubi_timeout.ShowWindow(SW_HIDE);
     m_yubi_status.ShowWindow(SW_SHOW);
     // Generic error message
-    TRACE(_T("yubiCompleted(%d)\n"), yrc);
+    pws_os::Trace(L"yubiCompleted(%d)\n", yrc);
     m_yubi_status.SetWindowText(CString(MAKEINTRESOURCE(IDSC_UNKNOWN_ERROR)));
     break;
   }
@@ -203,3 +214,16 @@ void CPKBaseDlg::OnTimer(UINT_PTR )
     YubiPoll();
 }
 
+HBRUSH CPKBaseDlg::OnCtlColor(CDC *pDC, CWnd *pWnd, UINT nCtlColor)
+{
+  HBRUSH hbr = CPWDialog::OnCtlColor(pDC, pWnd, nCtlColor);
+
+  if (nCtlColor == CTLCOLOR_STATIC) {
+    UINT nID = pWnd->GetDlgCtrlID();
+    if (nID == IDC_CAPSLOCK) {
+      pDC->SetTextColor(RGB(255, 0, 0));
+      pDC->SetBkMode(TRANSPARENT);
+    }
+  }
+  return hbr;
+}
