@@ -13,6 +13,7 @@ Passkey?  That's Russian for 'pass'.  You know, passkey
 down the streetsky.  [Groucho Marx]
 */
 
+#include "Windowsdefs.h"
 #include "PasswordSafe.h"
 #include "PWFileDialog.h"
 #include "ThisMfcApp.h"
@@ -85,9 +86,10 @@ CPasskeyEntry::CPasskeyEntry(CWnd* pParent, const CString& a_filespec, int index
   CString csSpecialBuild = pPWSver->GetSpecialBuild();
 
   if (nBuild == 0)
-    m_appversion.Format(L"V%d.%02d%s", nMajor, nMinor, csSpecialBuild);
+    m_appversion.Format(L"V%d.%02d%s", nMajor, nMinor, static_cast<LPCWSTR>(csSpecialBuild));
   else
-    m_appversion.Format(L"V%d.%02d.%02d%s", nMajor, nMinor, nBuild, csSpecialBuild);
+    m_appversion.Format(L"V%d.%02d.%02d%s", nMajor, nMinor, nBuild,
+                        static_cast<LPCWSTR>(csSpecialBuild));
 }
 
 CPasskeyEntry::~CPasskeyEntry()
@@ -121,16 +123,19 @@ BEGIN_MESSAGE_MAP(CPasskeyEntry, CPKBaseDlg)
   //{{AFX_MSG_MAP(CPasskeyEntry)
   ON_WM_DESTROY()
   ON_WM_TIMER()
+
   ON_BN_CLICKED(ID_HELP, OnHelp)
   ON_BN_CLICKED(IDC_CREATE_DB, OnCreateDb)
   ON_BN_CLICKED(IDC_EXIT, OnExit)
-  ON_CBN_EDITCHANGE(IDC_DATABASECOMBO, OnComboEditChange)
-  ON_CBN_SELCHANGE(IDC_DATABASECOMBO, OnComboSelChange)
+  ON_BN_CLICKED(IDC_YUBIKEY_BTN, OnYubikeyBtn)
   ON_BN_CLICKED(IDC_READONLY, OnBnClickedReadonly)
   ON_BN_CLICKED(IDC_BTN_BROWSE, OnOpenFileBrowser)
-  ON_MESSAGE(PWS_MSG_INSERTBUFFER, OnInsertBuffer)
   ON_STN_CLICKED(IDC_VKB, OnVirtualKeyboard)
-  ON_BN_CLICKED(IDC_YUBIKEY_BTN, OnYubikeyBtn)
+
+  ON_CBN_EDITCHANGE(IDC_DATABASECOMBO, OnComboEditChange)
+  ON_CBN_SELCHANGE(IDC_DATABASECOMBO, OnComboSelChange)
+
+  ON_MESSAGE(PWS_MSG_INSERTBUFFER, OnInsertBuffer)
   //}}AFX_MSG_MAP
 END_MESSAGE_MAP()
 
@@ -154,35 +159,13 @@ BOOL CPasskeyEntry::OnInitDialog(void)
   Fonts::GetInstance()->ApplyPasswordFont(GetDlgItem(IDC_PASSKEY));
   m_pctlPasskey->SetPasswordChar(PSSWDCHAR);
 
-  switch(m_index) {
-    case GCP_FIRST:
-      // At start up - give the user the option unless file is R-O
-      GetDlgItem(IDC_READONLY)->EnableWindow((m_bForceReadOnly || m_bFileReadOnly) ? FALSE : TRUE);
-      GetDlgItem(IDC_READONLY)->ShowWindow(SW_SHOW);
-      GetDlgItem(IDC_VERSION)->SetWindowText(m_appversion);
-      break;
-    case GCP_NORMAL:
-      // otherwise during open - user can - again unless file is R-O
-      if (m_bHideReadOnly) {
-        GetDlgItem(IDC_READONLY)->EnableWindow(FALSE);
-        GetDlgItem(IDC_READONLY)->ShowWindow(SW_HIDE);
-      } else {
-        GetDlgItem(IDC_READONLY)->EnableWindow((m_bForceReadOnly || m_bFileReadOnly) ? FALSE : TRUE);
-        GetDlgItem(IDC_READONLY)->ShowWindow(SW_SHOW);
-      }
-      break;
-    case GCP_RESTORE:
-    case GCP_WITHEXIT:
-      GetDlgItem(IDC_READONLY)->EnableWindow((m_bForceReadOnly || m_bFileReadOnly) ? FALSE : TRUE);
-      GetDlgItem(IDC_READONLY)->ShowWindow(SW_SHOW);
-      break;
-    case GCP_CHANGEMODE:
-      GetDlgItem(IDC_READONLY)->EnableWindow(FALSE);
-      GetDlgItem(IDC_READONLY)->ShowWindow(SW_HIDE);
-      break;
-    default:
-      ASSERT(FALSE);
-  }
+  GetDlgItem(IDC_READONLY)->EnableWindow((m_bForceReadOnly || m_bFileReadOnly || m_bHideReadOnly) ?
+                                         FALSE : TRUE);
+  GetDlgItem(IDC_READONLY)->ShowWindow(m_bHideReadOnly ? SW_HIDE : SW_SHOW);
+
+  CWnd *create_bn = GetDlgItem(IDC_CREATE_DB);
+  if (create_bn) // not always there
+    create_bn->EnableWindow((m_bForceReadOnly || m_PKE_ReadOnly) ? FALSE : TRUE);
 
   // Only show virtual Keyboard menu if we can load DLL
   if (!CVKeyBoardDlg::IsOSKAvailable()) {
@@ -191,6 +174,7 @@ BOOL CPasskeyEntry::OnInitDialog(void)
   }
 
   if (m_index == GCP_FIRST) {
+    GetDlgItem(IDC_VERSION)->SetWindowText(m_appversion);
     GetDlgItem(IDC_SELECTED_DATABASE)->ShowWindow(SW_HIDE);
 
     CRecentFileList *mru = app.GetMRU();
@@ -251,27 +235,37 @@ BOOL CPasskeyEntry::OnInitDialog(void)
   SetIcon(m_hIcon, TRUE);  // Set big icon
   SetIcon(m_hIcon, FALSE); // Set small icon
 
+  // Following brings to top when hotkey pressed.
+  // This is "stronger" than BringWindowToTop().
+  SetWindowPos(&CWnd::wndTopMost, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE);
+  SetActiveWindow();
+  SetForegroundWindow();
+
   if (app.WasHotKeyPressed()) {
     // Reset it
     app.SetHotKeyPressed(false);
-    // Following (1) brings to top when hotkey pressed,
-    // (2) ensures focus is on password entry field, where it belongs.
-    // This is "stronger" than BringWindowToTop().
-    SetWindowPos(&CWnd::wndTopMost, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE);
-    SetActiveWindow();
-    SetForegroundWindow();
-    m_pctlPasskey->SetFocus();
+    // Ensures focus is on password entry field, where it belongs.
+    // Do NOT use SetFocus in OnInitDialog as it bypasses the Dialog manager
+    //m_pctlPasskey->SetFocus();
+    GotoDlgCtrl(m_pctlPasskey);
     return FALSE;
   }
 
   // Following works fine for other (non-hotkey) cases:
-  SetForegroundWindow();
+  if (m_index == GCP_RESTORE || m_index == GCP_WITHEXIT) {
+    SetWindowPos(&CWnd::wndTopMost, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE);
+    SetActiveWindow();
+    SetForegroundWindow();
+  }
 
   // If the dbase field's !empty, the user most likely will want to enter
   // a password:
   if (m_index == GCP_FIRST && !m_filespec.IsEmpty()) {
     m_MRU_combo.SetEditSel(-1, -1);
-    m_pctlPasskey->SetFocus();
+    // Ensures focus is on password entry field, where it belongs.
+    // Do NOT use SetFocus in OnInitDialog as it bypasses the Dialog manager
+    //m_pctlPasskey->SetFocus();
+    GotoDlgCtrl(m_pctlPasskey);
     return FALSE;
   }
 
@@ -467,10 +461,12 @@ void CPasskeyEntry::OnComboEditChange()
   UpdateRO();
 }
 
-
 void CPasskeyEntry::OnBnClickedReadonly()
 {
   m_PKE_ReadOnly = ((CButton *)GetDlgItem(IDC_READONLY))->GetCheck() == BST_CHECKED;
+  CWnd *create_bn = GetDlgItem(IDC_CREATE_DB);
+  if (create_bn) // not always there
+    create_bn->EnableWindow(!m_PKE_ReadOnly);
 }
 
 void CPasskeyEntry::OnComboSelChange()
@@ -573,9 +569,9 @@ void CPasskeyEntry::SetHeight(const int num)
 
   if ((rect.top - sz.cy) < 0 || 
       (rect.bottom + sz.cy > ::GetSystemMetrics(SM_CYSCREEN))) {
-    int ifit = max((rect.top / ht), (::GetSystemMetrics(SM_CYSCREEN) - rect.bottom) / ht);
+    int ifit = std::max((rect.top / ht), (::GetSystemMetrics(SM_CYSCREEN) - rect.bottom) / ht);
     int ht2 = ht * ifit;
-    sz.cy = min(ht2, sz.cy);
+    sz.cy = std::min((long)ht2, sz.cy);
   }
 
   m_MRU_combo.SetWindowPos(NULL, 0, 0, sz.cx, sz.cy, SWP_NOMOVE | SWP_NOZORDER);
