@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2014 Rony Shapiro <ronys@users.sourceforge.net>.
+* Copyright (c) 2003-2016 Rony Shapiro <ronys@pwsafe.org>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -12,6 +12,8 @@
 #include "Properties.h"
 #include "InputBox.h"
 
+#include "DboxMain.h"
+
 // CProperties dialog
 
 IMPLEMENT_DYNAMIC(CProperties, CPWDialog)
@@ -46,10 +48,16 @@ BOOL CProperties::OnInitDialog()
 {
   CPWDialog::OnInitDialog();
 
+  CString ngroups;
+  ngroups.Format(IDS_NUMGROUPS_E,
+                 static_cast<LPCWSTR>(m_pdbp->numgroups.c_str()),
+                 static_cast<LPCWSTR>(m_pdbp->numemptygroups.c_str()));
+
   GetDlgItem(IDC_DATABASENAME)->SetWindowText(m_pdbp->database.c_str());
   GetDlgItem(IDC_DATABASEFORMAT)->SetWindowText(m_pdbp->databaseformat.c_str());
-  GetDlgItem(IDC_NUMGROUPS)->SetWindowText(m_pdbp->numgroups.c_str());
+  GetDlgItem(IDC_NUMGROUPS)->SetWindowText(ngroups);
   GetDlgItem(IDC_NUMENTRIES)->SetWindowText(m_pdbp->numentries.c_str());
+  GetDlgItem(IDC_NUMATTACHMENTS)->SetWindowText(m_pdbp->numattachments.c_str());
   GetDlgItem(IDC_SAVEDON)->SetWindowText(m_pdbp->whenlastsaved.c_str());
   GetDlgItem(IDC_SAVEDBY)->SetWindowText(m_pdbp->wholastsaved.c_str());
   GetDlgItem(IDC_SAVEDAPP)->SetWindowText(m_pdbp->whatlastsaved.c_str());
@@ -88,7 +96,6 @@ BOOL CProperties::OnInitDialog()
     GetDlgItem(IDC_CHANGE_NAME)->ShowWindow(SW_HIDE);
     GetDlgItem(IDC_CHANGE_DESCRIPTION)->EnableWindow(FALSE);
     GetDlgItem(IDC_CHANGE_DESCRIPTION)->ShowWindow(SW_HIDE);
-
   }
 
   return TRUE;
