@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2015 Rony Shapiro <ronys@users.sourceforge.net>.
+* Copyright (c) 2003-2016 Rony Shapiro <ronys@pwsafe.org>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -99,7 +99,7 @@ BOOL CShowCompareDlg::OnInitDialog()
 
 CString ConvertKeyBoardShortcut(int32 &iKBShortcut)
 {
-  CString kbs(_T(""));
+  CString kbs(L"");
   if (iKBShortcut != 0) {
     CString cs_temp;
     WORD wVirtualKeyCode = iKBShortcut & 0xff;
@@ -137,7 +137,7 @@ void CShowCompareDlg::PopulateResults(bool bShowAll)
     CItemData::RUNCMD, CItemData::EMAIL,
     CItemData::DCA, CItemData::SHIFTDCA,
     CItemData::PROTECTED, CItemData::SYMBOLS,
-    CItemData::POLICY, CItemData::POLICYNAME, CItemData::KBSHORTCUT,
+    CItemData::POLICY, CItemData::POLICYNAME, CItemData::KBSHORTCUT, CItemData::ATTREF,
     CItemData::CTIME, CItemData::PMTIME, CItemData::ATIME, CItemData::XTIME,
     CItemData::RMTIME, CItemData::XTIME_INT, CItemData::PWHIST, CItemData::NOTES
   };
@@ -148,7 +148,7 @@ void CShowCompareDlg::PopulateResults(bool bShowAll)
 
   // Exclude 5: UUID/GROUP/TITLE/USERNAME & RESERVED (01,02,03,04,0B) but include 1: ENTRYTYPE
   // The developer will still need to ensure new fields are processed below
-  ASSERT(sizeof(iFields)/sizeof(iFields[0]) == (CItemData::LAST - 5 + 1));
+  ASSERT(sizeof(iFields)/sizeof(iFields[0]) == (CItem::LAST_DATA - 5 + 1));
 
   StringX sxDefPolicyStr;
   LoadAString(sxDefPolicyStr, IDSC_DEFAULT_POLICY);
@@ -319,7 +319,7 @@ void CShowCompareDlg::PopulateResults(bool bShowAll)
     time_t t1(0), t2(0);
     short int si1, si2;
     StringX sxValue1, sxValue2;
-    stringT sFieldName = pci->FieldName((CItemData::FieldType)i);
+    std::wstring sFieldName = pci->FieldName((CItemData::FieldType)i);
 
     // Get field values
     // For aliases - use base entry passwords
@@ -383,6 +383,10 @@ void CShowCompareDlg::PopulateResults(bool bShowAll)
           case CItemData::XTIME_INT:  /* 0x11 */
           case CItemData::PROTECTED:  /* 0x15 */
             break;
+          case CItemData::ATTREF:     /* 0x1a */
+            sxValue1 = pci->HasAttRef() ? sxNo : sxYes;
+            sxValue2 = pci_other->HasAttRef() ? sxNo : sxYes;
+            break;
           case CItemData::DCA:        /* 0x13 */
           case CItemData::SHIFTDCA:   /* 0x17 */
             pci->GetDCA(si1, i == CItemData::SHIFTDCA);
@@ -511,7 +515,7 @@ void CShowCompareDlg::PopulateResults(bool bShowAll)
         }
 
         LoadAString(sFieldName, IDS_PWHENTRY);
-        size_t maxentries = max(pwhistlist1.size(), pwhistlist2.size());
+        size_t maxentries = std::max(pwhistlist1.size(), pwhistlist2.size());
         StringX sxBlank = L" ";
         for (size_t n = 0; n < maxentries; n++) {
           m_ListCtrl.SetItemData(iPos, LVCFMT_RIGHT);
@@ -525,7 +529,7 @@ void CShowCompareDlg::PopulateResults(bool bShowAll)
             sxValue2 = L"";
 
           if (bShowAll || sxValue1 != sxValue2) {
-            stringT str;
+            std::wstring str;
             Format(str, L"%s - %d", sFieldName.c_str(), n+1);
             iPos = m_ListCtrl.InsertItem(iPos, str.c_str());
             m_ListCtrl.SetItemText(iPos, 1, sxValue1.c_str());
@@ -586,7 +590,7 @@ CString CShowCompareDlg::GetDCAString(int iValue, bool isShift) const
   CString cs(MAKEINTRESOURCE(ui));
   if (iValue == -1) {
     CString cs1(cs);
-    cs.Format(IDS_APP_DEFAULT, cs1);
+    cs.Format(IDS_APP_DEFAULT, static_cast<LPCWSTR>(cs1));
   }
   return cs;
 }
