@@ -17,6 +17,7 @@
 
 #include "PasswordSafe.h"
 
+#include "Windowsdefs.h"
 #include "resource.h"
 #include "resource2.h"  // Menu, Toolbar & Accelerator resources
 #include "resource3.h"  // String resources
@@ -156,7 +157,7 @@ ThisMfcApp::~ThisMfcApp()
   if (m_bACCEL_Table_Created && app.m_ghAccelTable != NULL)
     DestroyAcceleratorTable(app.m_ghAccelTable);
 
-  // Alhough the system will do this automatically - I like to be clean!
+  // Although the system will do this automatically - I like to be clean!
   CloseHandle(m_hMutexOneInstance);
 
   PWSprefs::DeleteInstance();
@@ -188,9 +189,9 @@ static bool CheckFile(const CString &fn)
   CString cs_msg(L"");
 
   if (status == INVALID_FILE_ATTRIBUTES) {
-    cs_msg.Format(IDS_FILEERROR1, fn);
+    cs_msg.Format(IDS_FILEERROR1, static_cast<LPCWSTR>(fn));
   } else if (status & FILE_ATTRIBUTE_DIRECTORY) {
-    cs_msg.Format(IDS_FILEERROR2, fn);
+    cs_msg.Format(IDS_FILEERROR2, static_cast<LPCWSTR>(fn));
   }
 
   if (cs_msg.IsEmpty()) {
@@ -335,11 +336,11 @@ void ThisMfcApp::LoadLocalizedStuff()
     //  and supplied Help file
     free((void *)m_pszHelpFilePath);
 
-    cs_HelpPath.Format(L"%spwsafe.chm", cs_HelpDir);
+    cs_HelpPath.Format(L"%spwsafe.chm", static_cast<LPCWSTR>(cs_HelpDir));
     if (PathFileExists(cs_HelpPath)) {
       m_pszHelpFilePath = _wcsdup(cs_HelpPath);
       m_csHelpFile = cs_HelpPath;
-      pws_os::Trace(L"Using help file: %s\n", cs_HelpPath);
+      pws_os::Trace(L"Using help file: %s\n", static_cast<LPCWSTR>(cs_HelpPath));
     } else {
       m_pszHelpFilePath = NULL;
       m_csHelpFile = L"";
@@ -364,16 +365,17 @@ void ThisMfcApp::LoadLocalizedStuff()
     cs_LANG = szLang; cs_CTRY = szCtry;
   }
 
-  // Find reseource-only DLL if requested
+  // Find resource-only DLL if requested
   const CString format_string = (cs_CTRY.IsEmpty()) ?
-                      _T("pwsafe%s%s.dll") : _T("pwsafe%s_%s.dll");
-  cs_ResPath.Format(format_string, cs_LANG, cs_CTRY);
+                      L"pwsafe%s%s.dll" : L"pwsafe%s_%s.dll";
+  cs_ResPath.Format(format_string, static_cast<LPCWSTR>(cs_LANG),
+                       static_cast<LPCWSTR>(cs_CTRY));
   m_hInstResDLL = HMODULE(pws_os::LoadLibrary(LPCTSTR(cs_ResPath),
                                               pws_os::LOAD_LIBRARY_APP));
 
   if (m_hInstResDLL == NULL && !cs_CTRY.IsEmpty()) {
     // Now try base
-    cs_ResPath.Format(_T("pwsafe%s.dll"), cs_LANG);
+    cs_ResPath.Format(L"pwsafe%s.dll", static_cast<LPCWSTR>(cs_LANG));
     m_hInstResDLL = HMODULE(pws_os::LoadLibrary(LPCTSTR(cs_ResPath),
                                                 pws_os::LOAD_LIBRARY_APP));
   }
@@ -388,14 +390,14 @@ void ThisMfcApp::LoadLocalizedStuff()
       CGeneralMsgBox gmb;
       CString oops;
       oops.Format(L"Executable/language DLL (%s) version mismatch %d.%d.%d/%d.%d.%d.\n", 
-                  cs_ResPath,
+                  static_cast<LPCWSTR>(cs_ResPath),
                   HIWORD(dw_fileMajorMinor), LOWORD(dw_fileMajorMinor), HIWORD(dw_fileBuildRevision),
                   HIWORD(m_dwMajorMinor), LOWORD(m_dwMajorMinor), HIWORD(m_dwBuildRevision));
       gmb.AfxMessageBox(oops);
       pws_os::FreeLibrary(m_hInstResDLL);
       m_hInstResDLL = NULL;
     } else { // Passed version check
-      pws_os::Trace(L"Using language DLL '%s'.\n", cs_ResPath);
+      pws_os::Trace(L"Using language DLL '%s'.\n", static_cast<LPCWSTR>(cs_ResPath));
       GetDLLVersionData(cs_ResPath, m_ResLangID);
     }
   } // end of resource dll hunt
@@ -407,21 +409,24 @@ void ThisMfcApp::LoadLocalizedStuff()
   bool helpFileFound = false;
 
   if (!cs_LANG.IsEmpty() && !cs_CTRY.IsEmpty()) {
-    cs_HelpPath.Format(L"%spwsafe%s_%s.chm", cs_HelpDir, cs_LANG, cs_CTRY);
+    cs_HelpPath.Format(L"%spwsafe%s_%s.chm", static_cast<LPCWSTR>(cs_HelpDir),
+                       static_cast<LPCWSTR>(cs_LANG),
+                       static_cast<LPCWSTR>(cs_CTRY));
     if (PathFileExists(cs_HelpPath)) {
       helpFileFound = true;
     }
   }
 
   if (!helpFileFound && !cs_LANG.IsEmpty()) {
-    cs_HelpPath.Format(L"%spwsafe%s.chm", cs_HelpDir, cs_LANG);
+    cs_HelpPath.Format(L"%spwsafe%s.chm", static_cast<LPCWSTR>(cs_HelpDir),
+                        static_cast<LPCWSTR>(cs_LANG));
     if (PathFileExists(cs_HelpPath)) {
       helpFileFound = true;
     }
   }
 
   if (!helpFileFound) {
-    cs_HelpPath.Format(L"%spwsafe.chm", cs_HelpDir);
+    cs_HelpPath.Format(L"%spwsafe.chm",static_cast<LPCWSTR>( cs_HelpDir));
     if (PathFileExists(cs_HelpPath)) {
       helpFileFound = true;
     }
@@ -442,7 +447,7 @@ void ThisMfcApp::LoadLocalizedStuff()
 
     m_pszHelpFilePath = _wcsdup(cs_HelpPath);
     m_csHelpFile = cs_HelpPath;
-    pws_os::Trace(L"Using help file: %s\n", cs_HelpPath);
+    pws_os::Trace(L"Using help file: %s\n", static_cast<LPCWSTR>(cs_HelpPath));
   }
 }
 
@@ -652,6 +657,84 @@ void ThisMfcApp::SetLanguage()
   SetupMenu();
 }
 
+bool ThisMfcApp::GetConfigFromCommandLine(StringX &sxConfigFile, StringX &sxHost, StringX &sxUser)
+{
+  // ONLY get config file, host & user name if supplied
+  // This MUST be done before PWSprefs is first initialised
+  bool rc(false);
+  sxConfigFile = sxHost = sxUser = L"";
+
+  if (m_lpCmdLine[0] != L'\0') {
+    CString args = m_lpCmdLine;
+
+    // Start command line parsing by pushing each argument into a vector
+    // Quotes are stripped here
+    vector<CString> argvec;
+    int pos = 0;
+    CString tok;
+    do {
+      if (args[pos] == L'\"')
+        tok = args.Tokenize(L"\"", pos);
+      else
+        tok = args.Tokenize(L" ", pos);
+      if (!tok.IsEmpty())
+        argvec.push_back(tok);
+    } while (pos != -1 && pos < args.GetLength());
+
+    vector<CString>::iterator arg = argvec.begin();
+
+    while (arg != argvec.end()) {
+      if ((*arg)[0] == L'-') {
+        switch ((*arg)[1]) {
+        case L'U': case L'u': // set effective user
+          // ensure there's another non-flag argument
+          if ((arg + 1) == argvec.end() || (arg + 1)[0] == L'-') {
+            // Better stop as we don't know how bad things are!
+            return false;
+          } else {
+            arg++;
+            sxUser = *arg;
+            rc = true;
+          }
+          break;
+        case L'H': case L'h': // set effective host
+          // ensure there's another non-flag argument
+          if ((arg + 1) == argvec.end() || (arg + 1)[0] == L'-') {
+            // Better stop as we don't know how bad things are!
+            return false;
+          } else {
+            arg++;
+            sxHost = *arg;
+            rc = true;
+          }
+          break;
+        case L'G': case L'g': // override default config file
+          // ensure there's another non-flag argument
+          if ((arg + 1) == argvec.end() || (arg + 1)[0] == L'-') {
+            // Better stop as we don't know how bad things are!
+            return false;
+          } else {
+            arg++;
+            if (!pws_os::FileExists(std::wstring(*arg))) {
+              // FileExists returns false if specified file not found
+              // DON'T use PWSprefs here
+              // Better stop as we don't know how bad things are!
+              return false;
+            } else {
+              sxConfigFile = *arg;
+              rc = true;
+            }
+          }
+          break;
+        } // switch on flag
+      }
+      arg++;
+    } // while argvec
+  } // Command line not empty
+
+  return rc;
+}
+
 bool ThisMfcApp::ParseCommandLine(DboxMain &dbox, bool &allDone)
 {
   /*
@@ -741,11 +824,12 @@ bool ThisMfcApp::ParseCommandLine(DboxMain &dbox, bool &allDone)
         arg++;
         continue;
       }
+
       if ((*arg)[0] == L'-') {
         switch ((*arg)[1]) {
         case L'E': case L'e':
           isEncrypt = true;
-        // deliberate fallthru
+        // deliberate fallthrough
         case L'D': case L'd':
           allDone = true; // no need for further processing for -e/-d
         {
@@ -756,16 +840,15 @@ bool ThisMfcApp::ParseCommandLine(DboxMain &dbox, bool &allDone)
             Usage();
             return false;
           }
+
           // get password from user
-          StringX passkey;
           CCryptKeyEntry dlg(isEncrypt);
           INT_PTR nResponse = dlg.DoModal();
 
-          if (nResponse == IDOK) {
-            passkey = LPCWSTR(dlg.m_cryptkey1);
-          } else {
+          if (nResponse != IDOK) {
             return false;
           }
+          StringX passkey = LPCWSTR(dlg.m_cryptkey1);
 
           BOOL status;
           if (isEncrypt) {
@@ -775,7 +858,6 @@ bool ThisMfcApp::ParseCommandLine(DboxMain &dbox, bool &allDone)
               CGeneralMsgBox gmb;
               gmb.AfxMessageBox(errstr.c_str(), NULL, MB_OK | MB_ICONEXCLAMATION);
             }
-            return true;
           } else {
             std::wstring errstr;
             status = PWSfile::Decrypt(std::wstring(*(arg+1)), passkey, errstr);
@@ -783,8 +865,8 @@ bool ThisMfcApp::ParseCommandLine(DboxMain &dbox, bool &allDone)
               CGeneralMsgBox gmb;
               gmb.AfxMessageBox(errstr.c_str(), NULL, MB_OK | MB_ICONEXCLAMATION);
             }
-            return true;
           }
+          return true;
         } // -e or -d flag
         case L'C': case L'c':
           m_core.SetCurFile(L"");
@@ -812,31 +894,35 @@ bool ThisMfcApp::ParseCommandLine(DboxMain &dbox, bool &allDone)
           // ensure there's another non-flag argument
           if ((arg + 1) == argvec.end() || (arg + 1)[0] == L'-') {
             Usage();
-            return FALSE;
+            return false;
           } else {
             arg++;
-            SysInfo::GetInstance()->SetEffectiveUser(LPCWSTR(*arg));
+            // No need to set effective user as done in previous parsing
+            // of command line to get config information
           }
           break;
         case L'H': case L'h': // set effective host
           // ensure there's another non-flag argument
           if ((arg + 1) == argvec.end() || (arg + 1)[0] == L'-') {
             Usage();
-            return FALSE;
+            return false;
           } else {
             arg++;
-            SysInfo::GetInstance()->SetEffectiveHost(LPCWSTR(*arg));
+            // No need to set effective host as done in previous parsing
+            // of command line to get config information
           }
           break;
         case L'G': case L'g': // override default config file
           // ensure there's another non-flag argument
           if ((arg + 1) == argvec.end() || (arg + 1)[0] == L'-') {
             Usage();
-            return FALSE;
+            return false;
           } else {
             arg++;
-            if (!PWSprefs::SetConfigFile(std::wstring(*arg))) {
-              // SetConfigFile returns false if specified file not found
+            // No need to set config file as done in previous parsing
+            // of command line to get config information
+            if (!pws_os::FileExists(std::wstring(*arg))) {
+              // FileExists returns false if specified file not found
               CGeneralMsgBox gmb;
               CString missing_cfg;
               // Classic chicken-and-egg here: Since the local
@@ -845,8 +931,8 @@ bool ThisMfcApp::ParseCommandLine(DboxMain &dbox, bool &allDone)
               // safest to keep this particular error message in English
               // and out of the language-specific dlls.
               missing_cfg.Format(L"Configuration file %s not found - creating it.",
-                                 *arg);
-              gmb.AfxMessageBox(missing_cfg, L"Error", MB_OK|MB_ICONINFORMATION);
+                                 static_cast<LPCWSTR>(*arg));
+              gmb.AfxMessageBox(missing_cfg, L"Error", MB_OK | MB_ICONINFORMATION);
             }
           }
           break;
@@ -861,7 +947,7 @@ bool ThisMfcApp::ParseCommandLine(DboxMain &dbox, bool &allDone)
         if (CheckFile(*arg)) {
           fileGiven = true;
           // We send core the full path 'cause otherwise incr. backups & such get confused.
-          stringT abspath = pws_os::fullpath(arg->GetString());
+          std::wstring abspath = pws_os::fullpath(arg->GetString());
           if (abspath.empty()) // Should never happen, but try to slog on if so.
             abspath = arg->GetString();
           m_core.SetCurFile(abspath.c_str());
@@ -871,18 +957,16 @@ bool ThisMfcApp::ParseCommandLine(DboxMain &dbox, bool &allDone)
       } // if arg is a flag or not
       arg++;
     } // while argvec
+
     // If start silent && no filename specified, start closed as well
     if (startSilent && !fileGiven)
       dbox.SetStartClosed(true);
   } // Command line not empty
 
   if (!allDone) {
-    // MUST (indirectly) create PWSprefs first thing after parsing argv
-    // Ensures all things like saving locations etc. are set up.
-    PWSprefs *prefs = PWSprefs::GetInstance();
     // Update dialog orientation preference if user specified:
     if (dialogOrientation != -1)
-      prefs->SetPref(PWSprefs::DlgOrientation, dialogOrientation);
+      PWSprefs::GetInstance()->SetPref(PWSprefs::DlgOrientation, dialogOrientation);
   } // !allDone
   return true;
 }
@@ -912,7 +996,43 @@ BOOL ThisMfcApp::InitInstance()
   */
   SetRegistryKey(L"Password Safe");
 
+  m_core.SetReadOnly(false);
+  // Command line parsing MUST be done before the first PWSprefs lookup!
+  // (since user/host/config file may be overridden!)
+  bool allDone = false;
+
+  // Get config information from the command line before "really" parsing the command line!
+  StringX sxConfigFile, sxHost, sxUser;
+  if (GetConfigFromCommandLine(sxConfigFile, sxHost, sxUser)) {
+    // No errors - so set config file, user & host if supplied
+    // Set up config file
+    if (!sxConfigFile.empty())
+      PWSprefs::SetConfigFile(sxConfigFile.c_str());
+
+    if (!sxHost.empty())
+      SysInfo::GetInstance()->SetEffectiveHost(sxHost.c_str());
+
+    if (!sxUser.empty())
+      SysInfo::GetInstance()->SetEffectiveUser(sxUser.c_str());
+  }
+
+  // Set up PWSprefs for this PasswordSafe session.
+  // Whilst the language may be changed by the user via a menu, there is no user
+  // facility to change the config file, host or user later.
+  PWSprefs::GetInstance();
+
+  // Now load translations (localized messages used in Usage and encrypt/decrypt dialogs)
+  LoadLocalizedStuff();
+
+#ifndef _DEBUG
+  if (m_hInstResDLL != NULL)
+    LocalizeFaultHandler(m_hInstResDLL);
+#endif
+
+  // Do not create dbox before config data obtained as it would create PWSprefs
+  // using the potentially incorrect config data
   DboxMain dbox(NULL);
+
   std::bitset<UIInterFace::NUM_SUPPORTED> bsSupportedFunctions;
   bsSupportedFunctions.set(UIInterFace::DATABASEMODIFIED);
   bsSupportedFunctions.set(UIInterFace::UPDATEGUI);
@@ -922,13 +1042,10 @@ BOOL ThisMfcApp::InitInstance()
 
   m_core.SetUIInterFace(&dbox, UIInterFace::NUM_SUPPORTED, bsSupportedFunctions);
 
-  m_core.SetReadOnly(false);
-  // Command line parsing MUST be done before the first PWSprefs lookup!
-  // (since user/host/config file may be overriden!)
-  bool allDone = false;
-
-  // Note: Even though PWSprefs has not yet been created, parsing the command line
-  // will (via DboxMain) fill in values within it!
+  // Parse the command line again.  If there were errors getting the config file,
+  // host or user before, then this time around we will issue messages but they
+  // will probably be in English unless the config data was OK previously and
+  // the config file specifies a different language.
   bool parseVal = ParseCommandLine(dbox, allDone);
 
   // allDone will be true iff -e or -d options given, in which case
@@ -948,19 +1065,13 @@ BOOL ThisMfcApp::InitInstance()
   UNREFERENCED_PARAMETER(pFonts);
   UNREFERENCED_PARAMETER(pPWSver);
 
-  LoadLocalizedStuff();
-#ifndef _DEBUG
-  if (m_hInstResDLL != NULL)
-    LocalizeFaultHandler(m_hInstResDLL);
-#endif
-
   // Update Quiet value if via environmental variable rather than 
   // command line flag
   CString cs_PWS_QUIET;
   if (cs_PWS_QUIET.GetEnvironmentVariable(L"PWS_QUIET") != FALSE)
     m_noSysEnvWarnings = true;
 
-  // Check if the user allows muliple instances.
+  // Check if the user allows multiple instances.
   // For this to apply, consistently, must use the same copy of PasswordSafe
   // configuration file.
   if (!prefs->GetPref(PWSprefs::MultipleInstances)) {
@@ -1001,10 +1112,10 @@ BOOL ThisMfcApp::InitInstance()
   AfxInitRichEdit2();
 
   // PWScore needs it to get into database header if/when saved
-  m_core.SetApplicationNameAndVersion(AfxGetAppName(), m_dwMajorMinor);
+  m_core.SetApplicationNameAndVersion(AfxGetAppName(), m_dwMajorMinor, m_dwBuildRevision);
 
   if (m_core.GetCurFile().empty()) {
-    stringT path = prefs->GetPref(PWSprefs::CurrentFile).c_str();
+    std::wstring path = prefs->GetPref(PWSprefs::CurrentFile).c_str();
     pws_os::AddDrive(path);
     m_core.SetCurFile(path.c_str());
   }
@@ -1418,7 +1529,7 @@ void ThisMfcApp::GetLanguageFiles()
     if (len == 15)
       cs_CC = cs_dll.Mid(9, 2);
 
-    cs_temp.Format(L"pwsafe%s.chm", cs_LL);
+    cs_temp.Format(L"pwsafe%s.chm", static_cast<LPCWSTR>(cs_LL));
     cs_helpfile = cs_ExePath + cs_temp;
     std::wstring wsHelpFile = (LPCWSTR)cs_helpfile;
     bool bHelpFileExists = pws_os::FileExists(wsHelpFile);
@@ -1476,7 +1587,7 @@ void ThisMfcApp::GetLanguageFiles()
         }
         delete[] szLanguage_NativeUpper;
       }
-      // Get language name in Englsh
+      // Get language name in English
       int lnum = ::GetLocaleInfo(lcid, LOCALE_SENGLANGUAGE, NULL, 0);
       if (lnum > 0) {
         szLanguage_English = new wchar_t[lnum + 1];
