@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2014 Rony Shapiro <ronys@users.sourceforge.net>.
+* Copyright (c) 2003-2016 Rony Shapiro <ronys@pwsafe.org>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -39,8 +39,10 @@ int CWZAdvanced::dialog_lookup[] = {
   IDD_WZADVANCED,       // SYNCH
   IDD_WZADVANCED,       // EXPORT_TEXT
   IDD_WZADVANCEDBOTTOM, // EXPORT_ENTRYTEXT (reduced dialog - bottom half only)
+  IDD_WZADVANCEDBOTTOM, // EXPORT_GROUPTEXT (reduced dialog - bottom half only)
   IDD_WZADVANCED,       // EXPORT_XML
-  IDD_WZADVANCEDBOTTOM  // EXPORT_ENTRYXML (reduced dialog - bottom half only)
+  IDD_WZADVANCEDBOTTOM, // EXPORT_ENTRYXML (reduced dialog - bottom half only)
+  IDD_WZADVANCEDBOTTOM, // EXPORT_GROUPXML (reduced dialog - bottom half only)
 };
 
 /////////////////////////////////////////////////////////////////////////////
@@ -124,7 +126,7 @@ BOOL CWZAdvanced::OnInitDialog()
   CWZPropertyPage::OnInitDialog();
 
   CString cs_text, cs_tmp;
-  int iItem(-1), i;
+  int iItem(-1);
 
   // Make Next button say the function
   cs_tmp.LoadString(m_pWZPSH->GetButtonID());
@@ -154,12 +156,18 @@ BOOL CWZAdvanced::OnInitDialog()
     case WZAdvanced::EXPORT_ENTRYTEXT:
       cs_text.LoadString(IDS_EXPORT_TEXTX_SINGLE);
       break;
+    case WZAdvanced::EXPORT_GROUPTEXT:
+      cs_text.LoadString(IDS_EXPORT_TEXTX_GROUP);
+      break;
     case WZAdvanced::EXPORT_XML:
       cs_text.Format(IDS_EXPORT_XMLX);
       break;
     case WZAdvanced::EXPORT_ENTRYXML:
       cs_text.LoadString(IDS_EXPORT_XMLX_SINGLE);
       break;
+    case WZAdvanced::EXPORT_GROUPXML:
+      cs_text.LoadString(IDS_EXPORT_XMLX_GROUP);
+      break;
     default:
       ASSERT(FALSE);
   }
@@ -184,7 +192,7 @@ BOOL CWZAdvanced::OnInitDialog()
       }
     }
 
-    for (i = 0; i < cboSubgroupFunction->GetCount(); i++) {
+    for (int i = 0; i < cboSubgroupFunction->GetCount(); i++) {
       if ((int)cboSubgroupFunction->GetItemData(i) == m_subgroup_function) {
         cboSubgroupFunction->SetCurSel(i);
         break;
@@ -214,7 +222,7 @@ BOOL CWZAdvanced::OnInitDialog()
       cboSubgroupObject->SetItemData(iItem, CItemData::NOTES);
     }
 
-    for (i = 0; i < cboSubgroupObject->GetCount(); i++) {
+    for (int i = 0; i < cboSubgroupObject->GetCount(); i++) {
       if ((int)cboSubgroupObject->GetItemData(i) == m_subgroup_object) {
         cboSubgroupObject->SetCurSel(i);
         break;
@@ -228,7 +236,7 @@ BOOL CWZAdvanced::OnInitDialog()
     GetDlgItem(IDC_ADVANCED_SUBGROUP_CASE)->EnableWindow(bEnable);
   }
 
-  // Merge does not allow field selection - get out now
+  // Merge and export to current DB do not allow field selection - get out now
   if (m_iIndex == WZAdvanced::MERGE)
     return TRUE;
 
@@ -304,8 +312,10 @@ BOOL CWZAdvanced::OnInitDialog()
     case WZAdvanced::SYNCH:
     case WZAdvanced::EXPORT_TEXT:
     case WZAdvanced::EXPORT_ENTRYTEXT:
+    case WZAdvanced::EXPORT_GROUPTEXT:
     case WZAdvanced::EXPORT_XML:
     case WZAdvanced::EXPORT_ENTRYXML:
+    case WZAdvanced::EXPORT_GROUPXML:
       // All these are already selected fields
       cs_text.LoadString(IDS_COMPCTIME);
       cs_text = cs_text.Mid(2, cs_text.GetLength() - 3);
@@ -437,6 +447,7 @@ BOOL CWZAdvanced::OnInitDialog()
   switch (m_iIndex) {
     case WZAdvanced::EXPORT_XML:
     case WZAdvanced::EXPORT_ENTRYXML:
+    case WZAdvanced::EXPORT_GROUPXML:
       cs_text.LoadString(IDS_GROUP);
       iItem = m_pLC_Selected->InsertItem(++iItem, cs_text);
       m_pLC_Selected->SetItemData(iItem, CItemData::GROUP | NORMALFIELD);
@@ -521,6 +532,7 @@ BOOL CWZAdvanced::OnInitDialog()
 
     case WZAdvanced::EXPORT_TEXT:
     case WZAdvanced::EXPORT_ENTRYTEXT:
+    case WZAdvanced::EXPORT_GROUPTEXT:
       cs_text.LoadString(IDS_GROUP);
       iItem = m_pLC_Selected->InsertItem(++iItem, cs_text);
       m_pLC_Selected->SetItemData(iItem, CItemData::GROUP | NORMALFIELD);
@@ -597,7 +609,6 @@ BOOL CWZAdvanced::OnInitDialog()
   cs_ToolTip.LoadString(IDS_ADVANCED_DESELECTALL);
   m_pToolTipCtrl->AddTool(GetDlgItem(IDC_ADVANCED_DESELECTALL), cs_ToolTip);
 
-
   return TRUE;
 }
 
@@ -646,7 +657,7 @@ void CWZAdvanced::Set(CItemData::FieldBits bsFields)
   findinfo.flags = LVFI_PARAM;
   // Note: Mandatory fields have a ItemData value + 0x800 rather than 0x1000
   // and so will not be found and so not moved anywhere.
-  for (int i = 0; i < CItemData::LAST; i++) {
+  for (int i = 0; i < CItem::LAST_DATA; i++) {
     // Don't move or allow non-allowed fields
     if (!m_bsAllowedFields.test(i))
       continue;
@@ -683,7 +694,6 @@ LRESULT CWZAdvanced::OnWizardNext()
 {
   CGeneralMsgBox gmb;
   CString cs_text;
-  DWORD_PTR dw_data;
 
   UpdateData();
   m_bsFields.reset();
@@ -694,7 +704,7 @@ LRESULT CWZAdvanced::OnWizardNext()
 
     for (int i = 0; i < num_selected; i++) {
       nItem = m_pLC_Selected->GetNextItem(nItem, LVNI_ALL);
-      dw_data = LOWORD(m_pLC_Selected->GetItemData(nItem));
+      DWORD_PTR dw_data = LOWORD(m_pLC_Selected->GetItemData(nItem));
       m_bsFields.set(dw_data & 0xff, true);
     }
 
@@ -709,8 +719,10 @@ LRESULT CWZAdvanced::OnWizardNext()
           break;
         case WZAdvanced::EXPORT_TEXT:
         case WZAdvanced::EXPORT_ENTRYTEXT:
+        case WZAdvanced::EXPORT_GROUPTEXT:
         case WZAdvanced::EXPORT_XML:
         case WZAdvanced::EXPORT_ENTRYXML:
+        case WZAdvanced::EXPORT_GROUPXML:
           cs_error_msg.LoadString(IDS_NOFIELDSFOREXPORT);
           break;
         case WZAdvanced::MERGE:
@@ -764,12 +776,16 @@ LRESULT CWZAdvanced::OnWizardNext()
     case ID_MENUITEM_COMPARE:
     case ID_MENUITEM_SYNCHRONIZE:
     case ID_MENUITEM_EXPORT2PLAINTEXT:
+    case ID_MENUITEM_EXPORTGRP2PLAINTEXT:
     case ID_MENUITEM_EXPORT2XML:
-      iAll = 1;
+    case ID_MENUITEM_EXPORTGRP2XML:
+    case ID_MENUITEM_EXPORTGRP2DB:
+      iAll = 1;  // All entries
       break;
     case ID_MENUITEM_EXPORTENT2PLAINTEXT:
     case ID_MENUITEM_EXPORTENT2XML:
-      iAll = -1;
+    case ID_MENUITEM_EXPORTENT2DB:
+      iAll = -1;  // Single Entry
       break;
     default:
       ASSERT(0);
@@ -779,25 +795,27 @@ LRESULT CWZAdvanced::OnWizardNext()
   // No need to check selection for Merge as this applies to 2nd database
   // not the current database
   if (iAll != 0  && nID != ID_MENUITEM_MERGE) {
-    OrderedItemList orderedItemList;
+    OrderedItemList OIL;
     CString cs_title, cs_temp;
 
-    if (iAll == 1) {
+    if (iAll > 0) {
+      // All entries
       // Note: MakeOrderedItemList gets its members by walking the 
       // tree therefore, if a filter is active, it will ONLY export
       // those being displayed.
-      m_pWZPSH->WZPSHMakeOrderedItemList(orderedItemList);
+      m_pWZPSH->WZPSHMakeOrderedItemList(OIL);
     } else {
+      // Single Entry
       // Note: Only selected entry
       CItemData *pci = m_pWZPSH->WZPSHgetSelectedItem();
-      orderedItemList.push_back(*pci);
+      OIL.push_back(*pci);
     }
 
     // Check if there are any that meet the user's filter
     int rc = m_pWZPSH->WZPSHTestSelection(bAdvanced, (LPCWSTR)m_subgroup_name,
-                     m_subgroup_object, m_subgroup_function, &orderedItemList);
+      m_subgroup_object, m_subgroup_function, &OIL);
 
-    orderedItemList.clear(); // cleanup soonest
+    OIL.clear(); // cleanup soonest
 
     if (rc != PWScore::SUCCESS) {
       UINT uimsg(0);
@@ -811,7 +829,11 @@ LRESULT CWZAdvanced::OnWizardNext()
         case ID_MENUITEM_EXPORT2PLAINTEXT:
         case ID_MENUITEM_EXPORT2XML:
         case ID_MENUITEM_EXPORTENT2PLAINTEXT:
+        case ID_MENUITEM_EXPORTGRP2PLAINTEXT:
         case ID_MENUITEM_EXPORTENT2XML:
+        case ID_MENUITEM_EXPORTGRP2XML:
+        case ID_MENUITEM_EXPORTENT2DB:
+        case ID_MENUITEM_EXPORTGRP2DB:
           uimsg = IDS_NONE_EXPORTED;
           break;
         default:
@@ -819,10 +841,9 @@ LRESULT CWZAdvanced::OnWizardNext()
           break;
       }
 
-      CGeneralMsgBox gmb;
       CString cs_msg;
       cs_temp.LoadString(uimsg);
-      cs_msg.Format(IDS_NO_ENTRIES_PROCESSED, cs_temp);
+      cs_msg.Format(IDS_NO_ENTRIES_PROCESSED, static_cast<LPCWSTR>(cs_temp));
       cs_title.LoadString(IDS_NO_ENTRIES_SELECTED);
       gmb.MessageBox(cs_msg, cs_title, MB_OK | MB_ICONWARNING);
       return -1;
@@ -888,18 +909,14 @@ void CWZAdvanced::OnSelectSome()
 
 void CWZAdvanced::OnSelectAll()
 {
-  CString cs_text;
-  int iItem;
-  DWORD_PTR dw_data;
-
   int num_in_list = m_pLC_List->GetItemCount();
   if (num_in_list == 0)
     return;
 
   for (int i = 0; i < num_in_list; i++) {
-    cs_text = m_pLC_List->GetItemText(i, 0);
-    dw_data = m_pLC_List->GetItemData(i);
-    iItem = m_pLC_Selected->InsertItem(0, cs_text);
+    CString cs_text = m_pLC_List->GetItemText(i, 0);
+    DWORD_PTR dw_data = m_pLC_List->GetItemData(i);
+    int iItem = m_pLC_Selected->InsertItem(0, cs_text);
     m_pLC_Selected->SetItemData(iItem, dw_data);
   }
 
@@ -909,10 +926,6 @@ void CWZAdvanced::OnSelectAll()
 
 void CWZAdvanced::OnDeselectSome()
 {
-  CString cs_text;
-  int iItem;
-  DWORD_PTR dw_data;
-
   int num_selected = m_pLC_Selected->GetItemCount();
   if (num_selected == 0)
     return;
@@ -924,15 +937,15 @@ void CWZAdvanced::OnDeselectSome()
     if (nItem == -1)
       continue;
 
-    cs_text = m_pLC_Selected->GetItemText(nItem, 0);
-    dw_data = m_pLC_Selected->GetItemData(nItem);
+    CString cs_text = m_pLC_Selected->GetItemText(nItem, 0);
+    DWORD_PTR dw_data = m_pLC_Selected->GetItemData(nItem);
 
     // Ignore mandatory fields - can't be deselected
     if (m_bsMandatoryFields.test(dw_data & 0xff))
       continue;
 
     m_pLC_Selected->DeleteItem(nItem);
-    iItem = m_pLC_List->InsertItem(0, cs_text);
+    int iItem = m_pLC_List->InsertItem(0, cs_text);
     m_pLC_List->SetItemData(iItem, dw_data);
   }
 
@@ -942,22 +955,18 @@ void CWZAdvanced::OnDeselectSome()
 
 void CWZAdvanced::OnDeselectAll()
 {
-  CString cs_text;
-  int iItem;
-  DWORD_PTR dw_data;
-
   int num_selected = m_pLC_Selected->GetItemCount();
   if (num_selected == 0)
     return;
 
   for (int i = num_selected - 1; i >= 0; i--) {
-    cs_text = m_pLC_Selected->GetItemText(i, 0);
-    dw_data = m_pLC_Selected->GetItemData(i);
+    CString cs_text = m_pLC_Selected->GetItemText(i, 0);
+    DWORD_PTR dw_data = m_pLC_Selected->GetItemData(i);
     // Ignore mandatory fields - can't be deselected
     if (m_bsMandatoryFields.test(dw_data & 0xff))
       continue;
 
-    iItem = m_pLC_List->InsertItem(0, cs_text);
+    int iItem = m_pLC_List->InsertItem(0, cs_text);
     m_pLC_List->SetItemData(iItem, dw_data);
     m_pLC_Selected->DeleteItem(i);
   }
@@ -999,7 +1008,7 @@ int CALLBACK CWZAdvanced::AdvCompareFunc(LPARAM lParam1, LPARAM lParam2,
   // 4. Times last 100...
   // 5. Invalid and so shouldn't be here -1
 
-  const int iSortOrder[CItemData::LAST] = {
+  const int iSortOrder[CItem::LAST_DATA] = {
       -1 /* NAME       = 0x00 */,
       -1 /* UUID       = 0x01 */,
        0 /* GROUP      = 0x02 */,
