@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2015 Rony Shapiro <ronys@users.sourceforge.net>.
+* Copyright (c) 2003-2016 Rony Shapiro <ronys@pwsafe.org>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -32,26 +32,30 @@ IMPLEMENT_DYNAMIC(CManageFiltersDlg, CPWDialog)
 
 CManageFiltersDlg::CManageFiltersDlg(CWnd* pParent,
                                bool bFilterActive,
-                               PWSFilters &mapfilters)
+                               PWSFilters &mapfilters,
+                               bool bCanHaveAttachments)
   : CPWDialog(CManageFiltersDlg::IDD, pParent),
-  m_bFilterActive(bFilterActive), m_MapFilters(mapfilters),
+  m_bFilterActive(bFilterActive), m_MapMFDFilters(mapfilters),
   m_selectedfilterpool(FPOOL_LAST), m_selectedfiltername(L""),
   m_activefilterpool(FPOOL_LAST), m_activefiltername(L""),
   m_selectedfilter(-1), m_activefilter(-1),
   m_bDBFiltersChanged(false),
   m_num_to_export(0), m_num_to_copy(0),
   m_pCheckImageList(NULL), m_pImageList(NULL),
-  m_iSortColumn(-1), m_bSortAscending(-1)
+  m_iSortColumn(-1), m_bSortAscending(-1), m_bDBReadOnly(false),
+  m_bCanHaveAttachments(bCanHaveAttachments)
 {
   PWSFilters::iterator mf_iter;
 
-  for (mf_iter = m_MapFilters.begin();
-       mf_iter != m_MapFilters.end();
+  for (mf_iter = m_MapMFDFilters.begin();
+       mf_iter != m_MapMFDFilters.end();
        mf_iter++) {
     m_vcs_filters.push_back(mf_iter->first);
   }
 
   const COLORREF crTransparent = RGB(192, 192, 192);
+
+  // Load all images as list in enum CheckImage and in the order specified in it
   CBitmap bitmap;
   BITMAP bm;
   bitmap.LoadBitmap(IDB_CHECKED);
@@ -59,20 +63,26 @@ CManageFiltersDlg::CManageFiltersDlg(CWnd* pParent,
 
   m_pCheckImageList = new CImageList;
   BOOL status = m_pCheckImageList->Create(bm.bmWidth, bm.bmHeight,
-                                     ILC_MASK | ILC_COLOR, 3, 0);
+                                     ILC_MASK | ILC_COLOR, 4, 0);
   ASSERT(status != 0);
 
   m_pCheckImageList->Add(&bitmap, crTransparent);
   bitmap.DeleteObject();
-  bitmap.LoadBitmap(IDB_UNCHECKED);
+  bitmap.LoadBitmap(IDB_CHECKED_DISABLED);
   m_pCheckImageList->Add(&bitmap, crTransparent);
   bitmap.DeleteObject();
   bitmap.LoadBitmap(IDB_EMPTY);
   m_pCheckImageList->Add(&bitmap, crTransparent);
   bitmap.DeleteObject();
-  bitmap.LoadBitmap(IDB_BLANK);
+  bitmap.LoadBitmap(IDB_EMPTY_DISABLED);
   m_pCheckImageList->Add(&bitmap, crTransparent);
   bitmap.DeleteObject();
+
+  if (m_bCanHaveAttachments) {
+    m_sMediaTypes = GetMainDlg()->GetAllMediaTypes();
+  }
+
+  m_bDBReadOnly = GetMainDlg()->IsDBReadOnly();
 }
 
 CManageFiltersDlg::~CManageFiltersDlg()
@@ -109,13 +119,16 @@ void CManageFiltersDlg::DoDataExchange(CDataExchange* pDX)
 
 BEGIN_MESSAGE_MAP(CManageFiltersDlg, CPWDialog)
   ON_WM_DESTROY()
+
+  ON_COMMAND(IDHELP, OnHelp)
+
   ON_BN_CLICKED(IDC_FILTERNEW, OnFilterNew)
   ON_BN_CLICKED(IDC_FILTEREDIT, OnFilterEdit)
   ON_BN_CLICKED(IDC_FILTERCOPY, OnFilterCopy)
   ON_BN_CLICKED(IDC_FILTERDELETE, OnFilterDelete)
   ON_BN_CLICKED(IDC_FILTERIMPORT, OnFilterImport)
   ON_BN_CLICKED(IDC_FILTEREXPORT, OnFilterExport)
-  ON_COMMAND(IDHELP, OnHelp)
+
   ON_NOTIFY(NM_CLICK, IDC_FILTERLC, OnClick)
   ON_NOTIFY(NM_CLICK, IDC_FILTERPROPERTIES, OnClick)
   ON_NOTIFY(NM_CUSTOMDRAW, IDC_FILTERLC, OnCustomDraw)
@@ -130,6 +143,10 @@ BOOL CManageFiltersDlg::OnInitDialog()
 {
   CPWDialog::OnInitDialog();
 
+  if (m_bDBReadOnly) {
+    GetDlgItem(IDC_FILTERCOPY)->EnableWindow(FALSE);
+  }
+
   // Make some columns centered
   LVCOLUMN lvc;
   lvc.mask = LVCF_FMT;
@@ -184,8 +201,8 @@ BOOL CManageFiltersDlg::OnInitDialog()
   m_FilterLC.GetItemRect(0, &rect, LVIR_BOUNDS);
   IMAGEINFO imageinfo;
   m_pCheckImageList->GetImageInfo(0, &imageinfo);
-  int irowheight = max(rect.Height(),
-                       abs(imageinfo.rcImage.top - imageinfo.rcImage.bottom));
+  int irowheight = std::max(rect.Height(),
+                       (int)abs(imageinfo.rcImage.top - imageinfo.rcImage.bottom));
   m_pImageList = new CImageList;
   m_pImageList->Create(1, irowheight, ILC_COLOR4, 1, 1);
   m_FilterLC.SetImageList(m_pImageList, LVSIL_SMALL);
@@ -240,6 +257,11 @@ BOOL CManageFiltersDlg::OnInitDialog()
   GetDlgItem(IDC_FILTEREDIT)->EnableWindow(FALSE);
   GetDlgItem(IDC_FILTERDELETE)->EnableWindow(FALSE);
 
+  // Don't import if we can't validate
+#ifndef USE_XML_LIBRARY
+  GetDlgItem(IDC_FILTERIMPORT)->EnableWindow(FALSE);
+#endif
+
   UpdateData(FALSE);
 
   return FALSE;
@@ -290,15 +312,17 @@ void CManageFiltersDlg::OnClick(NMHDR *pNotifyStruct, LRESULT *pLResult)
       }
       break;
     case MFLC_COPYTODATABASE:
-      if (bOnImage == TRUE && pflt_idata->flt_key.fpool != FPOOL_DATABASE) {
-        if ((pflt_idata->flt_flags & MFLT_REQUEST_COPY_TO_DB) == MFLT_REQUEST_COPY_TO_DB) {
-          pflt_idata->flt_flags &= ~MFLT_REQUEST_COPY_TO_DB;
-          m_num_to_copy--;
-        } else {
-          pflt_idata->flt_flags |= MFLT_REQUEST_COPY_TO_DB;
-          m_num_to_copy++;
+      if (!m_bDBReadOnly) {
+        if (bOnImage == TRUE && pflt_idata->flt_key.fpool != FPOOL_DATABASE) {
+          if ((pflt_idata->flt_flags & MFLT_REQUEST_COPY_TO_DB) == MFLT_REQUEST_COPY_TO_DB) {
+            pflt_idata->flt_flags &= ~MFLT_REQUEST_COPY_TO_DB;
+            m_num_to_copy--;
+          } else {
+            pflt_idata->flt_flags |= MFLT_REQUEST_COPY_TO_DB;
+            m_num_to_copy++;
+          }
+          GetDlgItem(IDC_FILTERCOPY)->EnableWindow(m_num_to_copy > 0);
         }
-        GetDlgItem(IDC_FILTERCOPY)->EnableWindow(m_num_to_copy > 0);
       }
       break;
     case MFLC_EXPORT:
@@ -325,8 +349,8 @@ void CManageFiltersDlg::OnClick(NMHDR *pNotifyStruct, LRESULT *pLResult)
   flt_key.fpool = m_selectedfilterpool;
   flt_key.cs_filtername = m_selectedfiltername;
 
-  mf_iter = m_MapFilters.find(flt_key);
-  if (mf_iter == m_MapFilters.end())
+  mf_iter = m_MapMFDFilters.find(flt_key);
+  if (mf_iter == m_MapMFDFilters.end())
     return;
 
   st_filters *pfilters = &mf_iter->second;
@@ -350,10 +374,10 @@ do_edit:
 
   if (bCreated) {
     PWSFilters::const_iterator mf_citer;
-    mf_citer = m_MapFilters.find(flt_key);
+    mf_citer = m_MapMFDFilters.find(flt_key);
 
     // Check if already there (i.e. ask user if to replace)
-    if (mf_citer != m_MapFilters.end()) {
+    if (mf_citer != m_MapMFDFilters.end()) {
       CGeneralMsgBox gmb;
       CString cs_msg(MAKEINTRESOURCE(IDS_REPLACEFILTER));
       CString cs_title(MAKEINTRESOURCE(IDS_FILTEREXISTS));
@@ -363,7 +387,7 @@ do_edit:
       if (rc == IDNO)
         goto do_edit;
 
-      m_MapFilters.erase(flt_key);
+      m_MapMFDFilters.erase(flt_key);
 
       // If this was active, we need to clear it and re-apply
       if (m_bFilterActive && 
@@ -372,7 +396,7 @@ do_edit:
         bJustDoIt = true;
       }
     }
-    m_MapFilters.insert(PWSFilters::Pair(flt_key, filters));
+    m_MapMFDFilters.insert(PWSFilters::Pair(flt_key, filters));
 
     // Update DboxMain
     GetMainDlg()->SetFilter(FPOOL_SESSION, filters.fname.c_str());
@@ -395,8 +419,8 @@ void CManageFiltersDlg::OnFilterEdit()
   flt_key.fpool = m_selectedfilterpool;
   flt_key.cs_filtername = m_selectedfiltername;
 
-  mf_iter = m_MapFilters.find(flt_key);
-  if (mf_iter == m_MapFilters.end())
+  mf_iter = m_MapMFDFilters.find(flt_key);
+  if (mf_iter == m_MapMFDFilters.end())
     return;
 
   // Pass a copy of (not reference to) of the current filter in case 
@@ -415,10 +439,10 @@ do_edit:
       flt_otherkey.fpool = m_selectedfilterpool;
       flt_otherkey.cs_filtername = filters.fname;
 
-      mf_citer = m_MapFilters.find(flt_otherkey);
+      mf_citer = m_MapMFDFilters.find(flt_otherkey);
 
       // Check if already there (i.e. ask user if to replace)
-      if (mf_citer != m_MapFilters.end()) {
+      if (mf_citer != m_MapMFDFilters.end()) {
         CGeneralMsgBox gmb;
         CString cs_msg(MAKEINTRESOURCE(IDS_REPLACEFILTER));
         CString cs_title(MAKEINTRESOURCE(IDS_FILTEREXISTS));
@@ -444,9 +468,9 @@ do_edit:
 
     // User may have changed name (and so key) - delete and add again
     // Have to anyway, even if name not changed.
-    m_MapFilters.erase(bReplacedOther ? flt_otherkey : flt_key);
+    m_MapMFDFilters.erase(bReplacedOther ? flt_otherkey : flt_key);
     flt_key.cs_filtername = filters.fname;
-    m_MapFilters.insert(PWSFilters::Pair(flt_key, filters));
+    m_MapMFDFilters.insert(PWSFilters::Pair(flt_key, filters));
     m_selectedfiltername = flt_key.cs_filtername.c_str();
 
     // Update DboxMain's current filter
@@ -473,18 +497,18 @@ void CManageFiltersDlg::OnFilterCopy()
     st_Filterkey flt_key;
     flt_key = pflt_idata->flt_key;
 
-    mf_iter = m_MapFilters.find(flt_key);
-    if (mf_iter == m_MapFilters.end())
+    mf_iter = m_MapMFDFilters.find(flt_key);
+    if (mf_iter == m_MapMFDFilters.end())
       return;
 
     PWSFilters::const_iterator mf_citer;
     st_Filterkey flt_keydb;
     flt_keydb.fpool = FPOOL_DATABASE;
     flt_keydb.cs_filtername = flt_key.cs_filtername;
-    mf_citer = m_MapFilters.find(flt_keydb);
+    mf_citer = m_MapMFDFilters.find(flt_keydb);
 
     // Check if already there (i.e. ask user if to replace)
-    if (mf_citer != m_MapFilters.end()) {
+    if (mf_citer != m_MapMFDFilters.end()) {
       CGeneralMsgBox gmb;
       CString cs_msg(MAKEINTRESOURCE(IDS_REPLACEFILTER));
       CString cs_title(MAKEINTRESOURCE(IDS_FILTEREXISTS));
@@ -493,9 +517,9 @@ void CManageFiltersDlg::OnFilterCopy()
         continue;  // skip this one
 
       // User agrees to replace
-      m_MapFilters.erase(flt_keydb);
+      m_MapMFDFilters.erase(flt_keydb);
     }
-    m_MapFilters.insert(PWSFilters::Pair(flt_keydb, mf_iter->second));
+    m_MapMFDFilters.insert(PWSFilters::Pair(flt_keydb, mf_iter->second));
 
     // Turn off copy flag
     pflt_idata->flt_flags &= ~MFLT_REQUEST_COPY_TO_DB;
@@ -504,7 +528,6 @@ void CManageFiltersDlg::OnFilterCopy()
   }
   if (bCopied) {
     m_bDBFiltersChanged = true;
-    GetMainDlg()->SetChanged(DboxMain::Data);
     GetMainDlg()->ChangeOkUpdate();
   }
 
@@ -524,8 +547,8 @@ void CManageFiltersDlg::OnFilterDelete()
   flt_key.fpool = m_selectedfilterpool;
   flt_key.cs_filtername = cs_selected;
 
-  mf_iter = m_MapFilters.find(flt_key);
-  if (mf_iter == m_MapFilters.end())
+  mf_iter = m_MapMFDFilters.find(flt_key);
+  if (mf_iter == m_MapMFDFilters.end())
     return;
 
   cs_pool = GetFilterPoolName(flt_key.fpool);
@@ -533,14 +556,14 @@ void CManageFiltersDlg::OnFilterDelete()
   // Now to confirm with user:
   CString cs_msg;
   CGeneralMsgBox gmb;
-  cs_msg.Format(IDS_CONFIRMFILTERDELETE, cs_pool, cs_selected);
+  cs_msg.Format(IDS_CONFIRMFILTERDELETE, static_cast<LPCWSTR>(cs_pool),
+                static_cast<LPCWSTR>(cs_selected));
   if (gmb.AfxMessageBox(cs_msg, NULL, MB_YESNO | MB_ICONWARNING | MB_DEFBUTTON2) != IDYES)
     return;
 
-  m_MapFilters.erase(flt_key);
+  m_MapMFDFilters.erase(flt_key);
   if (m_selectedfilterpool == FPOOL_DATABASE) {
     m_bDBFiltersChanged = true;
-    GetMainDlg()->SetChanged(DboxMain::Data);
     GetMainDlg()->ChangeOkUpdate();
   }
 
@@ -591,8 +614,8 @@ void CManageFiltersDlg::OnFilterExport()
       continue;
 
     PWSFilters::iterator mf_iter;
-    mf_iter = m_MapFilters.find(pflt_idata->flt_key);
-    if (mf_iter == m_MapFilters.end())
+    mf_iter = m_MapMFDFilters.find(pflt_idata->flt_key);
+    if (mf_iter == m_MapMFDFilters.end())
       continue;
 
     Filters.insert(PWSFilters::Pair(pflt_idata->flt_key, mf_iter->second));
@@ -648,7 +671,7 @@ void CManageFiltersDlg::DisplayFilterProperties(st_filters *pfilters)
   CString cs_num, cs_ftype, cs_criteria, cs_ltype, cs_act;
 
   vFilterRows::iterator Flt_iter;
-  bool bHistory(false), bPolicy(false);
+  bool bHistory(false), bPolicy(false), bAttachment(false);
   int i(0), iItem(0), n(0);
 
   m_FilterProperties.DeleteAllItems();
@@ -665,7 +688,7 @@ void CManageFiltersDlg::DisplayFilterProperties(st_filters *pfilters)
     cs_ftype = GetFieldTypeName(st_fldata.ftype);
     cs_criteria = PWSFilters::GetFilterDescription(st_fldata).c_str();
     cs_act.LoadString(st_fldata.bFilterActive ? IDS_YES : IDS_NO);
-    cs_act = cs_act.Mid(1);  // Remove leading ampersand from Yes/No
+    cs_act.Remove(L'&');  // Remove leading ampersand from Yes/No
     if (Flt_iter != pfilters->vMfldata.begin())
       cs_ltype.LoadString(st_fldata.ltype == LC_AND ? IDSC_AND : IDSC_OR);
     else
@@ -681,6 +704,8 @@ void CManageFiltersDlg::DisplayFilterProperties(st_filters *pfilters)
       bHistory = true;
     if (st_fldata.ftype == FT_POLICY)
       bPolicy = true;
+    if (st_fldata.ftype == FT_ATTACHMENT)
+      bAttachment = true;
   }
 
   if (bHistory) {
@@ -706,6 +731,7 @@ void CManageFiltersDlg::DisplayFilterProperties(st_filters *pfilters)
     cs_ftype = GetFieldTypeName(st_fldata.ftype);
     cs_criteria = PWSFilters::GetFilterDescription(st_fldata).c_str();
     cs_act.LoadString(st_fldata.bFilterActive ? IDS_YES : IDS_NO);
+    cs_act.Remove(L'&');  // Remove leading ampersand from Yes/No
     if (Flt_iter != pfilters->vHfldata.begin())
       cs_ltype.LoadString(st_fldata.ltype == LC_AND ? IDSC_AND : IDSC_OR);
     else
@@ -741,6 +767,7 @@ void CManageFiltersDlg::DisplayFilterProperties(st_filters *pfilters)
     cs_ftype = GetFieldTypeName(st_fldata.ftype);
     cs_criteria = PWSFilters::GetFilterDescription(st_fldata).c_str();
     cs_act.LoadString(st_fldata.bFilterActive ? IDS_YES : IDS_NO);
+    cs_act.Remove(L'&');  // Remove leading ampersand from Yes/No
     if (Flt_iter != pfilters->vPfldata.begin())
       cs_ltype.LoadString(st_fldata.ltype == LC_AND ? IDSC_AND : IDSC_OR);
     else
@@ -753,6 +780,42 @@ void CManageFiltersDlg::DisplayFilterProperties(st_filters *pfilters)
     m_FilterProperties.SetItemText(iItem, MFPRP_CRITERIA_TEXT, cs_criteria);
   }
 
+  if (bAttachment) {
+    i++;
+    CString cs_attachment, cs_temp;
+    cs_attachment.LoadString(IDS_SETATTACHMENTFILTER);
+    iItem = m_FilterProperties.InsertItem(i /* MFPRP_FILTER_NUMBER */, L"-");
+    m_FilterProperties.SetItemText(iItem, MFPRP_FILTER_ACTIVE, L"---");
+    m_FilterProperties.SetItemText(iItem, MFPRP_AND_OR, L"---");
+    m_FilterProperties.SetItemText(iItem, MFPRP_FIELD, L"---");
+    cs_temp = L"---  " + cs_attachment + L"  ---";
+    m_FilterProperties.SetItemText(iItem, MFPRP_CRITERIA_TEXT, cs_temp);
+  }
+
+  n = 0;
+  for (Flt_iter = pfilters->vAfldata.begin();
+       Flt_iter != pfilters->vAfldata.end(); Flt_iter++) {
+    st_FilterRow &st_fldata = *Flt_iter;
+    i++;
+    n++;
+
+    cs_num.Format(L"%d", n);
+    cs_ftype = GetFieldTypeName(st_fldata.ftype);
+    cs_criteria = PWSFilters::GetFilterDescription(st_fldata).c_str();
+    cs_act.LoadString(st_fldata.bFilterActive ? IDS_YES : IDS_NO);
+    cs_act.Remove(L'&');  // Remove leading ampersand from Yes/No
+    if (Flt_iter != pfilters->vAfldata.begin())
+      cs_ltype.LoadString(st_fldata.ltype == LC_AND ? IDSC_AND : IDSC_OR);
+    else
+      cs_ltype = L"";
+
+    iItem = m_FilterProperties.InsertItem(i /* MFPRP_FILTER_NUMBER */, cs_num);
+    m_FilterProperties.SetItemText(iItem, MFPRP_FILTER_ACTIVE, cs_act);
+    m_FilterProperties.SetItemText(iItem, MFPRP_AND_OR, cs_ltype);
+    m_FilterProperties.SetItemText(iItem, MFPRP_FIELD, cs_ftype);
+    m_FilterProperties.SetItemText(iItem, MFPRP_CRITERIA_TEXT, cs_criteria);
+  }
+
   bool bSave = m_FPROPHeader.GetStopChangeFlag();
   m_FPROPHeader.SetStopChangeFlag(false);
   for (int j = 0; j < MFPRP_CRITERIA_TEXT; j++) {
@@ -760,7 +823,7 @@ void CManageFiltersDlg::DisplayFilterProperties(st_filters *pfilters)
     int iw1 =  m_FilterProperties.GetColumnWidth(j);
     m_FilterProperties.SetColumnWidth(j, LVSCW_AUTOSIZE_USEHEADER);
     int iw2 =  m_FilterProperties.GetColumnWidth(j);
-    m_FilterProperties.SetColumnWidth(j, max(iw1, iw2));
+    m_FilterProperties.SetColumnWidth(j, std::max(iw1, iw2));
   }
   m_FilterProperties.SetColumnWidth(MFPRP_CRITERIA_TEXT, LVSCW_AUTOSIZE_USEHEADER);
   m_FilterProperties.SetRedraw(TRUE);
@@ -786,8 +849,8 @@ void CManageFiltersDlg::UpdateFilterList()
   i = 0;
   m_selectedfilter = -1;
 
-  for (mf_iter = m_MapFilters.begin();
-       mf_iter != m_MapFilters.end();
+  for (mf_iter = m_MapMFDFilters.begin();
+       mf_iter != m_MapMFDFilters.end();
        mf_iter++) {
     m_vcs_filters.push_back(mf_iter->first);
 
@@ -847,13 +910,13 @@ void CManageFiltersDlg::ResetColumns()
   iw1 = m_FilterLC.GetColumnWidth(MFLC_FILTER_NAME) + 6;
   m_FilterLC.SetColumnWidth(MFLC_FILTER_NAME, LVSCW_AUTOSIZE_USEHEADER);
   iw2 = m_FilterLC.GetColumnWidth(MFLC_FILTER_NAME);
-  m_FilterLC.SetColumnWidth(MFLC_FILTER_NAME, max(iw1, iw2));
+  m_FilterLC.SetColumnWidth(MFLC_FILTER_NAME, std::max(iw1, iw2));
 
   m_FilterLC.SetColumnWidth(MFLC_FILTER_SOURCE, LVSCW_AUTOSIZE);
   iw1 = m_FilterLC.GetColumnWidth(MFLC_FILTER_SOURCE);
   m_FilterLC.SetColumnWidth(MFLC_FILTER_SOURCE, LVSCW_AUTOSIZE_USEHEADER);
   iw2 = m_FilterLC.GetColumnWidth(MFLC_FILTER_SOURCE);
-  m_FilterLC.SetColumnWidth(MFLC_FILTER_SOURCE, max(iw1, iw2));
+  m_FilterLC.SetColumnWidth(MFLC_FILTER_SOURCE, std::max(iw1, iw2));
 
   m_FilterLC.SetColumnWidth(MFLC_INUSE, LVSCW_AUTOSIZE_USEHEADER);
   m_FilterLC.SetColumnWidth(MFLC_COPYTODATABASE, LVSCW_AUTOSIZE_USEHEADER);
@@ -897,7 +960,7 @@ void CManageFiltersDlg::OnItemChanging(NMHDR *pNotifyStruct, LRESULT *pLResult)
 
 void CManageFiltersDlg::OnCustomDraw(NMHDR *pNotifyStruct, LRESULT *pLResult)
 {
-  NMLVCUSTOMDRAW* pLVCD = reinterpret_cast<NMLVCUSTOMDRAW *>(pNotifyStruct);
+  NMLVCUSTOMDRAW *pLVCD = reinterpret_cast<NMLVCUSTOMDRAW *>(pNotifyStruct);
 
   *pLResult = CDRF_DODEFAULT;
   const int iItem = (int)pLVCD->nmcd.dwItemSpec;
@@ -968,7 +1031,7 @@ void CManageFiltersDlg::OnCustomDraw(NMHDR *pNotifyStruct, LRESULT *pLResult)
             iy = inner_rect.CenterPoint().y;
             // The '7' below is ~ half the bitmap size of 13.
             inner_rect.SetRect(ix - 7, iy - 7, ix + 7, iy + 7);
-            DrawImage(pDC, inner_rect, bActive ? 0 : 2);
+            DrawImage(pDC, inner_rect, bActive ? CHECKED : EMPTY);
             *pLResult = CDRF_SKIPDEFAULT;
             break;
           case MFLC_COPYTODATABASE:
@@ -987,7 +1050,13 @@ void CManageFiltersDlg::OnCustomDraw(NMHDR *pNotifyStruct, LRESULT *pLResult)
             iy = inner_rect.CenterPoint().y;
             // The '7' below is ~ half the bitmap size of 13.
             inner_rect.SetRect(ix - 7, iy - 7, ix + 7, iy + 7);
-            DrawImage(pDC, inner_rect, bCopy ? 0 : 2);
+            // Set image according to DB being R-O or not
+            CheckImage nImage;
+            if (m_bDBReadOnly)
+              nImage = bCopy ? CHECKED_DISABLED : EMPTY_DISABLED;
+            else
+              nImage = bCopy ? CHECKED : EMPTY;
+            DrawImage(pDC, inner_rect, nImage);
             break;
           case MFLC_EXPORT:
             // Make text 'invisible'
@@ -1001,7 +1070,7 @@ void CManageFiltersDlg::OnCustomDraw(NMHDR *pNotifyStruct, LRESULT *pLResult)
             iy = inner_rect.CenterPoint().y;
             // The '7' below is ~ half the bitmap size of 13.
             inner_rect.SetRect(ix - 7, iy - 7, ix + 7, iy + 7);
-            DrawImage(pDC, inner_rect, bExport ? 0 : 2);
+            DrawImage(pDC, inner_rect, bExport ? CHECKED : EMPTY);
             *pLResult = CDRF_SKIPDEFAULT;
             break;
           default:
@@ -1014,7 +1083,7 @@ void CManageFiltersDlg::OnCustomDraw(NMHDR *pNotifyStruct, LRESULT *pLResult)
   }
 }
 
-void CManageFiltersDlg::DrawImage(CDC *pDC, CRect &rect, int nImage)
+void CManageFiltersDlg::DrawImage(CDC *pDC, CRect &rect, CheckImage nImage)
 {
   // Draw check image in given rectangle
   if (rect.IsRectEmpty() || nImage < 0) {
@@ -1101,12 +1170,14 @@ CString CManageFiltersDlg::GetFieldTypeName(FieldType ft)
   case FT_ENTRYTYPE:     nID = IDS_ENTRYTYPE; break;
   case FT_ENTRYSTATUS:   nID = IDS_ENTRYSTATUS; break;
   case FT_UNKNOWNFIELDS: nID = IDS_UNKNOWNFIELDSFILTER; break;
+
   case HT_PRESENT:       nID = IDS_PRESENT; break;
   case HT_ACTIVE:        nID = IDS_HACTIVE; break;
   case HT_NUM:           nID = IDS_HNUM; break;
   case HT_MAX:           nID = IDS_HMAX; break;
   case HT_CHANGEDATE:    nID = IDS_HDATE; break;
   case HT_PASSWORDS:     nID = HT_PASSWORDS; break;
+
   case PT_PRESENT:       nID = IDS_PRESENT; break;
   case PT_LENGTH:        nID = IDSC_PLENGTH; break;
   case PT_LOWERCASE:     nID = IDS_PLOWER; break;
@@ -1116,9 +1187,22 @@ CString CManageFiltersDlg::GetFieldTypeName(FieldType ft)
   case PT_HEXADECIMAL:   nID = IDSC_PHEXADECIMAL; break;
   case PT_EASYVISION:    nID = IDSC_PEASYVISION; break;
   case PT_PRONOUNCEABLE: nID = IDSC_PPRONOUNCEABLE; break;
+  
+    case AT_PRESENT:       nID = IDS_PRESENT; break;
+    case AT_TITLE:         nID = IDS_FILETITLE; break;
+    case AT_CTIME:         nID = IDS_FILENAME; break;
+    case AT_MEDIATYPE:     nID = IDS_FILEMEDIATYPE; break;
+    case AT_FILENAME:      nID = IDS_FILENAME; break;
+    case AT_FILEPATH:      nID = IDS_FILEPATH; break;
+    case AT_FILECTIME:     nID = IDS_FILECTIME; break;
+    case AT_FILEMTIME:     nID = IDS_FILEMTIME; break;
+    case AT_FILEATIME:     nID = IDS_FILEATIME; break;
+    case FT_ATTACHMENT:    nID = IDS_ATTACHMENTS; break;
+
     default:
       ASSERT(0);
   }
+
   CString retval;
   retval.LoadString(nID);
   retval.TrimRight(L'\t'); // ?? do we still need to trim ??
@@ -1223,7 +1307,7 @@ void CManageFiltersDlg::OnColumnClick(NMHDR *pNotifyStruct, LRESULT *pLResult)
   if (m_iSortColumn == iIndex) {
     m_bSortAscending = !m_bSortAscending;
   } else {
-    // Turn off all previous sort arrrows
+    // Turn off all previous sort arrows
     CHeaderCtrl *phctrl = m_FilterLC.GetHeaderCtrl();
     for (int i = 0; i < phctrl->GetItemCount(); i++) {
       phctrl->GetItem(i, &hdi);
@@ -1259,7 +1343,6 @@ void CManageFiltersDlg::SortFilterView()
   phctrl->SetItem(m_iSortColumn, &hdi);
 }
 
-
 BOOL CManageFiltersDlg::PreTranslateMessage(MSG* pMsg)
 {
   if (pMsg->message == WM_KEYDOWN && pMsg->wParam == VK_F1) {
@@ -1274,4 +1357,3 @@ void CManageFiltersDlg::OnHelp()
 {
   ShowHelp(L"::/html/filters.html#ManagingFilters");
 }
-
