@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2015 Rony Shapiro <ronys@users.sourceforge.net>.
+* Copyright (c) 2003-2016 Rony Shapiro <ronys@pwsafe.org>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -17,20 +17,24 @@
 #include <vector>
 
 // Subitem indices for list of filters
-#define MFLC_FILTER_NAME    0
-#define MFLC_FILTER_SOURCE  1
-#define MFLC_INUSE          2
-#define MFLC_COPYTODATABASE 3
-#define MFLC_EXPORT         4
-#define MFLC_NUM_COLUMNS    5
+enum {
+  MFLC_FILTER_NAME = 0,
+  MFLC_FILTER_SOURCE,
+  MFLC_INUSE,
+  MFLC_COPYTODATABASE,
+  MFLC_EXPORT,
+  MFLC_NUM_COLUMNS
+};
 
 // Subitem indices for filter properties
-#define MFPRP_FILTER_NUMBER 0
-#define MFPRP_FILTER_ACTIVE 1
-#define MFPRP_AND_OR        2
-#define MFPRP_FIELD         3
-#define MFPRP_CRITERIA_TEXT 4
-#define MFPRP_NUM_COLUMNS   5
+enum {
+  MFPRP_FILTER_NUMBER = 0,
+  MFPRP_FILTER_ACTIVE,
+  MFPRP_AND_OR,
+  MFPRP_FIELD,
+  MFPRP_CRITERIA_TEXT,
+  MFPRP_NUM_COLUMNS
+};
 
 // Filter Flags
 enum {MFLT_SELECTED           = 0x8000,
@@ -52,7 +56,8 @@ class CManageFiltersDlg : public CPWDialog
 public:
   CManageFiltersDlg(CWnd* pParent,
                  bool bFilterActive,
-                 PWSFilters &pmapFilters);
+                 PWSFilters &pmapFilters,
+                 bool bCanHaveAttachments);
   virtual ~CManageFiltersDlg();
   void SetCurrentData(FilterPool activefilterpool, CString activefiltername)
   {m_activefilterpool = activefilterpool;
@@ -62,6 +67,8 @@ public:
 // Dialog Data
   enum { IDD = IDD_MANAGEFILTERS };
 
+  enum CheckImage { CHECKED = 0, CHECKED_DISABLED, EMPTY, EMPTY_DISABLED };
+
 protected:
   virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
 
@@ -86,7 +93,8 @@ protected:
   DECLARE_MESSAGE_MAP()
 
 private:
-  PWSFilters &m_MapFilters;
+  // This dialog's filter map - so as not to confuse with DboxMain and PWScore
+  PWSFilters &m_MapMFDFilters;
   std::vector<st_Filterkey> m_vcs_filters;
 
   CString GetFieldTypeName(FieldType ft);
@@ -95,14 +103,13 @@ private:
   void DisplayFilterProperties(st_filters *pfilter);
   void UpdateFilterList();
   void ResetColumns();
-  void DrawImage(CDC *pDC, CRect &rect, int nImage);
+  void DrawImage(CDC *pDC, CRect &rect, CheckImage nImage);
   void SortFilterView();
   static int CALLBACK FLTCompareFunc(LPARAM lParam1, LPARAM lParam2, 
                                      LPARAM pSelf);
   static CString GetFilterPoolName(FilterPool fp);
 
   CListCtrl m_FilterLC, m_FilterProperties;
-  CStatusBar m_statusBar;
   CImageList *m_pImageList, *m_pCheckImageList;
   CPWHdrCtrlNoChng m_FLCHeader;
   CPWHdrCtrlNoChng m_FPROPHeader;
@@ -113,4 +120,8 @@ private:
   int m_num_to_export, m_num_to_copy;
   bool m_bFilterActive, m_bDBFiltersChanged;
   int m_iSortColumn, m_bSortAscending;
+
+  bool m_bCanHaveAttachments;
+  bool m_bDBReadOnly;
+  std::set<StringX> m_sMediaTypes;
 };
