@@ -35,7 +35,6 @@
 #define IDD_PS_SYSTEM                   143
 #define IDD_QUERYSETDEF                 144
 #define IDD_SECURECLEAR                 145
-#define IDD_TRYAGAIN                    146
 #define IDD_PWFONTDIALOG                148
 #define IDD_SAMPLETEXT                  149
 #define IDD_SETFILTERS                  150
@@ -54,6 +53,7 @@
 #define IDD_FILTER_ENTRYSTATUS          163
 #define IDD_DISPLAYFSBKUPFILES          164
 #define IDD_FILTER_SIZE                 165
+#define IDD_FILTER_MEDIATYPES           166
 #define IDR_POPRESETCOLUMNWIDTH         168
 #define IDR_POPFIND                     169
 #define IDR_POPCOLUMNS                  170
@@ -90,6 +90,7 @@
 #define IDI_UNLOCKEDICON                209
 #define IDR_POPLANGUAGES                210
 #define IDR_POPCOPYALLTOORIGINAL        211
+#define IDR_POPLISTENTRIES              212
 #define IDB_CLOGO                       220
 #define IDB_CLOGO_SMALL                 221
 #define IDB_PSLOGO                      223
@@ -269,6 +270,7 @@
 #define IDB_UNCHECKED                   427
 #define IDB_EMPTY                       428
 #define IDB_BLANK                       429
+#define IDB_CHECKED_DISABLED            430
 #define IDB_DRAGGROUP_NEW               431
 #define IDB_DRAGTITLE_NEW               432
 #define IDB_DRAGUSER_NEW                433
@@ -307,6 +309,7 @@
 #define IDD_ADDEDIT_ADDITIONAL          466
 #define IDD_ADDEDIT_DATETIMES           467
 #define IDD_ADDEDIT_PASSWORDPOLICY      468
+#define IDB_UNCHECKED_DISABLED          469
 #define IDB_EXCLAMATION                 470
 #define IDB_CHANGEVKBDFONTMENU_CLASSIC  471
 #define IDB_CHANGEVKBDFONTMENU_NEW      472
@@ -316,7 +319,6 @@
 #define IDB_DRAGEMAIL_CLASSIC           476
 #define IDB_DRAGEMAILX_CLASSIC          477
 #define IDB_UNDO_CLASSIC                478
-#define IDD_ALIAS_PSWDCHANGE            479
 #define IDB_REDO_CLASSIC                480
 #define IDB_UNDO_NEW                    481
 #define IDB_REDO_NEW                    482
@@ -362,15 +364,35 @@
 #define IDD_SHOW_COMPARE                522
 #define IDD_COMPARE_WITH                523
 #define IDB_EMPTY_GROUP                 524
-
 #define IDB_YUBI_LOGO                   525
 #define IDB_YUBI_LOGO_DIS               526
 #define IDD_YUBIKEY                     527
-
+#define IDB_EXPORTDB_CLASSIC            528
+#define IDB_EXPORTDB_NEW                529
+#define IDB_EXPORTDB_NEW_D              530
+#define IDB_EXPORTV1_CLASSIC            531
+#define IDB_EXPORTV1_NEW                532
+#define IDB_EXPORTV1_NEW_D              533
+#define IDB_EXPORTV2_CLASSIC            534
+#define IDB_EXPORTV2_NEW                535
+#define IDB_EXPORTV2_NEW_D              536
+#define IDD_WZINPUTDB                   537
+#define IDD_ADDEDIT_ATT                 538
+#define IDD_ADDEDIT_ATT_SHORT           539
+#define IDB_EXPORTV3_CLASSIC            540
+#define IDB_EXPORTV3_NEW                541
+#define IDB_EXPORTV3_NEW_D              542
+#define IDB_EXPORTV4_CLASSIC            543
+#define IDB_EXPORTV4_NEW                544
+#define IDB_EXPORTV4_NEW_D              545
+#define IDD_NPWP_LIST_ENTRIES           546
+#define IDB_QUESTIONMARK                547
+#define IDD_VIEWATTACHMENT              548
+#define IDB_EMPTY_DISABLED              549
+#define IDB_REDEXCLAMATION              550
 #define IDC_APPVERSION                  1000
 #define IDC_VERSION                     1001
 #define IDC_APPVERSION2                 1002
-#define IDC_APPBUILTON                  1020
 #define IDC_CREATE_DB                   1003
 #define IDC_EXIT                        1004
 #define IDC_PASSWORD                    1005
@@ -387,10 +409,8 @@
 #define IDC_DELITEM                     1017
 #define IDC_VERIFY                      1018
 #define IDC_CONFIRMDELETE               1019
+#define IDC_APPBUILTON                  1020
 #define IDC_CONFIRMCOPY                 1021
-#define IDC_TRYAGAIN                    1022
-#define IDC_QUIT                        1023
-#define IDC_OPEN                        1024
 #define IDC_GENERATEPASSWORD            1025
 #define IDC_LOCKONMINIMIZE              1026
 #define IDC_DEFPWSHOWINEDIT             1028
@@ -401,8 +421,9 @@
 #define IDC_CRYPTKEY2                   1033
 #define IDC_DEFUSERNAME                 1034
 #define IDC_DEFPWLENGTH                 1036
-#define IDC_MESSAGE                     1037
+#define IDC_DEFAULT_USERNAME            1037
 #define IDC_QUERYCHECK                  1038
+#define IDC_SELECTED_DATABASE           1039
 #define IDC_NEW                         1041
 #define IDC_STATIC_LOGO                 1042
 #define IDC_ALWAYSONTOP                 1043
@@ -522,6 +543,7 @@
 #define IDC_SAVEDBY                     1195
 #define IDC_SAVEDON                     1196
 #define IDC_SAVEDAPP                    1197
+#define IDC_NUMATTACHMENTS              1198
 #define IDC_BACKUPBEFORESAVE            1199
 #define IDC_DFLTBACKUPPREFIX            1200
 #define IDC_USERBACKUPPREFIX            1201
@@ -558,7 +580,6 @@
 #define IDC_DEFENABLEGRIDLINES          1246
 #define IDC_LIST_HEADER                 1247
 #define IDC_CPLIST                      1248
-#define IDC_DB_DEFAULTAUTOTYPE          1249
 #define IDC_MINIMIZEONAUTOTYPE          1250
 #define IDC_ALTBROWSER_CMDLINE          1251
 #define IDC_DEFUNSHOWINTREE             1254
@@ -718,7 +739,6 @@
 #define IDC_DEFPWHIDESYSTRAY            1426
 #define IDC_ADVANCED_RESET              1427
 #define IDC_ENTRY                       1428
-#define IDC_STATIC_PROTECTED            1429
 #define IDC_DATABASE                    1430
 #define IDC_VIEWREPORT                  1431
 #define IDC_STATIC_WZWARNING            1432
@@ -727,10 +747,7 @@
 #define IDC_STATIC_WZFILE               1435
 #define IDC_STATIC_WZPROCESSING         1436
 #define IDC_STATIC_WZRESULTS            1437
-#define IDC_USEDEFAULTSYMBOLS           1438
-#define IDC_USEOWNSYMBOLS               1439
 #define IDC_OWNSYMBOLS                  1440
-#define IDC_STATIC_DEFAULTSYMBOLS       1441
 #define IDC_DEFAULTAUTOTYPE             1442
 #define IDC_STATIC_DRAGAUTO             1443
 #define IDC_COPYPSWDURL                 1444
@@ -742,7 +759,6 @@
 #define IDC_STATIC_PREFERENCES          1451
 #define IDC_YUBIKEY_BTN                 1452
 #define IDC_YUBIKEY_STATUS              1453
-#define IDC_YUBIKEY_BTN2                1454
 #define IDC_YUBIKEY2_BTN                1455
 #define IDC_YUBI_PROGRESS               1456
 #define IDC_YUBI_STATUS                 1457
@@ -756,9 +772,9 @@
 #define IDC_YUBI_SN                     1465
 #define IDC_YUBI_SK                     1466
 #define IDC_YUBI_GEN_BN                 1467
-#define IDC_BUTTON2                     1468
+#define IDC_ATT_REMOVE                  1468
 #define IDC_YUBI_API                    1469
-#define IDC_BUTTON1                     1470
+#define IDC_RESET_SYMBOLS               1470
 #define IDC_YUBI_SHOW_HIDE              1471
 #define IDC_EDIT                        1472
 #define IDC_LIST_POLICYENTRIES          1473
@@ -787,15 +803,75 @@
 #define IDC_STATIC_HASHITER             1496
 #define IDC_STATIC_HASHITER_MIN         1497
 #define IDC_STATIC_HASHITER_MAX         1498
+#define IDC_DB_DEF_AUTOTYPE_DELAY       1500
+#define IDC_DB_DEF_AUTOTYPE_TEXT        1501
+#define IDC_DADSPIN                     1502
+#define IDC_CAPSLOCK                    1503
+#define IDC_STATIC_NEWCOMBI             1504
+#define IDC_STATIC_VERIFY               1505
+#define IDC_STATIC_COMBI                1506
+#define IDC_PASSKEY2                    1507
+#define IDC_VERIFY2                     1508
+#define IDC_VKB2                        1509
+#define IDC_MYBASE                      1510
+#define IDC_ATT_IMAGE                   1511
+#define IDC_ATT_NAME                    1512
+#define IDC_ATT_FILE                    1513
+#define IDC_ATT_ADD                     1514
+#define IDC_ATT_EXPORT                  1515
+#define IDC_ATT_IMPORT                  1516
+#define IDC_STATIC_NOPREVIEW            1517
+#define IDC_FILESIZE                    1518
+#define IDC_FILECTIME                   1519
+#define IDC_FILEMTIME                   1520
+#define IDC_FILEMTYPE                   1521
+#define IDC_AFILTER_FLD_COMBOBOX        1522
+#define IDC_AFILTER_LGC_COMBOBOX        1523
+#define IDC_CMB_AVALIABLEMEDIATYPES     1524
+#define IDC_STATIC_TUTORIAL             1525
+#define IDC_NPWP_LIST_ENTRIES           1526
+#define IDC_BACKUPBEFORESAVEHELP        1527
+#define IDC_USERBACKUPOTHERLOCATIONHELP 1528
+#define IDC_SAVEIMMEDIATELYHELP         1529
+#define IDC_MAINTAINDATETIMESTAMPSHELP  1530
+#define IDC_OTHERBROWSERLOCATIONHELP    1531
+#define IDC_OTHEREDITORLOCATIONHELP     1532
+#define IDC_RESETPWHISTORYOFFHELP       1533
+#define IDC_RESETPWHISTORYONHELP        1534
+#define IDC_SETMAXPWHISTORYHELP         1535
+#define IDC_CLEARPWHISTORYHELP          1536
+#define IDC_SHORTCUTLISTHELP            1537
+#define IDC_ENTSHORTCUTLISTHELP         1538
+#define IDC_REGDELHELP                  1539
+#define IDC_MIGRATETOAPPDATAHELP        1541
+#define IDC_STATIC_INITIALTREEVIEW      1542
+#define IDC_STATIC_UNLOCKDIFFICULTY     1543
+#define IDC_STATIC_MANAGEPWH            1544
+#define IDC_LOCKONMINIMIZEHELP          1545
+#define IDC_LOCKONWORKSTATIONLOCKHELP   1546
+#define IDC_LOCKONIDLEHELP              1547
+#define IDC_ZOOM                        1548
+#define IDC_ZOOM_VALUE                  1549
+#define IDC_IMAGE_HSCROLL               1550
+#define IDC_IMAGE_VSCROLL               1551
+#define IDC_EXPORTFILTERS               1552
+#define IDC_EXPANDEDUSERBACKUPOTHRLOC   1553
+#define IDC_USERBACKUPOTHERLOCATIONHELP2 1554
+#define IDC_SMARTLABELHELP              1555
+#define IDC_PASSWORDHELP                1556
+#define IDC_NOTESHELP                   1557
+#define IDC_AUTOTYPEHELP                1558
+#define IDC_PASSWORDHELP2               1559
+#define IDC_PWHHELP                     1560
+#define IDC_STATIC_HOSTUSER             1561
 
 // Next default values for new objects
 // 
 #ifdef APSTUDIO_INVOKED
 #ifndef APSTUDIO_READONLY_SYMBOLS
-
-#define _APS_NEXT_RESOURCE_VALUE        528
+#define _APS_NEXT_RESOURCE_VALUE        551
 #define _APS_NEXT_COMMAND_VALUE         30001
-#define _APS_NEXT_CONTROL_VALUE         1499
-#define _APS_NEXT_SYMED_VALUE           166
+#define _APS_NEXT_CONTROL_VALUE         1562
+#define _APS_NEXT_SYMED_VALUE           540
 #endif
 #endif
