@@ -1,9 +1,5 @@
 //{{NO_DEPENDENCIES}}
 
-// Strings
-#define SUFFIX_FILTERS  L"Password Safe Databases (*.psafe3; *.dat)|*.psafe3; *.dat|"
-#define SUFFIX3_FILTERS L"Password Safe Databases (*.psafe3)|*.psafe3|"
-
 //  Usage
 #define IDS_USAGE                       5000
 
@@ -11,22 +7,11 @@
 #define IDS_ALWAYSONTOP                 5010
 
 // Status bar
-#define IDS_STATCOMPANY                 5020
-#define IDS_STATAUTOTYPE                5021
-#define IDS_STATBROWSE                  5022
-#define IDS_STATCOPYNOTES               5023
-#define IDS_STATCOPYPASSWORD            5024
-#define IDS_STATCOPYUSERNAME            5025
-#define IDS_STATVIEWEDIT                5026
-#define IDS_STATCOPYPASSWORDMIN         5027
 #define IDS_READ_ONLY                   5028
 #define IDS_BLANK                       5029
-#define IDS_STATBROWSEPLUS              5030
-#define IDS_STATRUN                     5031
-#define IDS_STATSENDEMAIL               5032
 #define IDS_READ_WRITE                  5033
 
-// File Dailog filters
+// File Dialog filters
 #define IDS_FDF_V3_ALL                  5050
 #define IDS_FDF_DB_ALL                  5051
 #define IDS_FDF_DB_BU_ALL               5052
@@ -36,6 +21,12 @@
 #define IDS_FDF_BU                      5056
 #define IDS_FDF_BUS                     5057
 #define IDS_FDF_PR_ALL                  5058
+#define IDS_FDF_TXT_ALL                 5059
+#define IDS_FDF_CSV_ALL                 5060
+#define IDS_FDF_V12_ALL                 5061
+#define IDS_FDF_V4_ALL                  5062
+#define IDS_FDF_ALL                     5063
+#define IDS_FDF_FILES                   5064
 
 // Tooltips
 #define IDS_RESETPWHISTORYOFF           5100
@@ -48,11 +39,18 @@
 #define IDS_OTHERBROWSERLOCATION        5108
 #define IDS_OTHEREDITORLOCATION         5109
 #define IDS_MIGRATETOAPPDATA            5110
+#define IDS_CLEARPWHISTORY              5111
+#define IDS_SAVEIMMEDIATELY             5112
+#define IDS_DBLOCK                      5113
+#define IDS_USERBACKUPOTHERLOCATION2    5114
+#define IDS_SMARTLABELHELP              5115
+#define IDS_PASSWORDHELP                5116
+#define IDS_NOTESHELP                   5117
+#define IDS_AUTOTYPEHELP                5118
+#define IDS_PWHHELP                     5119
+#define IDS_PASSWORDHELP2               5120
 
 // Program strings
-#define IDS_LITTLEENDIAN                5207
-#define IDS_BIGENDIAN                   5208
-#define IDS_ENDIANERROR                 5209
 #define IDS_FILEERROR1                  5210
 #define IDS_FILEERROR2                  5211
 #define IDS_ERRORMESSAGE                5212
@@ -85,10 +83,8 @@
 #define IDS_NUMITEMS                    5240
 #define IDS_NEVER                       5241
 #define IDS_HIDDENNOTES                 5242
-#define IDS_SHOWPASSWORDTXT1            5243
-#define IDS_HIDEPASSWORDTXT1            5244
-#define IDS_SHOWPASSWORDTXT2            5245
-#define IDS_HIDEPASSWORDTXT2            5246
+#define IDS_SHOWPASSWORDTXT             5245
+#define IDS_HIDEPASSWORDTXT             5246
 #define IDS_MUSTHAVETITLE               5247
 #define IDS_MUSTHAVEPASSWORD            5248
 #define IDS_DOTINVALID                  5249
@@ -102,7 +98,6 @@
 #define IDS_NOSAFE                      5259
 #define IDS_COPYURL                     5260
 #define IDS_VIEWENTRY2                  5261
-#define IDS_SHORTCUTROEXPLANATION       5262
 #define IDS_UNKNOWN                     5263
 #define IDS_VIEWENTRY                   5264
 #define IDS_ON                          5265
@@ -162,7 +157,6 @@
 #define IDS_FILEOPENERROR               5319
 #define IDS_CANTOPENREADING             5320
 #define IDS_UNKNOWNERROR                5321
-#define IDS_NEWFORMAT                   5322
 #define IDS_NEWFORMAT2                  5323
 #define IDS_VERSIONWARNING              5324
 #define IDS_CANTOPENWRITING             5325
@@ -213,7 +207,7 @@
 #define IDS_PICKBACKUP                  5375
 #define IDS_PICKRESTORE                 5376
 #define IDS_UNTITLEDRESTORE             5377
-#define IDS_VALIDATEOK                  5378
+#define IDS_VALIDATE_OK                 5378
 #define IDS_FAILEDSAVEPREF              5379
 #define IDS_FAILEDSAVEPREFRO            5380
 #define IDS_ENTRIESCHANGEDSTOP          5381
@@ -231,10 +225,6 @@
 #define IDS_OPTBACKUPTITLE              5393
 #define IDS_DEFDBNAME                   5394
 #define IDS_CREATENAME                  5395
-#define IDS_DEMO                        5396
-#define IDS_LIMIT_TITLE                 5397
-#define IDS_LIMIT_MSG                   5398
-#define IDS_LIMIT_MSG2                  5399
 #define IDS_MENUEXPCOLGROUP             5400
 #define IDS_MENUEDITENTRY               5401
 #define IDS_MENUVIEWENTRY               5402
@@ -242,7 +232,6 @@
 #define IDS_MENURENAMEGROUP             5406
 #define IDS_MAXNUMDAYSEXCEEDED          5407
 #define IDS_HELPMENU                    5408
-#define IDS_U3PURCHASE                  5409
 #define IDS_INVALIDEXPIRYWARNDAYS       5410
 #define IDS_DRAGNUMBER                  5411
 #define IDS_STRICT_LOCKED               5412
@@ -271,7 +260,6 @@
 #define IDS_ADVANCED                    5435
 #define IDS_YES                         5437
 #define IDS_NO                          5438
-#define IDS_XMLUNKNFLDIGNORED           5439
 #define IDS_XMLIMPORTFAILED             5440
 #define IDS_COPYUNKNOWNFIELDS           5441
 #define IDS_FINDX                       5442
@@ -321,6 +309,7 @@
 #define IDS_NOREPORTSEXIST              5488
 #define IDS_VISIT_WEBSITE               5489
 #define IDS_ICON                        5496
+#define IDS_ABASE_MISSING               5497
 #define IDS_ABASEINVALID                5499
 #define IDS_SBASEINVALID                5500
 #define IDS_MUSTHAVETARGET              5501
@@ -339,9 +328,7 @@
 #define IDS_EMPTYSAMPLETEXT             5514
 #define IDS_MENUCOPYEMAIL               5516
 #define IDS_COPYEMAIL                   5517
-#define IDS_SHORTCUTEXPLANATION         5519
 #define IDS_SCTARGET                    5520
-#define IDS_SAVEBEFOREEXPORT            5521
 #define IDS_ADVANCEDOPTIONS             5522
 #define IDS_ADVANCEDSUBSET              5523
 #define IDS_ADVCASE_INSENSITIVE         5524
@@ -373,7 +360,6 @@
 #define IDS_READWRITE                   5551
 #define IDS_SEARCH                      5552
 #define IDS_NEW                         5553
-#define IDS_IN_N_DAYS                   5554
 #define IDS_OVERRIDE_POLICY             5555
 #define IDS_PASSWORDEXPIRYDATEINT       5556
 #define IDS_NOFIELDSFORFILTER           5557
@@ -400,14 +386,10 @@
 #define IDS_HMAX                        5578
 #define IDS_HDATE                       5579
 #define IDS_HPSWD                       5580
-#define IDS_PLENGTH                     5581
 #define IDS_PLOWER                      5582
 #define IDS_PUPPER                      5583
 #define IDS_PDIGITS                     5584
 #define IDS_PSYMBOL                     5585
-#define IDS_PEASYVISION                 5586
-#define IDS_PPRONOUNCEABLE              5587
-#define IDS_PHEXADECIMAL                5588
 #define IDS_PASSWORDHISTORY             5589
 #define IDS_UNKNOWNFIELDSFILTER         5590
 #define IDS_FILTERACTIVE                5594
@@ -441,9 +423,6 @@
 #define IDS_VIEW                        5624
 #define IDS_FILTERSEXPORTEDOK           5625
 #define IDS_FILTERSIMPORTEDOK           5626
-#define IDS_SELECTFROMSTORE             5627
-#define IDS_DELETEFROMSTORE             5628
-#define IDS_VIEWFROMSTORE               5629
 #define IDS_CANTAUTOIMPORTFILTERS       5630
 #define IDS_REPLACEFILTER               5631
 #define IDS_FILTEREXISTS                5632
@@ -461,7 +440,7 @@
 #define IDS_FILTERAPPLY                 5644
 #define IDS_MINIMIZECONFLICT            5645
 #define IDS_CLICKTOCOPY                 5646
-#define IDS_SUBSETINDEXZERO             5647
+
 #define IDS_SUBSETINDEXTOOBIG           5648
 #define IDS_MENUCREATESHORTCUT          5649
 #define IDS_AREYOUSURE                  5650
@@ -494,8 +473,7 @@
 #define IDS_RUNCMDCOPIED                5678
 #define IDS_COL_MENUITEM                5680
 #define IDS_COL_SHORTCUT                5681
-#define IDS_SHCT_TOOLTIP0               5682
-#define IDS_SHCT_TOOLTIP1               5683
+#define IDS_SHCT_TOOLTIP                5682
 #define IDS_SHCT_WARNING1               5684
 #define IDS_SHCT_WARNING2               5685
 #define IDS_SHCT_WARNING3               5686
@@ -513,7 +491,6 @@
 #define IDS_ADDENTRY                    5698
 #define IDS_EDITENTRY                   5699
 #define IDS_NA                          5700
-#define IDS_ISANALIAS                   5701
 #define IDS_ISANALIASBASE               5702
 #define IDS_ISASHORTCUTBASE             5703
 #define IDS_COMPDCA                     5704
@@ -534,7 +511,7 @@
 #define IDS_TRAYSENDEMAIL               5722
 #define IDS_TRAYCOPYEMAIL               5723
 #define IDS_STOPREASON                  5724
-#define IDS_FILTER1                     5725
+#define IDS_FILTER_NAME                 5725
 #define IDS_DATETIMESTRING              5726
 #define IDS_ERROR                       5727
 #define IDS_ABORT                       5728
@@ -542,19 +519,15 @@
 #define IDS_IGNORE                      5730
 #define IDS_TRYAGAIN                    5731
 #define IDS_AREYOUSURE_OPT              5732
-#define IDS_HEADER                      5733
-#define IDS_RECORD                      5734
 #define IDS_NO_ENTRIES_SELECTED         5736
 #define IDS_NO_ENTRIES_PROCESSED        5737
 #define IDS_DCALONG                     5738
 #define IDS_ENTRYSTATUS                 5739
-#define IDS_NONSAVEDCHANGES             5740
 #define IDS_RECORDSUPDATED              5741
 #define IDS_FILEWRITEFAILURE            5742
 #define IDS_GENERATEPASSWORD            5743
 #define IDS_PASSWORDGENERATED1          5744
 #define IDS_PASSWORDGENERATED2          5745
-#define IDS_SYNCHWARNING                5746
 #define IDS_SYNCHRONIZEX                5747
 #define IDS_NOFIELDSFORSYNCH            5748
 #define IDS_RPTSYNCH                    5749
@@ -595,13 +568,10 @@
 #define IDS_TITLERENAMED                5786
 #define IDS_TITLEPWHERRORS              5787
 #define IDS_SAVEBEFOREPROCESS           5788
-#define IDS_SAVEBEFOREPROCESS2          5789
-#define IDS_SAVEBEFOREVALIDATE          5790
 #define IDS_CHOOSEDATABASEV             5791
 #define IDS_TRUNCATETEXT                5792
 #define IDS_WARNINGTEXTLENGTH           5793
 #define IDS_ENTRYSIZE                   5794
-#define IDS_INVALIDEXPIRYDATE           5795
 #define IDS_NOIBACKUP2                  5796
 #define IDS_SAVEAS                      5797
 #define IDS_INVALIDFUTUREDATE           5798
@@ -638,7 +608,6 @@
 #define IDS_EXP_SBASE                   5829
 #define IDS_EXP_SBASE_WARN              5830
 #define IDS_EXP_SBASE_EXP               5831
-#define IDS_EXPIREPASSWORDS             5832
 #define IDS_WZEXPORTTEXT                5833
 #define IDS_WZEXPORTXML                 5834
 #define IDS_WZCOMPARE                   5835
@@ -647,6 +616,181 @@
 #define IDS_NONE_EXPORTED               5838
 #define IDS_NONE_COMPARED               5839
 #define IDS_NONE_SYNCHRONIZED           5840
-#define IDS_EMBEDDEDRESOURCES           5841
+#define IDS_DEFAULTAUTOTYPE             5841
 #define IDS_SYMBOLS                     5842
 #define IDS_COMPSYMBOLS                 5843
+#define IDS_PROTECTEDENTRY              5844
+#define IDS_CHANGE_READWRITE            5845
+#define IDS_CHANGE_READONLY             5846
+#define IDS_BACKOUT_CHANGES             5847
+#define IDS_CHANGEMODE                  5848
+#define IDS_CHANGEMODE_FAILED           5849
+#define IDS_CM_FAIL_REASON1             5850
+#define IDS_CM_FAIL_REASON2             5851
+#define IDS_CM_FAIL_REASON3             5852
+#define IDS_CM_FAIL_REASON4             5853
+#define IDS_CM_FAIL_REASON5             5854
+#define IDS_SB_TT_DBCLICK               5855
+#define IDS_SB_TT_CBACTION              5856
+#define IDS_SB_TT_CONFIG                5857
+#define IDS_SB_TT_MODIFIED              5858
+#define IDS_SB_TT_MODE                  5859
+#define IDS_SB_TT_NUMENTRIES            5860
+#define IDS_SB_TT_FILTER                5861
+#define IDS_DRAGTOAUTOTYPE              5862
+#define IDS_REPLACEEXPORTFILE           5863
+#define IDS_COMPPROTECTED               5864
+#define IDS_WZACTIONFAILED              5865
+#define IDS_CANNOTCREATEDIR             5866
+#define IDS_WITHWARNINGS                5867
+#define IDS_RPTIMPORTKPV1TXT            5868
+#define IDS_RPTIMPORTKPV1CSV            5869
+#define IDS_IMPORTFAILED                5870
+#define IDS_PASSWORDLEN                 5871
+#define IDS_SHIFTDCA                    5872
+#define IDS_SHIFTDCALONG                5873
+#define IDS_COMPSHIFTDCA                5874
+#define IDS_ENTER_NEW_DB_NAME           5875
+#define IDS_ENTER_NEW_DB_DESCRIPTION    5876
+#define IDS_INPUTBOX_CHAR_COUNT         5877
+#define IDS_NUMGROUPS_E                 5878
+#define IDS_POLICYNAME                  5879
+#define IDS_NEWPOLICY                   5880
+#define IDS_EDITPOLICY                  5881
+#define IDS_DELETEPOLICY                5882
+#define IDS_LISTPOLICY                  5883
+#define IDS_UNDOPOLICY                  5884
+#define IDS_REDOPOLICY                  5885
+#define IDS_CANCELPOLICYCHANGES         5886
+#define IDS_SAVEPOLICYCHANGES           5887
+#define IDS_USECOUNT                    5888
+#define IDS_PSWDPOLICY                  5889
+#define IDS_POLICYFIELD                 5890
+#define IDS_VALUE                       5891
+#define IDS_AREYOUSURE_PN               5894
+#define IDS_INVALIDPOLICYNAME1          5895
+#define IDS_INVALIDPOLICYNAME2          5896
+#define IDS_XMLIMPORTWITHWARNINGS       5905
+#define IDS_CHANGED_POLICIES            5906
+#define IDS_DRAGPOLICY                  5907
+#define IDS_NOT_USED                    5908
+#define IDS_COMPPOLICYNAME              5909
+#define IDS_POLICY_VIEWENTRY            5910
+#define IDC_DETAILS                     5911
+#define IDS_LIST                        5912
+#define IDS_EDIT_DEFAULT_POLICY         5913
+#define IDS_ADD_NAMED_POLICY            5914
+#define IDS_EDIT_NAMED_POLICY           5915
+#define IDS_TESTPOLICY                  5916
+#define IDS_CLICKTOCOPYGENPSWD          5917
+#define IDS_PR_MUSTHAVECHARACTERS       5918
+#define IDS_OWNSYMBOLSMISSING           5919
+#define IDS_ENTRIESCLEARALL             5920
+#define IDS_NOTESZOOMIN                 5921
+#define IDS_NOTESZOOMOUT                5922
+#define IDS_VALIDATE_ISSUES             5923
+#define IDS_CURRENT_ENTRY               5924
+#define IDS_COMPARISON_ENTRY            5925
+#define IDS_PWHACTIVE                   5926
+#define IDS_PWHMAX                      5927
+#define IDS_PWHNUM                      5928
+#define IDS_PWHENTRY                    5929
+#define IDS_COPYALL                     5930
+#define IDS_SYNCHRONIZEALL              5931
+#define IDS_SYNCENTRY                   5932
+#define IDS_OPERATION_ABORTED           5933
+#define IDS_FORMAT_CMP_POLICY           5934
+#define IDS_EXPORTENTMENU               5935
+#define IDS_EXPORTENT2PLAINTEXT         5936
+#define IDS_EXPORTENT2XML               5937
+#define IDS_HELP_UNAVALIABLE            5938
+#define IDS_SUBSETINDEXTOOSMALL         5939
+#define IDS_KBSHORTCUT                  5940
+#define IDS_COMPKBSHORTCUT              5941
+#define IDS_KBS_INVALID                 5942
+#define IDS_KBS_ADD_ALT                 5943
+#define IDS_KBS_ADD_ALTSHIFT            5944
+#define IDS_KBS_ADD_CTRL                5945
+#define IDS_KBS_ADD_CTRLSHIFT           5946
+#define IDS_KBS_ADD_ALTCTRL             5947
+#define IDS_KBS_ADD_ALTCTRLSHIFT        5948
+#define IDS_KBS_CANTADD                 5949
+#define IDS_KBS_INUSEBYENTRY            5950
+#define IDS_KBS_INUSEBYMENU             5951
+#define IDS_KBS_TOOLTIP0                5952
+#define IDS_KBS_TOOLTIP1                5953
+#define IDS_KBS_SAMEASAPP               5954
+#define IDS_APPHOTKEY_OVERRIDE          5955
+#define IDS_ENTRYKBST_OVERRIDE          5956
+#define IDS_EMPTY_DB                    5957
+#define IDS_TOOMANYTRIES                5958
+#define IDS_EXPORTENT2DB                5959
+#define IDS_EXPORTGRPMENU               5960
+#define IDS_NAMEDBFILE                  5961
+#define IDS_WZEXPORTDB                  5962
+#define IDS_EXPORT_TEXTX_GROUP          5963
+#define IDS_EXPORT_XMLX_GROUP           5964
+#define IDS_RPTEXPORTDB                 5965
+#define IDS_WSSLCT_GROUP                5966
+#define IDS_EXDB                        5967
+#define IDS_EXPORTWARNING               5970
+#define IDS_NOTESFONT                   5971
+#define IDS_CANT_FIND_EXT_EDITOR        5972
+#define IDS_APP_DEFAULT                 5973
+#define IDS_ALL_IMAGE_FILES             5974
+#define IDS_NOPREVIEW_AVAILABLE         5975
+#define IDS_ATTACHMENT_NOTFOUND         5976
+#define IDS_MENUITEM_EXPORT2V3FORMAT    5977
+#define IDS_MENUITEM_EXPORT2V4FORMAT    5978
+#define IDS_ATTREF                      5979
+#define IDS_COMPATTREF                  5980
+#define IDS_FILETITLE                   5981
+#define IDS_FILENAME                    5982
+#define IDS_FILEPATH                    5983
+#define IDS_FILEMEDIATYPE               5984
+#define IDS_NUMATTACHMENTS              5985
+#define IDS_ATTACHMENTS                 5986
+#define IDS_CTIME                       5987
+#define IDS_FILECTIME                   5988
+#define IDS_FILEMTIME                   5989
+#define IDS_FILEATIME                   5990
+#define IDS_SETATTACHMENTFILTER         5991
+#define IDS_AFILTERINFO                 5992
+#define IDS_NOMEDIATYPE                 5993
+#define IDS_CLICKTOLISTENTRIES          5994
+#define IDS_NUMEMPTYGROUPS              5995
+#define IDS_ADDEDITFONT                 5996
+#define IDS_SHORTCUT_WARNING            5997
+#define IDS_NOIBACKUP3                  5998
+
+#define IDS_YUBI_CLICK_PROMPT           6000
+#define IDS_YUBI_INSERT_PROMPT          6001
+#define IDS_YUBI_TIMEOUT                6002
+#define IDS_YUBI_UNINITIALIZED          6003
+#define IDS_YUBI_INIT_FAILED            6004
+#define IDS_YUBI_INIT_SUCCESS           6005
+
+#define IDS_STATIC_DB_PREFS_RO_WARNING  6020
+#define IDS_DB_READ_ONLY                6021
+#define IDS_NO_DB                       6022
+#define IDS_SAVEDRESTOREDDB             6023
+#define IDS_UNSAVEDRESTOREDDB           6024
+#define IDS_VIEWATTACHMENT              6025
+#define IDS_IMAGE_SAVE_FAILED           6026
+#define IDS_IMAGE_LOAD_FAILED           6027
+#define IDS_IMAGE_LIMIT_1               6028
+#define IDS_IMAGE_IMPORT_FAILED         6029
+#define IDS_MEM_ALLOC_FAILED            6030
+#define IDS_MEM_LOCK_FAILED             6031
+#define IDS_CLEARMRU                    6032
+#define IDS_CONFIRMCLEARMRU             6033
+#define IDS_OPTBACKUPNODRIVE            6034
+#define IDS_EXPANDPATH                  6035
+#define IDS_CANT_EXPANDPATH             6036
+#define IDS_PWHISTORYCOPIED             6037
+#define IDS_OLDPSWDCOPIED               6038
+#define IDS_CHANGINGBASEENTRY           6039
+#define IDS_CHANGINGBASEENTRY1          6040
+#define IDS_CHANGINGBASEENTRY2          6041
+#define IDS_EXP_ALIAS                   6042
+#define IDS_HOSTUSER                    6043
