@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2015 Rony Shapiro <ronys@users.sourceforge.net>.
+ * Copyright (c) 2003-2016 Rony Shapiro <ronys@pwsafe.org>.
  * All rights reserved. Use of the code is allowed under the
  * Artistic License 2.0 terms, as specified in the LICENSE file
  * distributed with this code, or available from
@@ -51,7 +51,6 @@
 
 IMPLEMENT_CLASS( AddEditPropSheet, wxPropertySheetDialog )
 
-
 /*!
  * AddEditPropSheet event table definition
  */
@@ -65,11 +64,11 @@ BEGIN_EVENT_TABLE( AddEditPropSheet, wxPropertySheetDialog )
   EVT_BUTTON( ID_GO_BTN, AddEditPropSheet::OnGoButtonClick )
   EVT_BUTTON( ID_SEND_BTN, AddEditPropSheet::OnSendButtonClick )
   EVT_CHECKBOX( ID_CHECKBOX1, AddEditPropSheet::OnKeepHistoryClick )
-  EVT_RADIOBUTTON( ID_RADIOBUTTON, AddEditPropSheet::OnRadiobuttonSelected )
+  EVT_RADIOBUTTON( ID_RADIOBUTTON, AddEditPropSheet::OnExpRadiobuttonSelected )
   EVT_DATE_CHANGED( ID_DATECTRL, AddEditPropSheet::OnExpDateChanged )
-  EVT_RADIOBUTTON( ID_RADIOBUTTON1, AddEditPropSheet::OnRadiobuttonSelected )
+  EVT_RADIOBUTTON( ID_RADIOBUTTON1, AddEditPropSheet::OnExpRadiobuttonSelected )
   EVT_SPINCTRL( ID_SPINCTRL2, AddEditPropSheet::OnExpIntervalChanged )
-  EVT_RADIOBUTTON( ID_RADIOBUTTON4, AddEditPropSheet::OnRadiobuttonSelected )
+  EVT_RADIOBUTTON( ID_RADIOBUTTON4, AddEditPropSheet::OnExpRadiobuttonSelected )
   EVT_RADIOBUTTON( ID_RADIOBUTTON2, AddEditPropSheet::OnPWPRBSelected )
   EVT_COMBOBOX( ID_POLICYLIST, AddEditPropSheet::OnPolicylistSelected )
   EVT_RADIOBUTTON( ID_RADIOBUTTON3, AddEditPropSheet::OnPWPRBSelected )
@@ -137,7 +136,8 @@ bool AddEditPropSheet::Create( wxWindow* parent, wxWindowID id, const wxString&
   SetExtraStyle(wxWS_EX_VALIDATE_RECURSIVELY|wxWS_EX_BLOCK_EVENTS);
   wxPropertySheetDialog::Create( parent, id, caption, pos, size, style );
 
-  CreateButtons(wxOK|wxCANCEL|wxHELP);
+  int flags = (m_type == VIEW) ? (wxCLOSE|wxHELP) : (wxOK|wxCANCEL|wxHELP);
+  CreateButtons(flags);
   CreateControls();
   LayoutDialog();
   Centre();
@@ -156,7 +156,6 @@ AddEditPropSheet::~AddEditPropSheet()
 ////@end AddEditPropSheet destruction
 }
 
-
 /*!
  * Member initialisation
  */
@@ -169,6 +168,7 @@ void AddEditPropSheet::Init()
   m_BasicPanel = NULL;
   m_BasicFGSizer = NULL;
   m_groupCtrl = NULL;
+  m_UsernameCtrl = NULL;
   m_PasswordCtrl = NULL;
   m_ShowHideCtrl = NULL;
   m_Password2Ctrl = NULL;
@@ -232,6 +232,21 @@ private:
   int m_rbID;
 };
 
+static void setupDCAStrings(wxArrayString &as)
+{
+  // semi-duplicated in SetupDCAComboBoxes(),
+  // but leaving these empty now causes an assert
+  as.Add(_("Auto Type"));
+  as.Add(_("Browse"));
+  as.Add(_("Browse + Auto Type"));
+  as.Add(_("Copy Notes"));
+  as.Add(_("Copy Password"));
+  as.Add(_("Copy Password + Minimize"));
+  as.Add(_("Copy Username"));
+  as.Add(_("Edit/View Entry"));
+  as.Add(_("Execute Run command"));
+}
+
 /*!
  * Control creation for AddEditPropSheet
  */
@@ -269,8 +284,8 @@ void AddEditPropSheet::CreateControls()
   wxStaticText* itemStaticText12 = new wxStaticText( m_BasicPanel, wxID_STATIC, _("Username:"), wxDefaultPosition, wxDefaultSize, 0 );
   m_BasicFGSizer->Add(itemStaticText12, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 5);
 
-  wxTextCtrl* itemTextCtrl13 = new wxTextCtrl( m_BasicPanel, ID_TEXTCTRL1, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0 );
-  m_BasicFGSizer->Add(itemTextCtrl13, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 5);
+  m_UsernameCtrl = new wxTextCtrl( m_BasicPanel, ID_TEXTCTRL1, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0 );
+  m_BasicFGSizer->Add(m_UsernameCtrl , 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 5);
 
   m_BasicFGSizer->Add(10, 13, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL, 5);
 
@@ -359,15 +374,7 @@ void AddEditPropSheet::CreateControls()
   itemFlexGridSizer40->Add(itemStaticText45, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 5);
 
   wxArrayString m_DCAcomboBoxStrings;
-  m_DCAcomboBoxStrings.Add(_("Auto Type"));
-  m_DCAcomboBoxStrings.Add(_("Browse"));
-  m_DCAcomboBoxStrings.Add(_("Browse + Auto Type"));
-  m_DCAcomboBoxStrings.Add(_("Copy Notes"));
-  m_DCAcomboBoxStrings.Add(_("Copy Password"));
-  m_DCAcomboBoxStrings.Add(_("Copy Password + Minimize"));
-  m_DCAcomboBoxStrings.Add(_("Copy Username"));
-  m_DCAcomboBoxStrings.Add(_("Edit/View Entry"));
-  m_DCAcomboBoxStrings.Add(_("Execute Run command"));
+  setupDCAStrings(m_DCAcomboBoxStrings);
   m_DCAcomboBox = new wxComboBox( itemPanel38, ID_COMBOBOX, wxEmptyString, wxDefaultPosition, wxDefaultSize, m_DCAcomboBoxStrings, wxCB_READONLY );
   itemFlexGridSizer40->Add(m_DCAcomboBox, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL, 5);
 
@@ -375,15 +382,7 @@ void AddEditPropSheet::CreateControls()
   itemFlexGridSizer40->Add(itemStaticText47, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 5);
 
   wxArrayString m_SDCAcomboBoxStrings;
-  m_SDCAcomboBoxStrings.Add(_("Auto Type"));
-  m_SDCAcomboBoxStrings.Add(_("Browse"));
-  m_SDCAcomboBoxStrings.Add(_("Browse + Auto Type"));
-  m_SDCAcomboBoxStrings.Add(_("Copy Notes"));
-  m_SDCAcomboBoxStrings.Add(_("Copy Password"));
-  m_SDCAcomboBoxStrings.Add(_("Copy Password + Minimize"));
-  m_SDCAcomboBoxStrings.Add(_("Copy Username"));
-  m_SDCAcomboBoxStrings.Add(_("Edit/View Entry"));
-  m_SDCAcomboBoxStrings.Add(_("Execute Run command"));
+  setupDCAStrings(m_SDCAcomboBoxStrings);
   m_SDCAcomboBox = new wxComboBox( itemPanel38, ID_COMBOBOX2, wxEmptyString, wxDefaultPosition, wxDefaultSize, m_SDCAcomboBoxStrings, wxCB_READONLY );
   itemFlexGridSizer40->Add(m_SDCAcomboBox, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL, 5);
 
@@ -626,7 +625,7 @@ void AddEditPropSheet::CreateControls()
 
   // Set validators
   itemTextCtrl10->SetValidator( wxGenericValidator(& m_title) );
-  itemTextCtrl13->SetValidator( wxGenericValidator(& m_user) );
+  m_UsernameCtrl->SetValidator( wxGenericValidator(& m_user) );
   itemTextCtrl26->SetValidator( wxGenericValidator(& m_url) );
   itemTextCtrl31->SetValidator( wxGenericValidator(& m_email) );
   m_noteTX->SetValidator( wxGenericValidator(& m_notes) );
@@ -654,6 +653,8 @@ void AddEditPropSheet::CreateControls()
   // Setup symbols
   m_symbols = CPasswordCharPool::GetDefaultSymbols().c_str();
   m_ownsymbols->SetValue(m_symbols);
+
+  m_ExpTimeCtrl->SetRange(1, 3650);
 }
 
 /*!
@@ -809,6 +810,53 @@ static struct {short pv; wxString name;}
   }
 }
 
+void AddEditPropSheet::UpdateExpTimes()
+{
+  // From m_item to display
+  
+  m_item.GetXTime(m_tttXTime);
+  m_item.GetXTimeInt(m_XTimeInt);
+  m_XTime = m_CurXTime = m_item.GetXTimeL().c_str();
+
+  if (m_XTime.empty())
+    m_XTime = m_CurXTime = _("Never");
+
+  wxCommandEvent dummy;
+  wxDateTime exp;
+  if (m_tttXTime == 0) { // never expires
+    m_OnRB->SetValue(false);
+    m_InRB->SetValue(false);
+    m_NeverRB->SetValue(true);
+    exp = wxDateTime::Now();
+    dummy.SetEventObject(m_NeverRB);
+  } else {
+    exp = wxDateTime(m_tttXTime);
+    if (m_XTimeInt == 0) { // expiration specified as date
+      m_OnRB->SetValue(true);
+      m_InRB->SetValue(false);
+      m_NeverRB->SetValue(false);
+      m_ExpTimeCtrl->Enable(false);
+      m_Recurring = false;
+      dummy.SetEventObject(m_OnRB);
+    } else { // exp. specified as interval
+      m_OnRB->SetValue(false);
+      m_InRB->SetValue(true);
+      m_NeverRB->SetValue(false);
+      m_ExpDate->Enable(false);
+      m_ExpTimeCtrl->SetValue(m_XTimeInt);
+      m_Recurring = true;
+      dummy.SetEventObject(m_InRB);
+    }
+    m_RecurringCtrl->Enable(m_Recurring);
+    m_ExpDate->SetValue(exp);
+  }
+
+  if (exp > wxDateTime::Today())
+    exp = wxDateTime::Today(); // otherwise we can never move exp date back
+  m_ExpDate->SetRange(exp, wxDateTime(time_t(-1)));
+
+  OnExpRadiobuttonSelected(dummy); // setup enable/disable of expiry-related controls
+}
 
 void AddEditPropSheet::ItemFieldsToPropSheet()
 {
@@ -816,7 +864,7 @@ void AddEditPropSheet::ItemFieldsToPropSheet()
   std::vector<stringT>::iterator sviter;
 
   // Populate the group combo box
-  m_core.GetUniqueGroups(svec);
+  m_core.GetAllGroups(svec);
   for (sviter = svec.begin(); sviter != svec.end(); sviter++)
     m_groupCtrl->Append(sviter->c_str());
 
@@ -914,45 +962,7 @@ void AddEditPropSheet::ItemFieldsToPropSheet()
   } // m_type
 
   // Password Expiration
-  m_XTime = m_CurXTime = m_item.GetXTimeL().c_str();
-  if (m_XTime.empty())
-    m_XTime = m_CurXTime = _("Never");
-  m_item.GetXTime(m_tttXTime);
-
-  m_item.GetXTimeInt(m_XTimeInt);
-
-  wxCommandEvent dummy;
-  wxDateTime earliestExp;
-  if (m_tttXTime == 0) { // never expires
-    m_OnRB->SetValue(false);
-    m_InRB->SetValue(false);
-    m_NeverRB->SetValue(true);
-    earliestExp = wxDateTime::Now();
-    dummy.SetEventObject(m_NeverRB);
-  } else {
-    earliestExp = wxDateTime(m_tttXTime);
-    if (m_XTimeInt == 0) { // expiration specified as date
-      m_OnRB->SetValue(true);
-      m_InRB->SetValue(false);
-      m_NeverRB->SetValue(false);
-      m_ExpTimeCtrl->Enable(false);
-      m_Recurring = false;
-      dummy.SetEventObject(m_OnRB);
-    } else { // exp. specified as interval
-      m_OnRB->SetValue(false);
-      m_InRB->SetValue(true);
-      m_NeverRB->SetValue(false);
-      m_ExpDate->Enable(false);
-      m_ExpTimeCtrl->SetValue(m_XTimeInt);
-      m_Recurring = true;
-      dummy.SetEventObject(m_InRB);
-    }
-    m_RecurringCtrl->Enable(m_Recurring);
-  }
-
-  m_ExpDate->SetRange(earliestExp, wxDateTime(time_t(-1)));
-
-  OnRadiobuttonSelected(dummy); // setup enable/disable of expiry-related controls
+  UpdateExpTimes();
   // Modification times
   m_CTime = m_item.GetCTimeL().c_str();
   m_PMTime = m_item.GetPMTimeL().c_str();
@@ -1004,7 +1014,6 @@ void AddEditPropSheet::OnGoButtonClick( wxCommandEvent& /* evt */ )
     ::wxLaunchDefaultBrowser(m_url, wxBROWSER_NEW_WINDOW);
 }
 
-
 /*!
  * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON3
  */
@@ -1020,7 +1029,7 @@ void AddEditPropSheet::OnGenerateButtonClick( wxCommandEvent& /* evt */ )
       return;
     }
 
-    PWSclip::SetData(password);
+    PWSclipboard::GetInstance()->SetData(password);
     m_password = password.c_str();
     m_PasswordCtrl->ChangeValue(m_password.c_str());
     if (m_isPWHidden) {
@@ -1029,7 +1038,6 @@ void AddEditPropSheet::OnGenerateButtonClick( wxCommandEvent& /* evt */ )
   }
 }
 
-
 /*!
  * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON2
  */
@@ -1061,6 +1069,7 @@ void AddEditPropSheet::ShowPassword()
     m_PasswordCtrl->ChangeValue(pwd);
     m_PasswordCtrl->SetModified(true);
   }
+  m_PasswordCtrl->MoveAfterInTabOrder(m_UsernameCtrl);
   m_BasicFGSizer->Replace(tmp, m_PasswordCtrl);
   delete tmp;
   m_BasicFGSizer->Layout();
@@ -1084,6 +1093,7 @@ void AddEditPropSheet::HidePassword()
                                   wxDefaultPosition, wxDefaultSize,
                                   wxTE_PASSWORD);
   ApplyPasswordFont(m_PasswordCtrl);
+  m_PasswordCtrl->MoveAfterInTabOrder(m_UsernameCtrl);
   m_BasicFGSizer->Replace(tmp, m_PasswordCtrl);
   delete tmp;
   m_BasicFGSizer->Layout();
@@ -1095,16 +1105,16 @@ void AddEditPropSheet::HidePassword()
   m_Password2Ctrl->Enable(true);
 }
 
-static void GetSelectedDCA(const wxComboBox *pcbox, short &val,
+static short GetSelectedDCA(const wxComboBox *pcbox,
                            short lastval, short defval)
 {
   int sel = pcbox->GetSelection();
   intptr_t ival = -1;
   if (sel == wxNOT_FOUND) { // no selection
-    val = (lastval == defval) ? -1 : lastval;
+    return -1;
   } else {
     ival = reinterpret_cast<intptr_t>(pcbox->GetClientData(sel));
-    val = (ival == defval) ? -1 : ival;
+    return (ival == defval) ? -1 : ival;
   }
 }
 
@@ -1144,12 +1154,12 @@ void AddEditPropSheet::OnOk(wxCommandEvent& /* evt */)
       short lastDCA, lastShiftDCA;
       const PWSprefs *prefs = PWSprefs::GetInstance();
       m_item.GetDCA(lastDCA);
-      GetSelectedDCA(m_DCAcomboBox, m_DCA, lastDCA,
-                     short(prefs->GetPref(PWSprefs::DoubleClickAction)));
+      m_DCA = GetSelectedDCA(m_DCAcomboBox, lastDCA,
+                             short(prefs->GetPref(PWSprefs::DoubleClickAction)));
 
       m_item.GetShiftDCA(lastShiftDCA);
-      GetSelectedDCA(m_SDCAcomboBox, m_ShiftDCA, lastShiftDCA,
-                     short(prefs->GetPref(PWSprefs::ShiftDoubleClickAction)));
+      m_ShiftDCA = GetSelectedDCA(m_SDCAcomboBox, lastShiftDCA,
+                                  short(prefs->GetPref(PWSprefs::ShiftDoubleClickAction)));
       // Check if modified
       int lastXTimeInt;
       m_item.GetXTimeInt(lastXTimeInt);
@@ -1204,7 +1214,7 @@ void AddEditPropSheet::OnOk(wxCommandEvent& /* evt */)
       } else {
         m_core.GetPolicyFromName(oldPolName, oldPWP);
       }
-      // now get dbox's effecive policy:
+      // now get dbox's effective policy:
       pwp = GetSelectedPWPolicy();
 
       bIsModified = (group        != m_item.GetGroup().c_str()       ||
@@ -1242,7 +1252,7 @@ void AddEditPropSheet::OnOk(wxCommandEvent& /* evt */)
         if (m_defPWPRB->GetValue()) {
           polName = m_cbxPolicyNames->GetValue();
           if (polName == _("Default Policy"))
-            polName = wxT("");
+            polName = wxEmptyString;
         } else {
           m_item.SetPWPolicy(pwp);
         }
@@ -1260,8 +1270,11 @@ void AddEditPropSheet::OnOk(wxCommandEvent& /* evt */)
         m_item.SetRMTime(t);
       if (m_tttXTime != lastXtime)
         m_item.SetXTime(m_tttXTime);
+      if (m_Recurring) {
       if (m_XTimeInt != lastXTimeInt)
         m_item.SetXTimeInt(m_XTimeInt);
+      } else
+        m_item.SetXTimeInt(0);
       // All fields in m_item now reflect user's edits
       // Let's update the core's data
       uuid_array_t uuid;
@@ -1326,7 +1339,7 @@ void AddEditPropSheet::OnOk(wxCommandEvent& /* evt */)
 #endif
       if (m_item.IsAlias()) {
         m_item.SetXTime((time_t)0);
-        m_item.SetPWPolicy(_T(""));
+        m_item.SetPWPolicy(wxEmptyString);
       } else {
         m_item.SetXTime(m_tttXTime);
       }
@@ -1342,7 +1355,6 @@ void AddEditPropSheet::OnOk(wxCommandEvent& /* evt */)
   }
 }
 
-
 /*!
  * wxEVT_COMMAND_CHECKBOX_CLICKED event handler for ID_CHECKBOX1
  */
@@ -1355,7 +1367,6 @@ void AddEditPropSheet::OnKeepHistoryClick(wxCommandEvent &)
    }
 }
 
-
 #if 0 // XXX Remove, as we did away with this checkbox!
 void AddEditPropSheet::OnOverrideDCAClick( wxCommandEvent& /* evt */ )
 {
@@ -1399,7 +1410,7 @@ void AddEditPropSheet::SetXTime(wxObject *src)
         wxString rstr;
         rstr.Printf(_(" (every %d days)"), m_XTimeInt);
         m_XTime += rstr;
-      } // recurring
+      }
     } else {
       ASSERT(0);
     }
@@ -1412,7 +1423,7 @@ void AddEditPropSheet::SetXTime(wxObject *src)
  * wxEVT_COMMAND_RADIOBUTTON_SELECTED event handler for ID_RADIOBUTTON
  */
 
-void AddEditPropSheet::OnRadiobuttonSelected( wxCommandEvent& evt )
+void AddEditPropSheet::OnExpRadiobuttonSelected( wxCommandEvent& evt )
 {
   bool On = (evt.GetEventObject() == m_OnRB);
   bool Never = (evt.GetEventObject() == m_NeverRB);
@@ -1434,7 +1445,6 @@ void AddEditPropSheet::OnRadiobuttonSelected( wxCommandEvent& evt )
   m_RecurringCtrl->Enable(!On && !Never);
 }
 
-
 /*!
  * wxEVT_COMMAND_RADIOBUTTON_SELECTED event handler for ID_RADIOBUTTON2
  */
@@ -1491,7 +1501,6 @@ void AddEditPropSheet::OnUseHexCBClick( wxCommandEvent& /* evt */ )
  }
 }
 
-
 /*!
  * wxEVT_SET_FOCUS event handler for ID_TEXTCTRL7
  */
@@ -1507,7 +1516,6 @@ void AddEditPropSheet::OnNoteSetFocus( wxFocusEvent& /* evt */ )
 
 PWPolicy AddEditPropSheet::GetPWPolicyFromUI()
 {
-  Validate(); TransferDataFromWindow();
   wxASSERT_MSG(m_ourPWPRB->GetValue() && !m_defPWPRB->GetValue(), wxT("Trying to get Password policy from UI when db defaults are to be used"));
 
   PWPolicy pwp;
@@ -1602,7 +1610,6 @@ void AddEditPropSheet::OnClearPWHist(wxCommandEvent& /*evt*/)
     m_PWHistory.Empty();
 }
 
-
 /*!
  * wxEVT_COMMAND_CHECKBOX_CLICKED event handler for ID_CHECKBOX7
  */
@@ -1614,7 +1621,7 @@ void AddEditPropSheet::OnEZreadCBClick(wxCommandEvent& evt)
     // Check if pronounceable is also set - forbid both
     if (m_pwpPronounceCtrl->GetValue()) {
       m_pwpEasyCtrl->SetValue(false);
-      wxMessageBox(_("Sorry, \"easy-to-read\" and \"pronouncable\" cannot be both selected"),
+      wxMessageBox(_("Sorry, \"easy-to-read\" and \"pronounceable\" cannot be both selected"),
                    _("Error"), wxOK|wxICON_ERROR, this);
       return;
     }
@@ -1630,7 +1637,6 @@ void AddEditPropSheet::OnEZreadCBClick(wxCommandEvent& evt)
   m_ownsymbols->SetValue(m_symbols);
 }
 
-
 /*!
  * wxEVT_COMMAND_CHECKBOX_CLICKED event handler for ID_CHECKBOX8
  */
@@ -1642,7 +1648,7 @@ void AddEditPropSheet::OnPronouceableCBClick( wxCommandEvent& evt)
     // Check if ezread is also set - forbid both
     if (m_pwpEasyCtrl->GetValue()) {
       m_pwpPronounceCtrl->SetValue(false);
-      wxMessageBox(_("Sorry, \"pronouncable\" and \"easy-to-read\" cannot be both selected"),
+      wxMessageBox(_("Sorry, \"pronounceable\" and \"easy-to-read\" cannot be both selected"),
                    _("Error"), wxOK|wxICON_ERROR, this);
       return;
     }
@@ -1697,11 +1703,10 @@ void AddEditPropSheet::OnSendButtonClick( wxCommandEvent& event )
     StringX mail_cmd= tostringx(_("mailto:"));
     mail_cmd += tostringx(m_email);
     PWSRun runner;
-    runner.issuecmd(mail_cmd, wxT(""), false);
+    runner.issuecmd(mail_cmd, wxEmptyString, false);
   }
 }
 
-
 /*!
  * wxEVT_COMMAND_COMBOBOX_SELECTED event handler for ID_POLICYLIST
  */
@@ -1723,7 +1728,6 @@ void AddEditPropSheet::OnPolicylistSelected( wxCommandEvent& event )
   EnablePWPolicyControls(false);
 }
 
-
 /*!
  * wxEVT_DATE_CHANGED event handler for ID_DATECTRL
  */
@@ -1733,7 +1737,6 @@ void AddEditPropSheet::OnExpDateChanged( wxDateEvent& event )
   SetXTime(event.GetEventObject());
 }
 
-
 /*!
  * wxEVT_COMMAND_SPINCTRL_UPDATED event handler for ID_SPINCTRL2
  */
@@ -1743,7 +1746,6 @@ void AddEditPropSheet::OnExpIntervalChanged( wxSpinEvent& event )
   SetXTime(event.GetEventObject());
 }
 
-
 /*!
  * wxEVT_COMMAND_CHECKBOX_CLICKED event handler for ID_CHECKBOX6
  */
@@ -1756,7 +1758,6 @@ void AddEditPropSheet::OnSymbolsCB( wxCommandEvent& event )
   FindWindow(ID_RESET_SYMBOLS)->Enable(checked);
 }
 
-
 /*!
  * wxEVT_SET_FOCUS event handler for IDC_OWNSYMBOLS
  */
@@ -1769,7 +1770,6 @@ void AddEditPropSheet::OnOwnSymSetFocus( wxFocusEvent& event )
 ////@end wxEVT_SET_FOCUS event handler for IDC_OWNSYMBOLS in AddEditPropSheet.
 }
 
-
 /*!
  * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_RESET_SYMBOLS
  */
@@ -1787,7 +1787,6 @@ void AddEditPropSheet::OnResetSymbolsClick( wxCommandEvent& WXUNUSED(event) )
   m_ownsymbols->SetValue(m_symbols);
 }
 
-
 /*!
  * wxEVT_COMMAND_CHECKBOX_CLICKED event handler for ID_CHECKBOX5
  */
@@ -1797,7 +1796,6 @@ void AddEditPropSheet::OnDigitsCB( wxCommandEvent& event )
   m_pwpDigSpin->Enable(event.IsChecked());
 }
 
-
 /*!
  * wxEVT_COMMAND_CHECKBOX_CLICKED event handler for ID_CHECKBOX4
  */
@@ -1807,7 +1805,6 @@ void AddEditPropSheet::OnUppercaseCB( wxCommandEvent& event )
   m_pwpUCSpin->Enable(event.IsChecked());
 }
 
-
 /*!
  * wxEVT_COMMAND_CHECKBOX_CLICKED event handler for ID_CHECKBOX3
  */
@@ -1816,4 +1813,3 @@ void AddEditPropSheet::OnLowercaseCB( wxCommandEvent& event )
 {
   m_pwpLCSpin->Enable(event.IsChecked());
 }
-
