@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2015 Rony Shapiro <ronys@users.sourceforge.net>.
+ * Copyright (c) 2003-2016 Rony Shapiro <ronys@pwsafe.org>.
  * All rights reserved. Use of the code is allowed under the
  * Artistic License 2.0 terms, as specified in the LICENSE file
  * distributed with this code, or available from
@@ -34,14 +34,12 @@
 ////@begin XPM images
 ////@end XPM images
 
-
 /*!
  * CreateShortcutDlg type definition
  */
 
 IMPLEMENT_CLASS( CreateShortcutDlg, wxDialog )
 
-
 /*!
  * CreateShortcutDlg event table definition
  */
@@ -55,7 +53,6 @@ BEGIN_EVENT_TABLE( CreateShortcutDlg, wxDialog )
 
 END_EVENT_TABLE()
 
-
 /*!
  * CreateShortcutDlg constructors
  */
@@ -73,7 +70,6 @@ CreateShortcutDlg::CreateShortcutDlg(wxWindow* parent, PWScore &core,
   Create(parent, id, caption, pos, size, style);
 }
 
-
 /*!
  * CreateShortcutDlg creator
  */
@@ -99,7 +95,7 @@ void CreateShortcutDlg::ItemFieldsToDialog()
 {
   // Populate the combo box
   std::vector<stringT> aryGroups;
-  m_core.GetUniqueGroups(aryGroups);
+  m_core.GetAllGroups(aryGroups);
   for (size_t igrp = 0; igrp < aryGroups.size(); igrp++) {
     m_groupCtrl->Append(aryGroups[igrp].c_str());
   }
@@ -108,7 +104,6 @@ void CreateShortcutDlg::ItemFieldsToDialog()
   // we're invoked via right-click n a node
 }
 
-
 /*!
  * CreateShortcutDlg destructor
  */
@@ -119,7 +114,6 @@ CreateShortcutDlg::~CreateShortcutDlg()
 ////@end CreateShortcutDlg destruction
 }
 
-
 /*!
  * Member initialisation
  */
@@ -131,7 +125,6 @@ void CreateShortcutDlg::Init()
 ////@end CreateShortcutDlg member initialisation
 }
 
-
 /*!
  * Control creation for CreateShortcutDlg
  */
@@ -190,7 +183,6 @@ void CreateShortcutDlg::CreateControls()
 ////@end CreateShortcutDlg content construction
 }
 
-
 /*!
  * Should we show tooltips?
  */
@@ -224,7 +216,6 @@ wxIcon CreateShortcutDlg::GetIconResource( const wxString& WXUNUSED(name) )
 ////@end CreateShortcutDlg icon retrieval
 }
 
-
 /*!
  * wxEVT_COMMAND_BUTTON_CLICKED event handler for wxID_OK
  */
@@ -238,8 +229,8 @@ void CreateShortcutDlg::OnOkClick( wxCommandEvent& /* evt */ )
       return;
 
     CItemData shortcut;
-    shortcut.CreateUUID();
     shortcut.SetShortcut();
+    shortcut.CreateUUID();
     shortcut.SetPassword(wxT("[Shortcut]"));
     const wxString group = m_groupCtrl->GetValue();
 
@@ -254,10 +245,8 @@ void CreateShortcutDlg::OnOkClick( wxCommandEvent& /* evt */ )
     shortcut.SetXTime(time_t(0));
     shortcut.SetStatus(CItemData::ES_ADDED);
 
-    uuid_array_t base_uuid;
-    m_base->GetUUID(base_uuid);
-    m_core.Execute(AddEntryCommand::Create(&m_core, shortcut, base_uuid));
+    m_core.Execute(AddEntryCommand::Create(&m_core, shortcut,
+					   m_base->GetUUID()));
   }
   EndModal(wxID_OK);
 }
-
