@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2015 Rony Shapiro <ronys@users.sourceforge.net>.
+ * Copyright (c) 2003-2016 Rony Shapiro <ronys@pwsafe.org>.
  * All rights reserved. Use of the code is allowed under the
  * Artistic License 2.0 terms, as specified in the LICENSE file
  * distributed with this code, or available from
@@ -54,7 +54,6 @@
 
 IMPLEMENT_DYNAMIC_CLASS( COptions, wxPropertySheetDialog )
 
-
 /*!
  * COptions event table definition
  */
@@ -89,25 +88,25 @@ EVT_BUTTON( wxID_OK, COptions::OnOk )
 END_EVENT_TABLE()
 
 const wxString BUSuffix[] = {
-  L"None",
-  L"YYYYMMMDD_HHMMSS",
-  L"Incremented Number [001-999]",
+  _("None"),
+  _("YYYYMMMDD_HHMMSS"),
+  _("Incremented Number [001-999]"),
 };
 
 enum {NO_SFX, TS_SFX, INC_SFX}; // For backup file suffix name
 
 // Following in enum order (see PWSprefs.h)
 const wxString DCAStrings[] = {
-  L"Copy password to clipboard",
-  L"Edit/View selected entry",
-  L"Autotype",
-  L"Browse to URL",
-  L"Copy notes to clipboard",
-  L"Copy username to clipboard",
-  L"Copy password to clipboard, minimize",
-  L"Browse to URL + Autotype",
-  L"Run Command",
-  L"Send email",
+  _("Copy password to clipboard"),
+  _("Edit/View selected entry"),
+  _("Autotype"),
+  _("Browse to URL"),
+  _("Copy notes to clipboard"),
+  _("Copy username to clipboard"),
+  _("Copy password to clipboard, minimize"),
+  _("Browse to URL + Autotype"),
+  _("Run Command"),
+  _("Send email"),
 };
 
 /*!
@@ -125,7 +124,6 @@ COptions::COptions( wxWindow* parent, wxWindowID id, const wxString& caption, co
   Create(parent, id, caption, pos, size, style);
 }
 
-
 /*!
  * COptions creator
  */
@@ -152,7 +150,6 @@ bool COptions::Create( wxWindow* parent, wxWindowID id, const wxString& caption,
   return true;
 }
 
-
 /*!
  * COptions destructor
  */
@@ -163,7 +160,6 @@ COptions::~COptions()
 ////@end COptions destruction
 }
 
-
 /*!
  * Member initialisation
  */
@@ -194,12 +190,12 @@ void COptions::Init()
   m_seclockonidleCB = NULL;
   m_secidletimeoutSB = NULL;
   m_sysusesystrayCB = NULL;
+  m_systrayclosediconcolourRB = NULL;
   m_sysmaxREitemsSB = NULL;
   m_systrayWarning = NULL;
 ////@end COptions member initialisation
 }
 
-
 /*!
  * Control creation for COptions
  */
@@ -211,7 +207,7 @@ void COptions::CreateControls()
   wxBoxSizer* itemBoxSizer3 = new wxBoxSizer(wxVERTICAL);
   itemPanel2->SetSizer(itemBoxSizer3);
 
-  wxCheckBox* itemCheckBox4 = new wxCheckBox( itemPanel2, ID_CHECKBOX10, _("Save database immediately after Edit or Add"), wxDefaultPosition, wxDefaultSize, 0 );
+  wxCheckBox* itemCheckBox4 = new wxCheckBox( itemPanel2, ID_CHECKBOX10, _("Save database immediately after any change"), wxDefaultPosition, wxDefaultSize, 0 );
   itemCheckBox4->SetValue(false);
   itemBoxSizer3->Add(itemCheckBox4, 0, wxALIGN_LEFT|wxALL, 5);
 
@@ -251,7 +247,7 @@ void COptions::CreateControls()
   itemStaticBoxSizer7->Add(itemBoxSizer15, 0, wxGROW|wxALL, 0);
   wxArrayString m_busuffixCBStrings;
   for (int i = 0; i < int(sizeof(BUSuffix)/sizeof(BUSuffix[0])); ++i) {
-    m_busuffixCBStrings.Add(_(BUSuffix[i]));
+    m_busuffixCBStrings.Add(BUSuffix[i]);
   }
   m_busuffixCB = new wxComboBox( itemPanel2, ID_COMBOBOX2, wxEmptyString, wxDefaultPosition, wxSize(itemPanel2->ConvertDialogToPixels(wxSize(140, -1)).x, -1), m_busuffixCBStrings, wxCB_READONLY );
   itemBoxSizer15->Add(m_busuffixCB, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);
@@ -379,12 +375,18 @@ void COptions::CreateControls()
   wxArrayString m_DCACBStrings;
   wxArrayString m_SDCACBStrings;
   for (int i = 0; i < int(sizeof(DCAStrings)/sizeof(DCAStrings[0])); ++i) {
-    wxString tmp = _(DCAStrings[i]);
+    wxString tmp = DCAStrings[i];
     m_DCACBStrings.Add(tmp);
     m_SDCACBStrings.Add(tmp);
   }
 
-  m_DCACB = new wxComboBox( itemPanel44, ID_COMBOBOX3, wxEmptyString, wxDefaultPosition, wxDefaultSize, m_DCACBStrings, wxCB_READONLY|wxCB_SORT );
+  // This is to avoid a nasty assert on OSX with wx3.0.2
+  auto cbStyle = wxCB_READONLY;
+#ifndef  __WXMAC__
+  cbStyle |= wxCB_SORT;
+#endif
+
+  m_DCACB = new wxComboBox( itemPanel44, ID_COMBOBOX3, wxEmptyString, wxDefaultPosition, wxDefaultSize, m_DCACBStrings, cbStyle );
   itemBoxSizer49->Add(m_DCACB, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);
 
   wxBoxSizer* itemBoxSizer53 = new wxBoxSizer(wxHORIZONTAL);
@@ -392,7 +394,7 @@ void COptions::CreateControls()
   wxStaticText* itemStaticText54 = new wxStaticText( itemPanel44, wxID_STATIC, _("Shift double-click action"), wxDefaultPosition, wxDefaultSize, 0 );
   itemBoxSizer53->Add(itemStaticText54, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);
 
-  m_SDCACB = new wxComboBox( itemPanel44, ID_COMBOBOX, wxEmptyString, wxDefaultPosition, wxDefaultSize, m_SDCACBStrings, wxCB_READONLY|wxCB_SORT );
+  m_SDCACB = new wxComboBox( itemPanel44, ID_COMBOBOX, wxEmptyString, wxDefaultPosition, wxDefaultSize, m_SDCACBStrings, cbStyle );
   itemBoxSizer53->Add(m_SDCACB, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);
 
   wxStaticBox* itemStaticBoxSizer56Static = new wxStaticBox(itemPanel44, wxID_ANY, _("Autotype"));
@@ -618,6 +620,15 @@ void COptions::CreateControls()
   itemBoxSizer105->Add(itemCheckBox122, 0, wxALIGN_LEFT|wxALL, 5);
 #endif
 
+  wxArrayString itemRadioBox125Strings;
+  itemRadioBox125Strings.Add(_("Black"));
+  itemRadioBox125Strings.Add(_("Cyan"));
+  itemRadioBox125Strings.Add(_("White"));
+  itemRadioBox125Strings.Add(_("Yellow"));
+  m_systrayclosediconcolourRB = new wxRadioBox( itemPanel104, ID_RADIOBOX, _("Initial tray icon color"), wxDefaultPosition, wxDefaultSize, itemRadioBox125Strings, 1, wxRA_SPECIFY_ROWS );
+  m_systrayclosediconcolourRB->SetSelection(0);
+  itemBoxSizer105->Add(m_systrayclosediconcolourRB, 0, wxGROW|wxALL, 5);
+
   GetBookCtrl()->AddPage(itemPanel104, _("System"));
 
   wxPanel* itemPanel123 = new wxPanel( GetBookCtrl(), ID_PANEL7, wxDefaultPosition, wxDefaultSize, wxSUNKEN_BORDER|wxTAB_TRAVERSAL );
@@ -672,7 +683,6 @@ void COptions::CreateControls()
 ////@end COptions content construction
 }
 
-
 /*!
  * Should we show tooltips?
  */
@@ -750,12 +760,12 @@ void COptions::PrefsToPropSheet()
   if (m_doubleclickaction < 0 ||
       m_doubleclickaction >= int(sizeof(DCAStrings)/sizeof(DCAStrings[0])))
     m_doubleclickaction = 0;
-  m_DCACB->SetValue(_(DCAStrings[m_doubleclickaction]));
+  m_DCACB->SetValue(DCAStrings[m_doubleclickaction]);
   m_shiftdoubleclickaction = prefs->GetPref(PWSprefs::ShiftDoubleClickAction);
   if (m_shiftdoubleclickaction < 0 ||
       m_shiftdoubleclickaction >= int(sizeof(DCAStrings)/sizeof(DCAStrings[0])))
     m_shiftdoubleclickaction = 0;
-  m_SDCACB->SetValue(_(DCAStrings[m_shiftdoubleclickaction]));
+  m_SDCACB->SetValue(DCAStrings[m_shiftdoubleclickaction]);
   m_minauto = prefs->GetPref(PWSprefs::MinimizeOnAutotype);
   m_autotypeStr = prefs->GetPref(PWSprefs::DefaultAutotypeString).c_str();
   if (m_autotypeStr.empty())
@@ -792,6 +802,7 @@ void COptions::PrefsToPropSheet()
   // System preferences
   m_sysmaxREitemsSB->SetValue(prefs->GetPref(PWSprefs::MaxREItems));
   m_sysusesystrayCB->SetValue(prefs->GetPref(PWSprefs::UseSystemTray));
+  m_systrayclosediconcolourRB->SetSelection(prefs->GetPref(PWSprefs::ClosedTrayIconColour));
   if (!wxTaskBarIcon::IsAvailable()) {
     m_systrayWarning->Show();
     Layout();
@@ -810,7 +821,7 @@ void COptions::PrefsToPropSheet()
 static int DCAStr2Int(const wxString &str)
 {
   for (int i = 0; i < int(sizeof(DCAStrings)/sizeof(DCAStrings[0])); ++i)
-    if (str == _(DCAStrings[i])) {
+    if (str == DCAStrings[i]) {
       return i;
     }
   ASSERT(0);
@@ -886,6 +897,7 @@ void COptions::PropSheetToPrefs()
   // System preferences
   prefs->SetPref(PWSprefs::MaxREItems, m_sysmaxREitemsSB->GetValue());
   prefs->SetPref(PWSprefs::UseSystemTray, m_sysusesystrayCB->GetValue());
+  prefs->SetPref(PWSprefs::ClosedTrayIconColour, m_systrayclosediconcolourRB->GetSelection());
   m_sysstartup = false; // XXX TBD
   prefs->SetPref(PWSprefs::MaxMRUItems, m_sysmaxmru);
   prefs->SetPref(PWSprefs::MRUOnFileMenu, m_sysmruonfilemenu);
@@ -919,7 +931,7 @@ void COptions::PropSheetToPrefs()
   prefs->SetPref(PWSprefs::DefaultUsername, tostringx(m_defusernameTXT->GetValue()), true);
 
   if (m_autotypeStr.empty() || m_autotypeStr == DEFAULT_AUTOTYPE)
-      prefs->SetPref(PWSprefs::DefaultAutotypeString, wxT(""), true);
+      prefs->SetPref(PWSprefs::DefaultAutotypeString, wxEmptyString, true);
   else
     prefs->SetPref(PWSprefs::DefaultAutotypeString, tostringx(m_autotypeStr), true);
 
@@ -939,7 +951,6 @@ void COptions::OnOk(wxCommandEvent& /* evt */)
   }
 }
 
-
 /*!
  * wxEVT_COMMAND_CHECKBOX_CLICKED event handler for ID_CHECKBOX11
  */
@@ -955,7 +966,6 @@ void COptions::OnBackupB4SaveClick( wxCommandEvent& /* evt */ )
   }
 }
 
-
 /*!
  * wxEVT_COMMAND_RADIOBUTTON_SELECTED event handler for ID_RADIOBUTTON4
  */
@@ -965,7 +975,6 @@ void COptions::OnBuPrefix( wxCommandEvent& evt )
   evt.Skip();
 }
 
-
 /*!
  * wxEVT_SET_FOCUS event handler for ID_TEXTCTRL9
  */
@@ -976,7 +985,6 @@ void COptions::OnBuPrefixTxtSetFocus( wxFocusEvent& /* evt */ )
   m_usrbuprefixRB->SetValue(true);
 }
 
-
 /*!
  * wxEVT_COMMAND_COMBOBOX_SELECTED event handler for ID_COMBOBOX2
  */
@@ -1018,7 +1026,6 @@ void COptions::OnSuffixCBSet( wxCommandEvent& /* evt */ )
   m_suffixExample->SetLabel(example);
 }
 
-
 /*!
  * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON
  */
@@ -1031,7 +1038,6 @@ void COptions::OnBuDirBrowseClick( wxCommandEvent& /* evt */ )
     m_usrbudirTxt->SetValue(dirdlg.GetPath());
 }
 
-
 /*!
  * wxEVT_COMMAND_RADIOBUTTON_SELECTED event handler for ID_RADIOBUTTON6
  */
@@ -1043,8 +1049,6 @@ void COptions::OnBuDirRB( wxCommandEvent& /* evt */ )
     m_buDirBN->Enable(enable);
 }
 
-
-
 /*!
  * wxEVT_COMMAND_CHECKBOX_CLICKED event handler for ID_CHECKBOX13
  */
@@ -1058,7 +1062,6 @@ void COptions::OnShowUsernameInTreeCB( wxCommandEvent& /* evt */ )
   }
 }
 
-
 /*!
  * wxEVT_COMMAND_CHECKBOX_CLICKED event handler for ID_CHECKBOX19
  */
@@ -1070,7 +1073,6 @@ void COptions::OnPreExpiryWarnClick( wxCommandEvent& /* evt */ )
   }
 }
 
-
 /*!
  * wxEVT_COMMAND_CHECKBOX_CLICKED event handler for ID_CHECKBOX24
  */
@@ -1083,7 +1085,6 @@ void COptions::OnUseDefaultUserClick( wxCommandEvent& /* evt */ )
   }
 }
 
-
 /*!
  * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON8
  */
@@ -1109,7 +1110,6 @@ void COptions::OnPWHistSaveClick( wxCommandEvent& /* evt */ )
   m_pwhistnumdfltSB->Enable(m_pwhistsaveCB->GetValue());
 }
 
-
 /*!
  * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_PWHISTNOCHANGE
  */
@@ -1121,7 +1121,6 @@ void COptions::OnPWHistApply( wxCommandEvent& evt )
   evt.Skip();
 }
 
-
 /*!
  * wxEVT_COMMAND_RADIOBUTTON_SELECTED event handler for ID_RADIOBUTTON8
  */
@@ -1132,7 +1131,6 @@ void COptions::OnPWHistRB( wxCommandEvent& evt )
   m_pwhistapplyBN->Enable(id != ID_PWHISTNOCHANGE);
 }
 
-
 /*!
  * wxEVT_COMMAND_CHECKBOX_CLICKED event handler for ID_CHECKBOX29
  */
@@ -1142,7 +1140,6 @@ void COptions::OnLockOnIdleClick( wxCommandEvent& /* evt */)
   m_secidletimeoutSB->Enable(m_seclockonidleCB->GetValue());
 }
 
-
 /*!
  * wxEVT_COMMAND_CHECKBOX_CLICKED event handler for ID_CHECKBOX30
  */
@@ -1150,6 +1147,7 @@ void COptions::OnLockOnIdleClick( wxCommandEvent& /* evt */)
 void COptions::OnUseSystrayClick( wxCommandEvent& /* evt */)
 {
   m_sysmaxREitemsSB->Enable(m_sysusesystrayCB->GetValue());
+  m_systrayclosediconcolourRB->Enable(m_sysusesystrayCB->GetValue());
 }
 
 void COptions::OnPageChanging(wxBookCtrlEvent& evt)
@@ -1164,4 +1162,3 @@ void COptions::OnPageChanging(wxBookCtrlEvent& evt)
       evt.Veto();
   }
 }
-
